<?php if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/**
 * Installation script for blocker
 *
 * @author No-CMS Module Generator
 */
class Info extends CMS_Module {

    /////////////////////////////////////////////////////////////////////////////
    // Default Functions
    /////////////////////////////////////////////////////////////////////////////

    // ACTIVATION
    protected function do_activate(){
        $this->remove_all();
        $this->build_all();
    }

    // DEACTIVATION
    protected function do_deactivate(){
        $this->backup_database(array(
            $this->t('ip'),
            $this->t('label'),
            $this->t('leech'),
            $this->t('useragent')
        ));
        $this->remove_all();
    }

    // UPGRADE
    protected function do_upgrade($old_version){
        // Add your migration logic here.
    }

    // OVERRIDE THIS FUNCTION TO PROVIDE "Module Setting" FEATURE
    public function setting(){
        $module_directory = $this->cms_module_path();
        $data = array();
        $data['IS_ACTIVE'] = $this->IS_ACTIVE;
        $data['module_directory'] = $module_directory;
        if(!$this->IS_ACTIVE){
            // get setting
            $module_table_prefix = $this->input->post('module_table_prefix');
            $module_prefix       = $this->input->post('module_prefix');
            // set values
            if(isset($module_table_prefix) && $module_table_prefix !== FALSE){
                cms_module_config($module_directory, 'module_table_prefix', $module_table_prefix);
            }
            if(isset($module_prefix) && $module_prefix !== FALSE){
                cms_module_prefix($module_directory, $module_prefix);
            }
            // get values
            $data['module_table_prefix'] = cms_module_config($module_directory, 'module_table_prefix');
            $data['module_prefix']       = cms_module_prefix($module_directory);
        }
        $this->view($module_directory.'/install_setting', $data, 'main_module_management');
    }

    /////////////////////////////////////////////////////////////////////////////
    // Private Functions
    /////////////////////////////////////////////////////////////////////////////

    // REMOVE ALL NAVIGATIONS, WIDGETS, AND PRIVILEGES
    private function remove_all(){
        $module_path = $this->cms_module_path();

        // remove navigations
        $this->cms_remove_navigation($this->n('manage_label'));


        // remove parent of all navigations
        $this->cms_remove_navigation($this->n('index'));

        // drop tables
        $this->dbforge->drop_table($this->t('useragent'), TRUE);
        $this->dbforge->drop_table($this->t('leech'), TRUE);
        $this->dbforge->drop_table($this->t('label'), TRUE);
        $this->dbforge->drop_table($this->t('ip'), TRUE);
    }

    // CREATE ALL NAVIGATIONS, WIDGETS, AND PRIVILEGES
    private function build_all(){
        $module_path = $this->cms_module_path();

        // parent of all navigations
        $this->cms_add_navigation($this->n('index'), 'Blocker',
            $module_path.'/blocker', PRIV_AUTHORIZED);

        // add navigations
        $this->cms_add_navigation($this->n('manage_label'), 'Manage Label',
            $module_path.'/manage_label', PRIV_AUTHORIZED, $this->n('index')
        );


        // create tables
        // ip
        $fields = array(
            'id'=> $this->TYPE_INT_UNSIGNED_AUTO_INCREMENT,
            'content'=> array("type"=>'text', "null"=>TRUE),
            'id_label'=> array("type"=>'int', "constraint"=>10, "null"=>TRUE)
        );
        $this->dbforge->add_field($fields);
        $this->dbforge->add_key('id', TRUE);
        $this->dbforge->create_table($this->t('ip'));

        // label
        $fields = array(
            'id'=> $this->TYPE_INT_UNSIGNED_AUTO_INCREMENT,
            'name'=> array("type"=>'varchar', "constraint"=>45, "null"=>TRUE),
            'status'=> array("type"=>'enum("not set","black list","white list")', "null"=>TRUE)
        );
        $this->dbforge->add_field($fields);
        $this->dbforge->add_key('id', TRUE);
        $this->dbforge->create_table($this->t('label'));

        // leech
        $fields = array(
            'id'=> $this->TYPE_INT_UNSIGNED_AUTO_INCREMENT,
            'content'=> array("type"=>'text', "null"=>TRUE),
            'id_label'=> array("type"=>'int', "constraint"=>10, "null"=>TRUE)
        );
        $this->dbforge->add_field($fields);
        $this->dbforge->add_key('id', TRUE);
        $this->dbforge->create_table($this->t('leech'));

        // useragent
        $fields = array(
            'id'=> $this->TYPE_INT_UNSIGNED_AUTO_INCREMENT,
            'content'=> array("type"=>'text', "null"=>TRUE),
            'id_label'=> array("type"=>'int', "constraint"=>10, "null"=>TRUE)
        );
        $this->dbforge->add_field($fields);
        $this->dbforge->add_key('id', TRUE);
        $this->dbforge->create_table($this->t('useragent'));

        // LABELS
        $data = array('name'=>'All IP Adresses', 'status'=>'not set');                       // 1
        $this->db->insert($this->t('label'), $data);
        $data = array('name'=>'Localhost and Developer IP Adresses', 'status'=>'not set');   // 2
        $this->db->insert($this->t('label'), $data);
        $data = array('name'=>'Bandwidth Leechs (Hotlinkers)', 'status'=>'not set');         // 3
        $this->db->insert($this->t('label'), $data);
        $data = array('name'=>'Bad Bots', 'status'=>'black list');                           // 4
        $this->db->insert($this->t('label'), $data);
        $data = array('name'=>'China IP Adresses', 'status'=>'not set');                     // 5
        $this->db->insert($this->t('label'), $data);
        $data = array('name'=>'Niger IP Adresses', 'status'=>'not set');                     // 6
        $this->db->insert($this->t('label'), $data);
        $data = array('name'=>'Ukraine IP Adresses', 'status'=>'not set');                   // 7
        $this->db->insert($this->t('label'), $data);

        // IP ADDRESSES
        $data = array('id_label'=>1, 'content'=>'all');
        $this->db->insert($this->t('ip'), $data);
        $data = array('id_label'=>2, 'content'=>'127.0.0.1');
        $this->db->insert($this->t('ip'), $data);
        $data = array('id_label'=>5, 'content'=>'1.0.1.0/24 1.0.2.0/23 1.0.8.0/21 1.0.32.0/19 1.1.0.0/24 1.1.2.0/23 1.1.4.0/22 1.1.8.0/21 1.1.16.0/20 1.1.32.0/19 1.2.0.0/23 1.2.2.0/24 1.2.4.0/22 1.2.8.0/21 1.2.16.0/20 1.2.32.0/19 1.2.64.0/18 1.3.0.0/16 1.4.1.0/24 1.4.2.0/23 1.4.4.0/22 1.4.8.0/21 1.4.16.0/20 1.4.32.0/19 1.4.64.0/18 1.8.0.0/16 1.10.0.0/21 1.10.8.0/23 1.10.11.0/24 1.10.12.0/22 1.10.16.0/20 1.10.32.0/19 1.10.64.0/18 1.12.0.0/14 1.24.0.0/13 1.45.0.0/16 1.48.0.0/14 1.56.0.0/13 1.68.0.0/14 1.80.0.0/12 1.116.0.0/14 1.180.0.0/14 1.184.0.0/15 1.188.0.0/14 1.192.0.0/13 1.202.0.0/15 1.204.0.0/14 4.78.174.2/32 5.63.25.0/24 14.0.0.0/21 14.0.12.0/22 14.1.0.0/22 14.16.0.0/12 14.102.128.0/22 14.102.156.0/22 14.103.0.0/16 14.104.0.0/13 14.112.0.0/12 14.130.0.0/15 14.134.0.0/15 14.144.0.0/12 14.192.60.0/22 14.192.76.0/22 14.196.0.0/15 14.204.0.0/15 14.208.0.0/12 23.224.0.0/22 23.224.4.0/24 23.224.5.0/25 23.224.5.128/28 23.224.195.0/24 23.224.196.0/27 23.224.196.32/28 23.224.196.48/29 23.224.196.64/26 23.224.196.128/25 23.224.197.0/24 23.224.198.0/24 23.224.199.0/28 23.228.94.128/27 23.228.112.192/26 23.234.64.224/29 23.234.65.128/28 23.234.65.240/28 23.234.66.0/27 23.234.68.0/28 23.234.68.128/28 23.234.68.168/29 23.234.68.184/29 23.234.68.248/29 23.234.69.96/28 23.234.69.144/28 23.234.69.160/28 23.234.69.240/28 23.234.70.40/29 23.234.70.104/29 23.234.70.208/28 23.234.71.48/28 23.234.71.128/28 23.234.71.184/29 23.234.72.80/28 23.234.72.200/29 23.234.73.64/29 23.234.73.160/28 23.234.74.32/28 23.234.74.160/28 23.234.74.184/29 23.234.75.32/27 23.234.75.80/28 23.234.75.104/29 23.234.75.112/29 23.234.75.160/27 23.234.75.192/27 23.234.76.72/29 23.234.76.192/29 23.234.77.128/27 23.234.78.32/29 23.234.78.80/29 23.234.78.224/28 23.234.79.88/29 23.234.79.128/28 23.234.79.216/29 23.234.79.232/29 23.234.80.8/29 23.234.80.16/29 23.234.81.160/28 23.234.81.192/29 23.234.82.128/29 23.234.83.32/27 23.234.84.0/28 23.234.84.80/28 23.234.84.240/28 23.234.85.16/29 23.234.85.144/28 23.234.85.160/27 23.234.85.232/29 23.234.86.72/29 23.234.86.160/27 23.234.87.96/28 23.234.87.168/29 23.234.87.192/28 23.234.87.224/29 23.234.88.176/28 23.234.88.192/28 23.234.88.248/29 23.234.89.0/27 23.234.89.248/29 23.234.90.176/28 23.234.91.16/29 23.234.91.64/27 23.234.92.32/27 23.234.92.64/27 23.234.92.144/28 23.234.92.176/28 23.234.93.0/28 23.234.93.32/28 23.234.94.144/28 23.234.94.224/28 23.234.95.208/28 23.234.96.80/29 23.234.96.144/28 23.234.97.80/29 23.234.97.144/29 23.234.97.208/28 23.234.98.48/28 23.234.98.152/29 23.234.98.168/29 23.234.98.176/28 23.234.98.224/27 23.234.100.16/28 23.234.100.112/29 23.234.100.160/27 23.234.100.208/29 23.234.101.216/29 23.234.102.8/29 23.234.102.16/28 23.234.102.32/28 23.234.102.136/29 23.234.103.56/29 23.234.103.160/28 23.234.104.0/29 23.234.104.64/27 23.234.104.224/28 23.234.105.88/29 23.234.105.224/27 23.234.106.80/28 23.234.106.160/27 23.234.107.88/29 23.234.107.128/27 23.234.107.224/29 23.234.107.248/29 23.234.108.64/27 23.234.108.96/28 23.234.109.64/28 23.234.109.152/29 23.234.109.240/29 23.234.110.32/28 23.234.110.224/28 23.234.111.48/28 23.234.111.160/27 23.234.111.224/27 23.234.112.0/26 23.234.112.224/27 23.234.113.0/28 23.234.113.64/27 23.234.113.160/27 23.234.113.192/26 23.234.115.160/29 23.234.115.240/28 23.234.116.16/28 23.234.117.96/28 23.234.117.176/28 23.234.118.208/28 23.235.144.0/21 23.235.152.0/23 23.235.154.0/24 23.235.155.0/25 23.235.155.128/26 23.235.155.192/28 23.235.155.240/28 23.236.121.64/28 23.248.168.64/26 23.248.168.208/29 23.251.32.0/23 23.251.34.0/26 23.251.34.64/27 23.251.34.104/29 23.251.34.112/28 23.251.34.128/25 23.251.35.0/24 23.251.36.0/29 23.251.36.16/28 23.251.36.32/27 23.251.36.64/26 23.251.36.128/25 23.251.37.0/24 23.251.38.0/23 23.251.40.0/26 23.251.40.72/29 23.251.40.80/28 23.251.40.96/27 23.251.40.128/25 23.251.41.8/29 23.251.41.16/28 23.251.41.32/27 23.251.41.64/26 23.251.41.128/28 23.251.41.152/29 23.251.41.160/27 23.251.41.192/26 23.251.42.0/23 23.251.44.0/22 23.251.48.0/20 27.8.0.0/13 27.16.0.0/12 27.34.232.0/21 27.36.0.0/14 27.40.0.0/13 27.50.40.0/21 27.50.128.0/17 27.54.72.0/21 27.54.152.0/21 27.54.192.0/18 27.98.208.0/20 27.98.224.0/19 27.99.128.0/17 27.103.0.0/16 27.106.128.0/18 27.106.204.0/22 27.109.32.0/19 27.111.221.176/28 27.112.0.0/18 27.112.80.0/20 27.113.128.0/18 27.115.0.0/17 27.116.44.0/22 27.121.72.0/21 27.121.120.0/21 27.128.0.0/15 27.131.220.0/22 27.144.0.0/16 27.148.0.0/14 27.152.0.0/13 27.184.0.0/13 27.192.0.0/11 27.224.0.0/14 31.187.66.0/24 31.214.140.128/26 31.220.30.224/27 36.0.0.0/22 36.0.8.0/21 36.0.16.0/20 36.0.32.0/19 36.0.64.0/18 36.0.128.0/17 36.1.0.0/16 36.4.0.0/14 36.16.0.0/12 36.32.0.0/14 36.36.0.0/16 36.37.0.0/19 36.37.36.0/23 36.37.39.0/24 36.37.40.0/21 36.37.48.0/20 36.40.0.0/13 36.48.0.0/15 36.51.0.0/16 36.56.0.0/13 36.96.0.0/11 36.128.0.0/10 36.192.0.0/11 36.248.0.0/14 36.254.0.0/16 37.252.249.0/24 38.72.196.0/23 38.100.175.112/32 38.100.175.114/31 38.100.175.116/30 38.100.175.120/29 38.101.161.177/32 38.104.84.40/32 38.104.84.42/31 39.0.0.0/24 39.0.2.0/23 39.0.4.0/22 39.0.8.0/21 39.0.16.0/20 39.0.32.0/19 39.0.64.0/18 39.0.128.0/17 39.64.0.0/11 39.128.0.0/10 42.0.0.0/22 42.0.8.0/21 42.0.16.0/21 42.0.24.0/22 42.0.32.0/19 42.0.128.0/17 42.1.0.0/19 42.1.32.0/20 42.1.48.0/21 42.1.56.0/22 42.1.128.0/17 42.4.0.0/14 42.48.0.0/13 42.56.0.0/14 42.62.0.0/17 42.62.128.0/19 42.62.160.0/20 42.62.180.0/22 42.62.184.0/21 42.63.0.0/16 42.80.0.0/15 42.83.64.0/20 42.83.80.0/22 42.83.88.0/21 42.83.96.0/19 42.83.128.0/17 42.84.0.0/14 42.88.0.0/13 42.96.64.0/19 42.96.96.0/21 42.96.108.0/22 42.96.112.0/20 42.96.128.0/17 42.97.0.0/16 42.99.0.0/18 42.99.64.0/19 42.99.96.0/20 42.99.112.0/22 42.99.120.0/21 42.100.0.0/14 42.120.0.0/15 42.122.0.0/16 42.123.0.0/19 42.123.36.0/22 42.123.40.0/21 42.123.48.0/20 42.123.64.0/18 42.123.128.0/17 42.128.0.0/12 42.156.0.0/19 42.156.36.0/22 42.156.40.0/21 42.156.48.0/20 42.156.64.0/18 42.156.128.0/17 42.157.0.0/16 42.158.0.0/15 42.160.0.0/12 42.176.0.0/13 42.184.0.0/15 42.186.0.0/16 42.187.0.0/18 42.187.64.0/19 42.187.96.0/20 42.187.112.0/21 42.187.120.0/22 42.187.128.0/17 42.192.0.0/13 42.201.0.0/17 42.202.0.0/15 42.204.0.0/14 42.208.0.0/12 42.224.0.0/12 42.240.0.0/16 42.242.0.0/15 42.244.0.0/14 42.248.0.0/13 46.36.194.111/32 46.36.194.112/29 46.36.194.120/32 46.136.190.0/24 46.248.24.0/23 46.254.51.192/28 49.4.0.0/14 49.51.0.0/16 49.52.0.0/14 49.64.0.0/11 49.112.0.0/13 49.120.0.0/14 49.128.0.0/24 49.128.2.0/23 49.140.0.0/15 49.143.233.24/29 49.143.233.32/32 49.143.233.34/31 49.143.233.36/30 49.152.0.0/14 49.208.0.0/14 49.220.0.0/14 49.232.0.0/14 49.239.0.0/18 49.239.192.0/18 49.246.224.0/19 50.118.128.0/24 54.222.0.0/15 57.92.96.0/20 58.14.0.0/15 58.16.0.0/13 58.24.0.0/15 58.30.0.0/15 58.32.0.0/11 58.65.232.0/21 58.66.0.0/15 58.68.128.0/17 58.82.0.0/17 58.83.0.0/16 58.87.64.0/18 58.99.128.0/17 58.100.0.0/15 58.116.0.0/14 58.128.0.0/13 58.144.0.0/16 58.154.0.0/15 58.192.0.0/11 58.240.0.0/12 59.32.0.0/11 59.64.0.0/12 59.80.0.0/14 59.107.0.0/16 59.108.0.0/14 59.151.0.0/17 59.155.0.0/16 59.172.0.0/14 59.191.0.0/17 59.191.240.0/20 59.192.0.0/10 60.0.0.0/11 60.55.0.0/16 60.63.0.0/16 60.160.0.0/11 60.194.0.0/15 60.200.0.0/13 60.208.0.0/12 60.232.0.0/15 60.235.0.0/16 60.245.128.0/17 60.247.0.0/16 60.252.0.0/16 60.253.128.0/17 60.255.0.0/16 61.4.80.0/20 61.4.176.0/20 61.8.160.0/20 61.14.25.200/29 61.14.29.8/29 61.14.29.80/28 61.28.0.0/17 61.28.195.0/24 61.28.196.0/24 61.29.128.0/17 61.45.128.0/18 61.45.224.0/20 61.47.128.0/18 61.48.0.0/13 61.87.192.0/18 61.128.0.0/10 61.216.99.204/30 61.232.0.0/14 61.236.0.0/15 61.240.0.0/14 63.150.129.176/29 63.162.142.0/24 63.167.100.192/28 63.209.188.6/32 64.37.114.160/27 64.62.151.8/32 64.62.151.10/31 64.62.151.12/30 64.62.206.224/27 64.71.151.96/27 64.71.172.0/24 64.75.14.192/27 64.78.162.0/24 64.78.165.96/28 64.127.103.0/29 64.187.144.0/22 64.187.149.0/24 64.187.150.0/23 64.187.152.0/23 64.187.156.0/23 64.224.124.225/32 64.224.124.226/31 64.224.124.228/30 64.224.124.232/30 64.224.124.236/31 64.224.124.238/32 65.19.134.112/32 65.19.134.114/31 65.19.134.116/30 65.19.135.0/24 65.19.152.0/24 65.19.188.0/24 65.49.72.64/32 65.49.72.66/31 65.49.72.68/30 65.49.72.72/29 65.49.72.80/28 65.49.72.96/27 65.55.209.0/24 65.100.229.0/26 65.100.229.64/27 65.100.229.96/29 65.100.229.128/27 65.100.229.192/28 65.110.57.90/31 65.110.57.92/30 65.110.57.96/30 65.255.36.0/22 65.255.41.0/25 65.255.42.0/25 66.37.216.192/27 66.117.2.4/30 66.117.2.8/30 66.117.2.24/29 66.117.2.32/29 66.117.2.40/30 66.117.2.104/29 66.117.9.48/28 66.117.9.80/28 66.117.9.96/28 66.117.9.112/29 66.117.9.136/29 66.117.9.168/29 66.117.9.176/29 66.117.9.192/27 66.117.9.224/29 66.117.11.96/28 66.118.176.120/29 66.160.130.0/24 66.160.162.0/24 66.165.126.0/28 66.187.71.132/30 66.187.71.200/29 66.187.71.232/29 66.187.72.88/29 66.187.72.200/29 66.187.75.28/30 66.201.188.0/24 66.212.31.96/27 67.29.170.6/32 67.29.172.70/32 67.43.45.0/24 67.72.120.10/32 67.215.7.176/29 67.215.12.40/29 67.215.12.208/29 67.215.241.72/29 67.219.96.192/29 67.219.103.208/29 68.64.160.0/30 68.64.160.52/30 68.64.160.88/29 68.64.160.112/28 68.64.160.128/30 68.64.160.224/30 68.64.160.236/30 68.64.160.244/30 68.64.160.248/29 68.64.161.96/28 68.64.161.120/29 68.64.161.224/29 68.64.161.240/30 68.64.166.84/30 68.64.166.120/29 68.64.167.0/29 68.64.167.32/29 68.64.167.52/30 68.64.167.84/30 68.64.167.96/27 68.64.167.180/30 68.64.167.216/29 68.64.168.8/30 68.64.168.32/29 68.64.168.52/30 68.64.169.8/29 68.64.169.16/28 68.64.169.32/29 68.64.169.48/28 68.64.169.64/28 68.64.169.96/27 68.64.169.160/30 68.64.169.176/29 68.68.101.88/29 68.68.101.200/29 68.68.101.224/29 68.68.102.80/28 68.68.102.104/29 68.68.104.128/29 68.68.105.160/27 68.68.109.128/28 68.71.37.96/28 68.71.38.112/29 68.71.51.208/29 68.71.250.96/27 68.171.102.0/23 69.90.99.136/29 69.165.64.0/23 69.165.66.0/25 69.165.66.128/26 69.165.66.192/27 69.165.66.232/29 69.165.66.240/28 69.165.67.0/24 69.165.68.0/23 69.165.72.0/23 69.165.74.0/24 69.165.75.0/28 69.165.75.24/29 69.165.75.32/28 69.165.75.56/29 69.165.75.64/26 69.165.75.128/25 69.165.76.0/22 69.194.135.176/29 69.197.2.170/32 69.197.2.182/32 69.197.3.88/32 69.197.4.4/32 69.197.4.70/32 69.197.4.132/32 69.197.5.1/32 69.197.5.4/31 69.197.5.66/32 69.197.5.69/32 69.197.5.134/32 69.197.5.196/32 69.197.5.206/32 69.197.6.2/32 69.197.6.5/32 69.197.6.67/32 69.197.6.70/32 69.197.6.193/32 69.197.6.195/32 69.197.6.198/32 69.197.6.200/32 69.197.7.2/32 69.197.7.80/32 69.197.7.84/32 69.197.7.129/32 69.197.7.140/32 69.197.8.71/32 69.197.8.78/32 69.197.8.87/32 69.197.12.64/26 69.197.12.192/26 69.197.17.48/29 69.197.17.112/29 69.197.17.128/26 69.197.17.192/27 69.197.18.0/25 69.197.18.192/26 69.197.20.0/27 69.197.20.112/28 69.197.20.128/29 69.197.20.160/29 69.197.20.176/28 69.197.21.80/28 69.197.21.96/28 69.197.21.160/28 69.197.21.176/29 69.197.21.192/27 69.197.22.0/26 69.197.22.64/30 69.197.22.88/29 69.197.22.128/25 69.197.23.16/29 69.197.23.48/29 69.197.24.8/29 69.197.25.96/27 69.197.25.188/30 69.197.26.0/25 69.197.26.128/26 69.197.26.200/29 69.197.26.224/27 69.197.27.128/27 69.197.27.160/28 69.197.27.224/27 69.197.28.32/29 69.197.28.48/28 69.197.29.128/27 69.197.30.64/28 69.197.30.96/27 69.197.31.48/28 69.197.31.64/28 69.197.31.96/27 69.197.31.128/28 69.197.32.27/32 69.197.33.32/27 69.197.33.128/25 69.197.34.64/26 69.197.34.166/32 69.197.35.105/32 69.197.38.72/29 69.197.38.112/28 69.197.38.160/27 69.197.38.192/26 69.197.39.16/29 69.197.39.32/28 69.197.39.128/25 69.197.40.0/28 69.197.40.32/27 69.197.40.64/27 69.197.40.160/27 69.197.42.116/30 69.197.42.120/29 69.197.44.32/27 69.197.47.0/28 69.197.47.64/27 69.197.47.128/27 69.197.47.160/29 69.197.47.176/28 69.197.47.240/28 69.197.48.0/24 69.197.49.8/29 69.197.49.32/27 69.197.49.64/26 69.197.50.120/29 69.197.51.0/24 69.197.56.16/29 69.197.56.64/26 69.197.58.0/26 69.197.58.96/27 69.197.58.208/28 69.197.59.86/32 69.197.60.128/26 69.197.61.32/28 69.197.61.48/29 69.197.61.88/29 69.197.61.112/28 69.197.62.64/27 69.197.62.160/27 69.197.62.192/26 69.197.63.16/28 69.197.63.240/28 72.1.121.48/28 72.8.129.16/28 72.8.129.80/28 72.8.129.148/32 72.8.129.164/32 72.8.129.192/26 72.8.131.128/26 72.8.133.128/26 72.8.134.109/32 72.8.134.119/32 72.8.134.128/28 72.8.134.192/26 72.8.135.0/24 72.8.136.0/23 72.8.141.16/28 72.8.145.0/24 72.8.157.8/30 72.8.157.16/29 72.8.157.110/32 72.8.157.208/29 72.8.158.0/24 72.8.160.42/32 72.8.162.0/28 72.8.162.16/29 72.8.163.0/24 72.8.166.64/28 72.8.166.80/29 72.8.166.112/29 72.8.167.16/29 72.8.167.48/28 72.8.167.128/28 72.8.167.184/30 72.8.167.220/30 72.8.168.64/28 72.8.168.88/29 72.8.168.108/30 72.8.168.128/25 72.8.172.64/28 72.8.176.54/32 72.8.176.74/32 72.8.176.78/32 72.8.176.154/32 72.8.176.158/32 72.8.177.10/32 72.8.177.14/32 72.8.177.30/32 72.8.177.186/32 72.8.177.190/32 72.8.177.202/32 72.8.178.18/32 72.8.178.46/32 72.8.178.66/32 72.8.178.74/32 72.8.178.130/32 72.8.178.170/32 72.8.178.174/32 72.8.178.186/32 72.8.178.190/32 72.8.179.146/32 72.8.179.150/32 72.8.180.42/32 72.8.180.54/32 72.8.180.62/32 72.8.180.166/32 72.8.180.206/32 72.8.182.114/32 72.8.182.190/32 72.8.183.106/32 72.8.187.0/24 72.8.188.2/32 72.8.188.6/32 72.8.188.136/29 72.8.188.144/28 72.8.188.200/29 72.8.190.109/32 72.13.4.160/28 72.13.81.136/29 72.13.81.144/29 72.13.82.72/29 72.13.82.88/29 72.13.82.120/29 72.20.4.142/32 72.20.4.158/32 72.20.5.160/30 72.20.5.178/32 72.20.6.26/32 72.20.6.76/30 72.20.7.16/28 72.20.8.16/28 72.20.9.16/29 72.20.9.192/28 72.20.9.224/27 72.20.10.8/29 72.20.10.192/28 72.20.10.224/28 72.20.11.100/30 72.20.11.104/30 72.20.12.16/28 72.20.12.160/28 72.20.12.176/29 72.20.13.32/28 72.20.13.48/29 72.20.14.208/28 72.20.16.224/28 72.20.18.96/27 72.20.18.144/28 72.20.21.128/27 72.20.21.160/28 72.20.22.10/32 72.20.22.24/32 72.20.22.28/32 72.20.22.192/26 72.20.24.0/25 72.20.24.128/28 72.20.24.168/29 72.20.24.192/27 72.20.24.224/28 72.20.25.60/30 72.20.26.160/29 72.20.26.226/32 72.20.26.231/32 72.20.26.232/31 72.20.26.234/32 72.20.26.237/32 72.20.26.243/32 72.20.26.249/32 72.20.27.216/29 72.20.27.224/29 72.20.30.160/29 72.20.30.206/32 72.20.30.240/28 72.20.31.0/24 72.20.32.48/29 72.20.33.8/29 72.20.34.10/32 72.20.34.46/32 72.20.34.50/32 72.20.34.82/32 72.20.34.106/32 72.20.34.118/32 72.20.34.168/29 72.20.36.96/27 72.20.37.80/28 72.20.38.184/29 72.20.41.208/29 72.20.43.40/29 72.20.45.92/30 72.20.45.144/29 72.20.48.128/25 72.20.52.128/28 72.20.53.40/29 72.20.54.128/25 72.20.56.150/32 72.20.57.140/32 72.20.57.152/31 72.20.57.155/32 72.20.57.159/32 72.20.61.0/24 72.21.89.232/29 72.28.106.8/29 72.28.106.64/27 72.28.117.0/24 72.52.93.224/32 72.52.93.226/31 72.52.93.228/30 72.52.93.232/29 72.52.93.240/28 72.52.94.240/32 72.52.94.242/31 72.52.94.244/30 74.91.16.32/28 74.91.16.56/29 74.91.16.64/29 74.91.16.160/27 74.91.17.0/26 74.91.17.232/29 74.91.18.16/29 74.91.18.32/27 74.91.18.152/29 74.91.18.160/28 74.91.18.184/29 74.91.18.200/29 74.91.18.208/29 74.91.18.248/29 74.91.20.88/29 74.91.20.136/29 74.91.20.144/28 74.91.20.200/29 74.91.20.208/29 74.91.20.232/29 74.91.29.0/29 74.91.29.48/28 74.91.29.64/28 74.91.30.0/28 74.91.30.24/29 74.91.30.48/28 74.91.30.72/29 74.91.30.80/28 74.91.30.96/29 74.91.30.192/29 74.91.30.208/28 74.91.31.64/26 74.91.31.128/25 74.112.204.168/29 74.121.185.240/29 74.121.190.0/26 74.121.190.96/27 74.121.190.136/29 74.121.191.0/26 74.121.191.96/27 74.121.191.192/27 74.127.38.96/27 76.74.174.160/29 78.24.201.80/28 78.24.202.40/29 78.24.203.40/29 78.24.206.40/29 80.255.45.192/26 81.92.125.240/28 82.98.85.248/29 82.211.52.0/23 82.211.54.0/24 84.41.110.40/29 84.200.200.0/25 87.254.206.0/23 91.190.173.0/24 91.234.36.0/24 95.130.197.0/25 95.130.198.0/25 95.143.72.40/29 98.126.139.200/29 98.126.221.0/24 98.126.222.0/24 101.0.0.0/22 101.1.0.0/22 101.2.172.0/22 101.4.0.0/14 101.16.0.0/12 101.32.0.0/14 101.36.0.0/18 101.36.64.0/20 101.36.80.0/22 101.36.88.0/21 101.36.96.0/19 101.36.128.0/17 101.37.0.0/16 101.38.0.0/15 101.40.0.0/13 101.48.0.0/15 101.50.56.0/22 101.52.0.0/16 101.53.100.0/22 101.54.0.0/16 101.55.224.0/21 101.64.0.0/13 101.72.0.0/14 101.76.0.0/15 101.78.0.0/22 101.78.32.0/19 101.80.0.0/12 101.96.0.0/21 101.96.8.0/22 101.96.16.0/20 101.96.128.0/17 101.99.96.0/19 101.101.64.0/19 101.101.100.0/24 101.101.102.0/23 101.101.104.0/21 101.101.112.0/20 101.102.64.0/19 101.102.100.0/23 101.102.102.0/24 101.102.104.0/21 101.102.112.0/20 101.104.0.0/14 101.110.64.0/19 101.110.96.0/20 101.110.116.0/22 101.110.120.0/21 101.120.0.0/14 101.124.0.0/15 101.126.0.0/16 101.128.0.0/22 101.128.8.0/21 101.128.16.0/20 101.128.32.0/19 101.129.0.0/16 101.130.0.0/15 101.132.0.0/14 101.144.0.0/12 101.192.0.0/13 101.200.0.0/15 101.203.128.0/19 101.203.160.0/21 101.203.172.0/22 101.203.176.0/20 101.204.0.0/14 101.224.0.0/13 101.232.0.0/15 101.234.64.0/21 101.234.76.0/22 101.234.80.0/20 101.234.96.0/19 101.236.0.0/14 101.240.0.0/13 101.248.0.0/15 101.251.0.0/22 101.251.8.0/21 101.251.16.0/20 101.251.32.0/19 101.251.64.0/18 101.251.128.0/17 101.252.0.0/15 101.254.0.0/16 103.1.8.0/22 103.1.20.0/22 103.1.24.0/22 103.1.72.0/22 103.1.88.0/22 103.1.168.0/22 103.2.108.0/22 103.2.156.0/22 103.2.164.0/22 103.2.200.0/21 103.2.208.0/21 103.3.84.0/22 103.3.88.0/21 103.3.96.0/19 103.3.128.0/20 103.3.148.0/22 103.3.152.0/21 103.4.56.0/22 103.4.168.0/22 103.4.184.0/22 103.5.36.0/22 103.5.52.0/22 103.5.56.0/22 103.5.252.0/22 103.6.76.0/22 103.6.220.0/22 103.7.4.0/22 103.7.28.0/22 103.7.212.0/22 103.7.216.0/21 103.8.4.0/22 103.8.8.0/22 103.8.32.0/22 103.8.52.0/22 103.8.108.0/22 103.8.156.0/22 103.8.200.0/21 103.8.220.0/22 103.9.8.0/22 103.9.152.0/22 103.9.248.0/21 103.10.0.0/22 103.10.16.0/22 103.10.84.0/22 103.10.111.0/24 103.10.140.0/22 103.11.180.0/22 103.12.32.0/22 103.12.68.0/22 103.12.136.0/22 103.12.184.0/22 103.12.232.0/22 103.13.124.0/22 103.13.144.0/22 103.13.196.0/22 103.13.244.0/22 103.14.84.0/22 103.14.132.0/22 103.14.136.0/22 103.14.156.0/22 103.14.240.0/22 103.15.4.0/22 103.15.8.0/22 103.15.16.0/22 103.15.96.0/22 103.15.200.0/22 103.16.52.0/22 103.16.80.0/21 103.16.88.0/22 103.16.108.0/22 103.16.124.0/22 103.17.40.0/22 103.17.120.0/22 103.17.160.0/22 103.17.204.0/22 103.17.228.0/22 103.18.192.0/22 103.18.208.0/21 103.18.224.0/22 103.19.12.0/22 103.19.40.0/21 103.19.64.0/21 103.19.72.0/22 103.19.232.0/22 103.20.12.0/22 103.20.32.0/22 103.20.112.0/22 103.20.128.0/22 103.20.160.0/22 103.20.248.0/22 103.21.112.0/21 103.21.136.0/21 103.21.176.0/22 103.21.208.0/22 103.21.240.0/22 103.22.0.0/18 103.22.64.0/19 103.22.100.0/22 103.22.104.0/21 103.22.112.0/20 103.22.188.0/22 103.22.228.0/22 103.22.252.0/22 103.23.8.0/22 103.23.56.0/22 103.23.160.0/21 103.23.176.0/22 103.23.228.0/22 103.24.116.0/22 103.24.128.0/22 103.24.144.0/22 103.24.176.0/22 103.24.184.0/22 103.24.220.0/22 103.24.228.0/22 103.24.248.0/21 103.25.8.0/23 103.25.20.0/22 103.25.24.0/21 103.25.32.0/21 103.25.40.0/22 103.25.48.0/22 103.25.64.0/21 103.25.148.0/22 103.25.156.0/22 103.25.216.0/22 103.26.0.0/22 103.26.64.0/22 103.26.156.0/22 103.26.160.0/22 103.26.228.0/22 103.26.240.0/22 103.27.4.0/22 103.27.12.0/22 103.27.24.0/22 103.27.56.0/22 103.27.96.0/22 103.27.176.0/22 103.27.208.0/22 103.27.240.0/22 103.28.4.0/22 103.28.8.0/22 103.28.204.0/22 103.29.16.0/22 103.29.128.0/21 103.29.136.0/22 103.30.20.0/22 103.30.96.0/22 103.30.148.0/22 103.30.200.0/22 103.30.216.0/22 103.30.228.0/22 103.30.232.0/21 103.31.0.0/22 103.31.48.0/20 103.31.64.0/21 103.31.72.0/22 103.31.148.0/22 103.31.160.0/22 103.31.168.0/22 103.31.200.0/22 103.240.16.0/22 103.240.36.0/22 103.240.72.0/22 103.240.84.0/22 103.240.124.0/22 103.240.156.0/22 103.240.172.0/22 103.240.244.0/22 103.241.12.0/22 103.241.92.0/22 103.241.96.0/22 103.241.160.0/22 103.241.184.0/21 103.241.220.0/22 103.242.8.0/22 103.242.64.0/22 103.242.128.0/21 103.242.160.0/22 103.242.168.0/21 103.242.176.0/22 103.242.200.0/22 103.242.212.0/22 103.242.220.0/22 103.242.240.0/22 103.243.24.0/22 103.243.136.0/22 103.243.252.0/22 103.244.16.0/22 103.244.56.0/21 103.244.64.0/20 103.244.80.0/21 103.244.148.0/22 103.244.164.0/22 103.244.232.0/22 103.244.252.0/22 103.245.23.0/24 103.245.52.0/22 103.245.60.0/22 103.245.80.0/22 103.245.124.0/22 103.245.128.0/22 103.246.8.0/21 103.246.120.0/21 103.246.132.0/22 103.246.152.0/21 103.247.168.0/21 103.247.176.0/22 103.247.200.0/22 103.247.212.0/22 103.248.0.0/23 103.248.64.0/22 103.248.100.0/22 103.248.124.0/22 103.248.152.0/22 103.248.168.0/22 103.248.180.0/22 103.248.192.0/22 103.248.212.0/22 103.248.224.0/21 103.249.12.0/22 103.249.52.0/22 103.249.128.0/22 103.249.136.0/22 103.249.144.0/22 103.249.164.0/22 103.249.168.0/21 103.249.176.0/22 103.249.188.0/22 103.249.192.0/22 103.249.244.0/22 103.249.252.0/22 103.250.32.0/22 103.250.104.0/22 103.250.124.0/22 103.250.180.0/22 103.250.192.0/22 103.250.216.0/22 103.250.224.0/22 103.250.236.0/22 103.250.248.0/21 103.251.32.0/22 103.251.84.0/22 103.251.96.0/22 103.251.124.0/22 103.251.128.0/22 103.251.160.0/22 103.251.204.0/22 103.251.236.0/22 103.251.240.0/22 103.252.28.0/22 103.252.36.0/22 103.252.64.0/22 103.252.104.0/22 103.252.172.0/22 103.252.204.0/22 103.252.208.0/22 103.252.232.0/22 103.252.248.0/22 103.253.4.0/22 103.253.60.0/22 103.253.204.0/22 103.253.220.0/22 103.253.224.0/22 103.253.232.0/22 103.254.8.0/22 103.254.20.0/22 103.254.64.0/20 103.254.112.0/22 103.254.148.0/22 103.254.176.0/22 103.254.188.0/22 103.254.196.0/24 103.254.220.0/22 106.0.0.0/24 106.0.2.0/23 106.0.4.0/22 106.0.8.0/21 106.0.16.0/20 106.0.64.0/18 106.2.0.0/15 106.4.0.0/14 106.8.0.0/15 106.11.0.0/16 106.12.0.0/14 106.16.0.0/12 106.32.0.0/12 106.48.0.0/15 106.50.0.0/16 106.52.0.0/14 106.56.0.0/13 106.74.0.0/15 106.80.0.0/12 106.108.0.0/14 106.112.0.0/12 106.224.0.0/12 108.174.66.96/28 108.174.66.112/29 108.177.157.0/26 109.71.4.0/24 110.6.0.0/15 110.16.0.0/14 110.34.140.0/22 110.40.0.0/14 110.44.144.0/20 110.48.0.0/16 110.51.0.0/16 110.52.0.0/15 110.56.0.0/13 110.64.0.0/15 110.72.0.0/15 110.75.0.0/16 110.76.0.0/18 110.76.156.0/22 110.76.184.0/22 110.76.192.0/18 110.77.0.0/17 110.80.0.0/13 110.88.0.0/14 110.93.32.0/19 110.94.0.0/15 110.96.0.0/11 110.152.0.0/14 110.156.0.0/15 110.165.32.0/19 110.166.0.0/15 110.172.192.0/18 110.173.0.0/19 110.173.32.0/20 110.173.64.0/18 110.173.192.0/19 110.176.0.0/12 110.192.0.0/11 110.228.0.0/14 110.232.32.0/19 110.236.0.0/15 110.240.0.0/12 111.0.0.0/10 111.66.0.0/16 111.67.192.0/20 111.68.64.0/19 111.72.0.0/13 111.85.0.0/16 111.91.192.0/19 111.112.0.0/14 111.116.0.0/15 111.118.200.0/21 111.119.64.0/18 111.119.128.0/19 111.120.0.0/14 111.124.0.0/16 111.126.0.0/15 111.128.0.0/11 111.160.0.0/13 111.170.0.0/16 111.172.0.0/14 111.176.0.0/13 111.186.0.0/15 111.192.0.0/12 111.208.0.0/13 111.221.128.0/17 111.222.0.0/16 111.223.240.0/22 111.223.248.0/22 111.224.0.0/13 111.235.96.0/19 111.235.156.0/22 111.235.160.0/19 112.0.0.0/11 112.32.0.0/12 112.48.0.0/13 112.56.0.0/14 112.60.0.0/15 112.62.0.0/16 112.63.0.0/17 112.63.128.0/19 112.63.160.0/23 112.63.162.0/26 112.63.162.64/27 112.63.162.96/28 112.63.162.112/30 112.63.162.124/30 112.63.162.128/28 112.63.162.144/30 112.63.162.152/29 112.63.162.160/27 112.63.162.192/26 112.63.163.0/24 112.63.164.0/22 112.63.168.0/21 112.63.176.0/20 112.63.192.0/18 112.64.0.0/14 112.73.6.0/23 112.73.8.0/21 112.73.16.0/20 112.73.32.0/19 112.73.64.0/18 112.73.128.0/17 112.74.0.0/15 112.80.0.0/12 112.96.0.0/13 112.109.128.0/17 112.111.0.0/16 112.112.0.0/14 112.116.0.0/15 112.122.0.0/15 112.124.0.0/14 112.128.0.0/14 112.132.0.0/16 112.137.48.0/21 112.192.0.0/14 112.224.0.0/11 113.0.0.0/13 113.8.0.0/15 113.11.192.0/19 113.12.0.0/14 113.16.0.0/15 113.18.0.0/16 113.24.0.0/14 113.31.0.0/16 113.44.0.0/14 113.48.0.0/14 113.52.160.0/19 113.54.0.0/15 113.56.0.0/15 113.58.0.0/16 113.59.0.0/17 113.59.224.0/22 113.62.0.0/15 113.64.0.0/10 113.128.0.0/15 113.130.96.0/20 113.130.112.0/21 113.132.0.0/14 113.136.0.0/13 113.194.0.0/15 113.197.100.0/22 113.200.0.0/15 113.202.0.0/16 113.204.0.0/14 113.208.96.0/19 113.208.128.0/17 113.209.0.0/16 113.212.0.0/18 113.212.100.0/22 113.212.184.0/21 113.213.0.0/17 113.214.0.0/15 113.218.0.0/15 113.220.0.0/14 113.224.0.0/12 113.240.0.0/13 113.248.0.0/14 114.28.0.0/16 114.31.211.0/24 114.54.0.0/15 114.60.0.0/14 114.64.0.0/14 114.68.0.0/16 114.79.64.0/18 114.80.0.0/12 114.96.0.0/13 114.104.0.0/14 114.110.0.0/20 114.110.64.0/18 114.111.0.0/19 114.111.160.0/19 114.112.0.0/20 114.112.16.0/22 114.112.22.0/23 114.112.24.0/21 114.112.32.0/19 114.112.64.0/18 114.112.128.0/23 114.112.131.0/24 114.112.132.0/22 114.112.136.0/21 114.112.144.0/20 114.112.160.0/19 114.112.192.0/19 114.113.0.0/17 114.113.128.0/18 114.113.196.0/22 114.113.200.0/21 114.113.208.0/20 114.113.224.0/20 114.113.240.0/23 114.113.243.0/24 114.113.244.0/23 114.113.247.0/24 114.113.248.0/23 114.114.0.0/15 114.116.0.0/14 114.132.0.0/16 114.135.0.0/16 114.138.0.0/15 114.141.64.0/21 114.141.128.0/18 114.196.0.0/15 114.198.248.0/21 114.208.0.0/12 114.224.0.0/11 115.24.0.0/14 115.28.0.0/15 115.32.0.0/14 115.44.0.0/14 115.48.0.0/12 115.69.64.0/20 115.84.0.0/18 115.84.192.0/19 115.85.192.0/18 115.100.0.0/14 115.104.0.0/14 115.120.0.0/14 115.124.16.0/20 115.148.0.0/14 115.152.0.0/13 115.166.64.0/19 115.168.0.0/13 115.180.0.0/14 115.190.0.0/15 115.192.0.0/11 115.224.0.0/12 116.0.8.0/21 116.0.24.0/21 116.1.0.0/16 116.2.0.0/15 116.4.0.0/14 116.8.0.0/14 116.13.0.0/16 116.16.0.0/12 116.50.0.0/20 116.52.0.0/14 116.56.0.0/15 116.58.128.0/20 116.58.208.0/20 116.60.0.0/14 116.66.0.0/17 116.69.0.0/16 116.70.0.0/17 116.76.0.0/14 116.85.0.0/16 116.89.144.0/20 116.90.80.0/20 116.90.184.0/21 116.95.0.0/16 116.112.0.0/14 116.116.0.0/15 116.128.0.0/10 116.192.0.0/16 116.193.16.0/20 116.193.32.0/19 116.193.176.0/21 116.194.0.0/15 116.196.0.0/16 116.197.180.0/24 116.198.0.0/16 116.199.0.0/17 116.199.128.0/19 116.204.0.0/15 116.207.0.0/16 116.208.0.0/14 116.212.160.0/20 116.213.64.0/18 116.213.128.0/17 116.214.32.0/19 116.214.64.0/20 116.214.110.0/23 116.214.128.0/17 116.215.0.0/16 116.216.0.0/14 116.224.0.0/12 116.242.0.0/15 116.244.0.0/14 116.248.0.0/15 116.251.64.0/18 116.252.0.0/15 116.254.128.0/17 116.255.128.0/17 117.8.0.0/13 117.21.0.0/16 117.22.0.0/15 117.24.0.0/13 117.32.0.0/13 117.40.0.0/14 117.44.0.0/15 117.48.0.0/14 117.53.48.0/20 117.53.176.0/20 117.57.0.0/16 117.58.0.0/17 117.59.0.0/16 117.60.0.0/14 117.64.0.0/13 117.72.0.0/15 117.74.64.0/19 117.74.128.0/17 117.75.0.0/16 117.76.0.0/14 117.80.0.0/12 117.100.0.0/15 117.103.16.0/20 117.103.40.0/21 117.103.72.0/21 117.103.128.0/20 117.104.168.0/21 117.104.188.0/24 117.106.0.0/15 117.112.0.0/13 117.120.64.0/18 117.120.128.0/17 117.121.0.0/17 117.121.128.0/18 117.121.192.0/21 117.122.128.0/17 117.124.0.0/14 117.128.0.0/10 118.24.0.0/15 118.26.0.0/16 118.28.0.0/14 118.64.0.0/15 118.66.0.0/16 118.67.112.0/20 118.72.0.0/13 118.80.0.0/15 118.84.0.0/15 118.88.32.0/19 118.88.64.0/18 118.88.128.0/17 118.89.0.0/16 118.91.240.0/20 118.102.16.0/20 118.102.32.0/21 118.103.240.0/21 118.112.0.0/13 118.120.0.0/14 118.124.0.0/15 118.126.0.0/16 118.127.128.0/19 118.132.0.0/14 118.144.0.0/14 118.178.0.0/16 118.180.0.0/14 118.184.0.0/16 118.186.0.0/15 118.188.0.0/16 118.190.0.0/15 118.192.0.0/16 118.193.0.0/19 118.193.32.0/20 118.193.48.0/27 118.193.48.32/31 118.193.48.34/32 118.193.48.36/30 118.193.48.40/29 118.193.48.48/28 118.193.48.64/26 118.193.48.128/25 118.193.49.0/24 118.193.50.0/23 118.193.52.0/22 118.193.56.0/21 118.193.64.0/18 118.193.128.0/17 118.194.0.0/17 118.194.128.0/18 118.194.192.0/19 118.194.224.0/22 118.194.232.0/21 118.194.240.0/20 118.195.0.0/16 118.196.0.0/14 118.202.0.0/15 118.204.0.0/14 118.212.0.0/15 118.224.0.0/14 118.228.0.0/15 118.230.0.0/16 118.239.0.0/16 118.242.0.0/16 118.244.0.0/14 118.248.0.0/13 119.0.0.0/15 119.2.0.0/19 119.2.128.0/17 119.3.0.0/16 119.4.0.0/14 119.8.0.0/16 119.10.0.0/17 119.15.136.0/21 119.16.0.0/16 119.18.192.0/20 119.18.208.0/21 119.18.224.0/19 119.19.0.0/16 119.20.0.0/14 119.27.64.0/18 119.27.128.0/17 119.28.0.0/15 119.30.48.0/20 119.31.165.0/24 119.31.192.0/19 119.32.0.0/13 119.40.0.0/18 119.40.64.0/20 119.40.128.0/17 119.41.0.0/16 119.42.0.0/19 119.42.128.0/20 119.42.224.0/19 119.44.0.0/15 119.48.0.0/13 119.57.0.0/16 119.58.0.0/16 119.59.128.0/17 119.60.0.0/15 119.62.0.0/16 119.63.32.0/19 119.75.208.0/20 119.78.0.0/15 119.80.0.0/16 119.82.208.0/20 119.84.0.0/14 119.88.0.0/14 119.96.0.0/13 119.108.0.0/15 119.112.0.0/12 119.128.0.0/12 119.144.0.0/14 119.148.160.0/19 119.151.192.0/18 119.160.200.0/21 119.161.128.0/17 119.162.0.0/15 119.164.0.0/14 119.176.0.0/12 119.232.0.0/15 119.235.128.0/18 119.248.0.0/14 119.252.96.0/21 119.252.240.0/20 119.253.0.0/16 119.254.0.0/15 120.0.0.0/12 120.24.0.0/14 120.30.0.0/15 120.32.0.0/12 120.48.0.0/15 120.52.0.0/14 120.64.0.0/13 120.72.32.0/19 120.72.128.0/17 120.76.0.0/14 120.80.0.0/13 120.88.8.0/21 120.90.0.0/15 120.92.0.0/16 120.94.0.0/15 120.128.0.0/13 120.136.128.0/18 120.137.0.0/17 120.138.32.128/27 120.138.32.216/29 120.138.32.224/28 120.138.33.120/29 120.138.34.32/28 120.138.34.128/27 120.138.36.80/28 120.138.40.16/28 120.138.40.32/29 120.138.40.64/26 120.138.40.128/26 120.143.128.0/19 120.192.0.0/10 121.0.8.0/21 121.0.16.0/20 121.4.0.0/15 121.8.0.0/13 121.16.0.0/12 121.32.0.0/13 121.40.0.0/14 121.46.0.0/18 121.46.128.0/17 121.47.0.0/16 121.48.0.0/15 121.50.8.0/21 121.51.0.0/16 121.52.160.0/19 121.52.208.0/20 121.52.224.0/19 121.54.170.0/24 121.54.176.0/21 121.55.0.0/18 121.56.0.0/15 121.58.0.0/17 121.58.136.0/21 121.58.144.0/20 121.58.160.0/21 121.59.0.0/16 121.60.0.0/15 121.62.0.0/16 121.63.0.0/17 121.63.128.0/19 121.63.160.0/23 121.63.162.0/24 121.63.163.0/25 121.63.163.128/28 121.63.163.152/30 121.63.163.160/27 121.63.163.192/26 121.63.164.0/22 121.63.168.0/21 121.63.176.0/20 121.63.192.0/18 121.68.0.0/14 121.76.0.0/15 121.79.128.0/18 121.89.0.0/16 121.100.128.0/17 121.101.0.0/18 121.101.208.0/20 121.192.0.0/13 121.200.192.0/21 121.201.0.0/16 121.204.0.0/14 121.224.0.0/12 121.248.0.0/14 121.255.0.0/16 122.0.64.0/18 122.0.128.0/17 122.4.0.0/14 122.8.0.0/15 122.10.128.0/19 122.10.160.0/22 122.10.164.0/24 122.10.166.0/23 122.10.168.0/21 122.10.176.0/20 122.10.192.0/22 122.10.196.0/23 122.10.216.0/22 122.10.228.0/22 122.10.232.0/21 122.10.240.0/21 122.10.248.0/22 122.11.0.0/17 122.12.0.0/15 122.14.0.0/17 122.14.128.0/19 122.14.160.0/20 122.14.176.0/21 122.14.184.0/22 122.14.188.0/23 122.14.191.0/24 122.14.192.0/18 122.48.0.0/16 122.49.0.0/18 122.51.0.0/16 122.64.0.0/11 122.96.0.0/15 122.102.0.0/20 122.102.64.0/19 122.112.0.0/18 122.112.64.0/19 122.112.96.0/20 122.112.112.0/23 122.112.118.0/23 122.112.120.0/21 122.112.128.0/17 122.113.0.0/16 122.114.0.0/16 122.115.0.0/18 122.115.64.0/21 122.115.72.0/22 122.115.80.0/20 122.115.96.0/19 122.115.128.0/17 122.119.0.0/16 122.128.120.0/21 122.136.0.0/13 122.144.128.0/17 122.152.192.0/18 122.156.0.0/14 122.188.0.0/14 122.192.0.0/14 122.198.0.0/16 122.200.64.0/18 122.201.48.0/20 122.204.0.0/14 122.224.0.0/12 122.240.0.0/13 122.248.24.0/21 122.248.48.0/20 122.248.134.88/29 122.255.64.0/21 123.0.0.0/24 123.0.128.0/18 123.4.0.0/14 123.8.0.0/13 123.49.128.0/17 123.50.160.0/19 123.52.0.0/14 123.56.0.0/14 123.60.0.0/15 123.62.0.0/16 123.64.0.0/11 123.96.0.0/15 123.98.0.0/17 123.99.128.0/17 123.100.0.0/19 123.100.232.0/24 123.101.0.0/16 123.103.0.0/17 123.108.128.0/20 123.108.208.0/20 123.112.0.0/12 123.128.0.0/14 123.132.0.0/16 123.133.0.0/17 123.133.128.0/19 123.133.160.0/20 123.133.176.0/21 123.133.184.0/23 123.133.187.0/24 123.133.188.0/22 123.133.192.0/18 123.134.0.0/15 123.136.80.0/20 123.137.0.0/16 123.138.0.0/15 123.144.0.0/12 123.160.0.0/12 123.176.60.0/22 123.176.80.0/20 123.177.0.0/16 123.178.0.0/15 123.180.0.0/14 123.184.0.0/13 123.196.0.0/15 123.199.128.0/17 123.206.0.0/15 123.232.0.0/14 123.242.0.0/17 123.244.0.0/14 123.249.0.0/16 124.6.64.0/18 124.14.0.0/15 124.16.0.0/15 124.20.0.0/14 124.28.192.0/18 124.29.0.0/17 124.31.0.0/16 124.40.112.0/20 124.40.128.0/18 124.40.192.0/19 124.42.0.0/16 124.47.0.0/18 124.64.0.0/15 124.66.0.0/17 124.67.0.0/16 124.68.0.0/14 124.72.0.0/13 124.88.0.0/13 124.108.8.0/21 124.108.40.0/21 124.109.96.0/21 124.112.0.0/13 124.126.0.0/15 124.128.0.0/13 124.147.128.0/17 124.151.0.0/16 124.152.0.0/16 124.156.0.0/16 124.160.0.0/13 124.172.0.0/14 124.192.0.0/15 124.196.0.0/16 124.200.0.0/13 124.220.0.0/14 124.224.0.0/12 124.240.0.0/17 124.240.128.0/18 124.242.0.0/16 124.243.192.0/18 124.248.0.0/17 124.249.0.0/16 124.250.0.0/15 124.254.0.0/18 125.31.192.0/18 125.32.0.0/12 125.58.128.0/17 125.61.128.0/17 125.62.0.0/18 125.64.0.0/11 125.96.0.0/15 125.98.0.0/16 125.104.0.0/13 125.112.0.0/12 125.169.0.0/16 125.171.0.0/16 125.208.0.0/18 125.210.0.0/15 125.213.0.0/17 125.214.96.0/19 125.215.0.0/18 125.216.0.0/13 125.254.128.0/17 128.204.145.32/28 131.228.188.0/23 134.159.175.0/27 134.196.0.0/16 139.9.0.0/16 139.129.0.0/16 139.148.0.0/16 139.155.0.0/16 139.159.0.0/16 139.170.0.0/16 139.176.0.0/16 139.183.0.0/16 139.186.0.0/16 139.189.0.0/16 139.196.0.0/14 139.200.0.0/13 139.208.0.0/13 139.220.0.0/15 139.224.0.0/16 139.226.0.0/15 140.75.0.0/16 140.143.0.0/16 140.205.0.0/16 140.206.0.0/15 140.210.0.0/16 140.224.0.0/16 140.237.0.0/16 140.240.0.0/16 140.242.6.0/24 140.243.0.0/16 140.246.0.0/16 140.249.0.0/16 140.250.0.0/16 140.255.0.0/16 144.0.0.0/16 144.7.0.0/16 144.12.0.0/16 144.52.0.0/16 144.123.0.0/16 144.255.0.0/16 146.247.122.0/28 147.243.224.0/19 150.0.0.0/16 150.115.0.0/16 150.121.0.0/16 150.122.0.0/16 150.138.0.0/15 150.223.0.0/16 150.255.0.0/16 153.0.0.0/16 153.3.0.0/16 153.34.0.0/15 153.36.0.0/15 153.99.0.0/16 153.101.0.0/16 153.118.0.0/15 157.0.0.0/16 157.18.0.0/16 157.61.0.0/16 157.122.0.0/16 157.148.0.0/16 157.156.0.0/16 157.255.0.0/16 159.226.0.0/16 159.253.146.160/29 159.253.151.2/31 160.62.10.0/24 161.207.0.0/16 162.105.0.0/16 162.209.128.0/20 162.209.145.0/24 162.209.146.0/23 162.209.148.0/22 162.209.152.0/21 162.209.160.0/21 162.209.168.0/22 162.209.172.0/24 162.209.174.0/23 162.209.176.0/24 162.209.178.0/24 162.209.180.0/24 162.209.182.0/23 162.209.184.0/21 162.209.192.0/23 162.209.194.0/24 162.209.196.0/22 162.209.200.0/21 162.209.208.0/20 162.209.224.0/20 162.211.121.64/28 162.211.121.128/27 162.211.121.160/31 162.211.122.12/30 162.211.122.20/30 162.211.122.40/30 162.211.122.48/28 162.211.122.80/28 162.211.125.32/27 162.211.126.16/28 162.211.126.32/30 162.211.126.40/29 162.211.126.48/29 162.211.126.144/30 162.211.126.160/27 162.211.126.192/26 162.211.127.64/29 162.211.127.160/27 162.211.127.232/30 162.218.28.0/22 163.0.0.0/16 163.125.0.0/16 163.142.0.0/16 163.177.0.0/16 163.179.0.0/16 163.204.0.0/16 166.63.11.120/29 166.63.13.48/29 166.63.13.216/29 166.63.14.216/29 166.63.20.48/29 166.111.0.0/16 167.139.0.0/16 168.160.0.0/16 171.8.0.0/13 171.34.0.0/15 171.36.0.0/14 171.40.0.0/13 171.80.0.0/12 171.104.0.0/13 171.112.0.0/12 171.208.0.0/12 172.247.0.0/16 173.45.18.0/23 173.209.63.144/29 173.224.126.178/32 173.224.127.8/32 173.224.127.10/32 173.224.127.14/32 173.231.193.0/24 173.231.194.0/24 173.231.195.0/25 173.231.196.0/22 173.231.200.0/25 173.231.201.0/24 173.231.202.0/24 173.231.203.0/25 173.231.204.128/25 173.231.206.0/25 173.231.207.0/25 173.231.208.0/25 173.231.210.128/25 173.231.211.0/24 173.231.213.128/25 173.231.214.0/25 173.231.216.0/24 173.231.219.0/25 173.231.223.0/24 173.231.227.128/25 173.231.228.0/24 173.231.230.0/24 173.231.231.0/25 173.231.232.128/25 173.231.233.0/24 173.231.235.128/25 173.231.237.0/25 173.231.239.0/25 173.231.244.128/25 173.231.250.128/25 173.231.251.0/24 173.231.253.0/25 173.245.68.16/29 173.245.72.0/29 173.245.72.40/29 173.245.72.80/28 173.245.78.48/29 173.245.78.160/27 173.245.78.192/27 173.245.79.32/27 173.249.207.0/24 173.253.64.0/21 173.253.80.0/20 173.253.98.0/23 173.253.100.0/22 173.253.104.0/21 173.253.112.0/29 173.253.112.64/29 173.253.113.8/29 173.253.113.48/29 173.253.113.80/28 173.253.113.216/29 173.253.114.80/29 173.253.114.128/29 173.253.114.208/29 173.253.115.8/29 173.253.115.96/29 173.253.115.144/29 173.253.115.168/29 173.253.115.240/29 173.253.116.160/29 173.253.116.200/29 173.253.116.240/28 173.253.117.88/29 173.253.117.96/29 173.253.117.112/29 173.253.117.128/29 173.253.117.144/29 173.253.117.184/29 173.253.117.208/29 173.253.118.72/29 173.253.118.80/29 173.253.118.176/29 173.253.119.16/29 173.253.119.40/29 173.253.119.56/29 173.253.119.104/29 173.253.119.112/29 173.253.119.136/29 173.253.119.168/29 173.253.120.8/29 173.253.120.80/28 173.253.120.120/29 173.253.121.144/29 173.253.122.16/29 173.253.122.88/29 173.253.122.112/29 173.253.122.152/29 173.253.122.160/28 173.253.123.8/29 173.253.123.248/29 173.253.124.24/29 173.253.124.88/29 173.253.124.120/29 173.253.124.184/29 173.253.124.192/29 173.253.125.80/29 173.253.125.168/29 173.253.125.184/29 173.255.2.56/29 173.255.2.136/29 173.255.5.80/29 174.136.102.0/28 174.136.104.200/29 174.139.116.168/29 174.139.116.176/29 175.0.0.0/12 175.16.0.0/13 175.24.0.0/14 175.30.0.0/15 175.42.0.0/15 175.44.0.0/16 175.46.0.0/15 175.48.0.0/12 175.64.0.0/11 175.102.0.0/16 175.106.128.0/17 175.146.0.0/15 175.148.0.0/14 175.152.0.0/14 175.160.0.0/12 175.178.0.0/16 175.184.128.0/18 175.185.0.0/16 175.186.0.0/15 175.188.0.0/14 176.67.86.40/30 176.67.86.100/30 178.174.19.128/25 178.175.134.232/29 180.76.0.0/14 180.84.0.0/15 180.86.0.0/16 180.88.0.0/14 180.94.56.0/21 180.94.96.0/20 180.95.128.0/17 180.96.0.0/11 180.129.128.0/17 180.130.0.0/16 180.136.0.0/13 180.148.16.0/21 180.148.152.0/21 180.148.216.0/21 180.148.224.0/19 180.149.128.0/19 180.150.160.0/19 180.152.0.0/13 180.160.0.0/12 180.178.192.0/18 180.184.0.0/14 180.188.0.0/17 180.189.148.0/22 180.200.252.0/22 180.201.0.0/16 180.202.0.0/15 180.208.0.0/15 180.210.224.0/19 180.212.0.0/15 180.222.224.0/19 180.223.0.0/16 180.233.0.0/18 180.233.64.0/19 180.235.64.0/19 182.16.192.0/19 182.18.0.0/17 182.23.184.0/21 182.23.200.0/21 182.32.0.0/12 182.48.96.0/19 182.49.0.0/16 182.50.0.0/20 182.50.112.0/20 182.51.0.0/16 182.54.0.0/17 182.61.0.0/16 182.80.0.0/13 182.88.0.0/14 182.92.0.0/16 182.96.0.0/11 182.128.0.0/12 182.144.0.0/13 182.157.0.0/16 182.160.64.0/19 182.174.0.0/15 182.200.0.0/13 182.236.128.0/17 182.238.0.0/16 182.239.0.0/19 182.240.0.0/13 182.254.0.0/16 183.0.0.0/10 183.64.0.0/13 183.78.180.0/22 183.81.180.0/22 183.84.0.0/15 183.91.128.0/22 183.91.136.0/21 183.91.144.0/20 183.92.0.0/14 183.128.0.0/11 183.160.0.0/13 183.168.0.0/15 183.170.0.0/16 183.172.0.0/14 183.182.0.0/19 183.182.83.64/28 183.182.83.80/29 183.182.83.88/30 183.182.83.112/28 183.184.0.0/13 183.192.0.0/10 184.175.105.228/30 185.12.148.116/30 185.25.48.128/29 185.25.48.144/29 185.25.50.144/28 185.25.50.168/29 185.25.51.0/26 185.25.51.160/28 185.25.51.184/29 185.25.51.224/28 185.33.92.0/22 186.2.168.0/29 192.0.17.0/27 192.0.18.40/29 192.0.22.96/27 192.0.27.192/27 192.80.161.0/24 192.124.154.0/24 192.137.31.0/24 192.151.208.0/20 192.151.244.0/22 192.151.248.0/22 192.151.252.0/23 192.151.255.0/24 192.155.97.0/27 192.161.92.0/22 192.188.170.0/24 192.196.224.176/28 192.196.224.208/28 192.196.224.240/29 192.196.225.112/28 192.196.226.48/28 192.196.227.96/29 192.196.227.144/28 192.196.228.48/28 192.196.228.80/29 192.196.228.104/29 192.196.229.136/29 192.196.231.72/29 192.196.231.112/29 192.196.231.128/29 192.196.231.208/29 192.196.232.40/29 192.196.232.208/29 192.196.233.64/28 192.196.233.200/29 192.196.234.72/29 192.196.234.240/28 192.196.235.16/28 192.196.235.64/28 192.196.235.128/29 192.196.235.152/29 192.196.235.160/27 192.196.236.112/28 192.196.237.96/28 192.211.217.0/24 192.252.176.0/27 192.252.176.32/28 192.252.176.64/26 192.252.176.128/28 192.252.176.152/29 192.252.176.160/27 192.252.176.192/27 192.252.176.224/28 192.252.177.0/26 192.252.177.64/28 192.252.177.80/29 192.252.177.96/27 192.252.177.128/26 192.252.177.200/29 192.252.177.208/28 192.252.177.224/27 192.252.178.0/27 192.252.178.32/29 192.252.178.48/28 192.252.178.64/26 192.252.178.136/29 192.252.178.144/28 192.252.178.160/27 192.252.178.208/28 192.252.178.224/27 192.252.179.0/27 192.252.179.32/28 192.252.179.56/29 192.252.179.64/26 192.252.179.128/27 192.252.179.168/29 192.252.179.176/28 192.252.179.192/26 192.252.180.0/26 192.252.180.64/28 192.252.180.80/29 192.252.180.96/27 192.252.180.128/29 192.252.180.144/28 192.252.180.160/27 192.252.180.192/26 192.252.181.0/26 192.252.181.64/27 192.252.181.104/29 192.252.181.112/28 192.252.181.128/26 192.252.181.192/27 192.252.181.232/29 192.252.181.240/28 192.252.182.0/28 192.252.182.16/29 192.252.182.32/27 192.252.182.64/26 192.252.182.128/28 192.252.182.144/29 192.252.182.160/27 192.252.182.192/27 192.252.182.224/28 192.252.182.248/29 192.252.183.0/26 192.252.183.72/29 192.252.183.80/28 192.252.183.96/27 192.252.183.128/27 192.252.183.168/29 192.252.183.176/28 192.252.183.192/26 192.252.184.0/27 192.252.184.32/28 192.252.184.56/29 192.252.184.64/29 192.252.184.80/28 192.252.184.96/28 192.252.184.120/29 192.252.184.128/26 192.252.184.200/29 192.252.184.208/28 192.252.184.224/27 192.252.185.0/26 192.252.185.64/27 192.252.185.96/28 192.252.185.112/29 192.252.185.128/27 192.252.185.160/28 192.252.185.184/29 192.252.185.192/27 192.252.185.224/28 192.252.185.240/29 192.252.186.0/26 192.252.186.80/28 192.252.186.96/28 192.252.186.112/29 192.252.186.128/25 192.252.187.0/26 192.252.187.64/27 192.252.187.104/29 192.252.187.112/28 192.252.187.128/26 192.252.187.192/28 192.252.187.216/29 192.252.187.224/29 192.252.187.240/28 192.252.188.0/29 192.252.188.16/28 192.252.188.32/27 192.252.188.64/26 192.252.188.128/27 192.252.188.160/28 192.252.188.176/29 192.252.188.192/26 192.252.189.0/28 192.252.189.24/29 192.252.189.32/27 192.252.189.64/26 192.252.189.128/28 192.252.189.152/29 192.252.189.160/27 192.252.189.192/26 192.252.190.0/26 192.252.190.72/29 192.252.190.80/28 192.252.190.96/27 192.252.190.136/29 192.252.190.144/28 192.252.190.160/28 192.252.190.176/29 192.252.190.192/27 192.252.190.224/29 192.252.190.240/28 192.252.191.0/27 192.252.191.32/29 192.252.191.48/28 192.252.191.64/26 192.252.191.128/26 192.252.191.192/28 192.252.191.216/29 192.252.191.224/27 192.253.224.0/28 192.253.224.24/29 192.253.224.32/29 192.253.224.48/28 192.253.224.64/26 192.253.224.128/29 192.253.224.144/28 192.253.224.160/29 192.253.224.176/29 192.253.224.200/29 192.253.224.208/29 192.253.224.232/29 192.253.225.8/29 192.253.225.16/28 192.253.225.32/28 192.253.225.48/29 192.253.225.72/29 192.253.225.80/28 192.253.225.112/28 192.253.225.128/28 192.253.225.160/27 192.253.225.200/29 192.253.225.224/28 192.253.225.240/29 192.253.226.0/28 192.253.226.24/29 192.253.226.40/29 192.253.226.56/29 192.253.226.64/29 192.253.226.80/28 192.253.226.96/27 192.253.226.128/28 192.253.226.152/29 192.253.226.160/27 192.253.226.200/29 192.253.226.216/29 192.253.226.232/29 192.253.226.240/28 192.253.227.0/29 192.253.227.16/29 192.253.227.40/29 192.253.227.48/29 192.253.227.64/29 192.253.227.80/28 192.253.227.96/27 192.253.227.128/28 192.253.227.152/29 192.253.227.168/29 192.253.227.176/29 192.253.227.192/27 192.253.227.224/29 192.253.227.240/29 192.253.228.0/26 192.253.228.64/28 192.253.228.96/27 192.253.228.128/29 192.253.228.144/28 192.253.228.160/28 192.253.228.176/29 192.253.228.192/27 192.253.228.224/29 192.253.229.0/29 192.253.229.32/27 192.253.229.64/28 192.253.229.80/29 192.253.229.96/29 192.253.229.112/28 192.253.229.128/26 192.253.229.192/29 192.253.229.208/29 192.253.229.232/29 192.253.229.240/28 192.253.230.0/28 192.253.230.24/29 192.253.230.32/29 192.253.230.48/28 192.253.230.64/27 192.253.230.96/28 192.253.230.112/29 192.253.230.152/29 192.253.230.160/29 192.253.230.176/28 192.253.230.192/28 192.253.230.224/27 192.253.231.0/26 192.253.231.64/27 192.253.231.104/29 192.253.231.112/28 192.253.231.128/27 192.253.231.168/29 192.253.231.176/28 192.253.231.192/29 192.253.231.248/29 192.253.232.16/29 192.253.232.48/28 192.253.232.64/27 192.253.232.96/28 192.253.232.112/29 192.253.232.128/27 192.253.232.168/29 192.253.232.192/28 192.253.232.216/29 192.253.232.224/27 192.253.233.0/27 192.253.233.32/29 192.253.233.48/28 192.253.233.64/29 192.253.233.80/29 192.253.233.112/28 192.253.233.144/29 192.253.233.168/29 192.253.233.176/28 192.253.233.192/26 192.253.234.0/27 192.253.234.32/28 192.253.234.48/29 192.253.234.72/29 192.253.234.80/29 192.253.234.104/29 192.253.234.120/29 192.253.234.128/27 192.253.234.168/29 192.253.234.176/28 192.253.234.192/27 192.253.234.224/29 192.253.234.240/28 192.253.235.0/27 192.253.235.32/28 192.253.235.56/29 192.253.235.64/28 192.253.235.88/29 192.253.235.96/27 192.253.235.128/28 192.253.235.144/29 192.253.235.160/27 192.253.235.192/29 192.253.235.216/29 192.253.235.224/27 192.253.236.0/27 192.253.236.40/29 192.253.236.48/28 192.253.236.64/27 192.253.236.120/29 192.253.236.128/27 192.253.236.192/27 192.253.236.224/28 192.253.237.8/29 192.253.237.16/28 192.253.237.32/27 192.253.237.64/29 192.253.237.80/29 192.253.237.104/29 192.253.237.152/29 192.253.237.160/27 192.253.237.192/27 192.253.237.232/29 192.253.237.240/28 192.253.238.8/29 192.253.238.24/29 192.253.238.48/29 192.253.238.64/29 192.253.238.80/28 192.253.238.112/28 192.253.238.128/27 192.253.238.160/29 192.253.238.176/28 192.253.238.192/27 192.253.238.224/29 192.253.238.240/29 192.253.239.16/29 192.253.239.32/28 192.253.239.48/29 192.253.239.64/28 192.253.239.96/27 192.253.239.128/28 192.253.239.160/28 192.253.239.176/29 192.253.239.192/28 192.253.239.216/29 192.253.239.224/29 192.253.239.240/29 194.117.103.78/31 194.117.103.120/32 194.117.103.150/32 194.117.103.154/31 194.117.103.181/32 194.117.103.195/32 194.117.103.215/32 194.117.103.218/32 194.117.103.230/32 194.117.119.101/32 194.138.202.0/24 195.112.167.20/30 195.112.167.56/30 195.112.167.84/30 195.112.167.156/30 195.112.177.20/30 195.112.177.124/30 198.1.131.64/28 198.1.137.208/28 198.16.32.0/20 198.16.48.0/21 198.44.128.0/27 198.44.128.160/28 198.44.128.184/29 198.44.128.192/27 198.44.128.240/29 198.44.129.0/29 198.44.129.24/29 198.44.129.48/28 198.44.129.96/27 198.44.129.224/28 198.44.131.112/28 198.44.131.128/27 198.44.132.16/29 198.44.132.64/29 198.44.132.96/27 198.44.132.128/29 198.44.132.184/29 198.44.133.0/28 198.44.134.32/27 198.44.134.200/29 198.44.136.200/29 198.44.136.208/28 198.44.137.32/28 198.44.138.240/28 198.44.139.128/28 198.44.140.96/28 198.44.141.152/29 198.44.142.72/29 198.44.143.216/29 198.44.145.112/28 198.44.146.128/28 198.44.146.176/29 198.44.147.248/29 198.44.148.160/27 198.44.148.240/29 198.44.149.224/29 198.44.150.72/29 198.44.150.232/29 198.44.150.240/28 198.44.151.112/29 198.44.151.128/27 198.44.151.192/27 198.44.152.0/28 198.44.152.40/29 198.44.153.80/29 198.44.153.168/29 198.44.154.208/28 198.44.155.0/28 198.44.155.160/28 198.44.160.0/21 198.44.168.0/23 198.44.170.0/24 198.44.171.0/26 198.44.171.64/28 198.44.171.88/29 198.44.171.96/27 198.44.171.128/25 198.44.172.0/22 198.44.176.0/22 198.44.180.0/28 198.44.180.16/29 198.44.180.32/27 198.44.180.64/26 198.44.180.128/25 198.44.181.0/24 198.44.182.0/23 198.44.184.0/22 198.44.188.0/23 198.54.132.0/25 198.54.134.0/24 198.54.135.0/25 198.84.95.0/24 198.177.56.0/25 198.177.56.128/27 198.177.56.192/26 198.177.57.0/25 198.177.57.160/27 198.177.57.192/26 198.177.58.0/24 198.177.59.0/27 198.177.59.32/29 198.177.59.48/28 198.177.59.64/26 198.177.59.128/25 198.177.60.0/26 198.177.60.128/25 198.177.61.0/28 198.177.61.32/27 198.177.61.64/26 198.177.61.128/27 198.177.61.160/28 198.177.61.192/26 198.177.62.0/23 198.210.59.101/32 198.210.59.102/31 198.210.59.104/29 198.210.59.112/28 198.210.59.128/26 198.210.59.192/30 198.244.50.0/24 198.244.53.0/24 198.244.54.0/24 198.244.56.0/24 198.244.59.0/26 198.244.59.128/25 198.244.61.0/25 198.244.62.128/26 198.244.63.0/24 199.3.210.0/25 199.15.116.2/31 199.15.116.4/30 199.15.116.8/29 199.15.116.16/28 199.15.116.32/28 199.15.116.48/29 199.15.116.56/30 199.15.116.60/31 199.15.116.62/32 199.15.116.66/31 199.15.116.68/30 199.15.116.72/29 199.15.116.80/28 199.15.116.96/28 199.15.116.112/29 199.15.116.120/30 199.15.116.124/31 199.15.116.126/32 199.15.119.0/24 199.16.206.136/29 199.16.206.176/29 199.38.241.224/29 199.38.242.128/27 199.38.242.224/27 199.38.243.32/27 199.46.126.128/29 199.46.126.168/29 199.46.126.176/29 199.101.144.40/29 199.101.144.56/29 199.101.144.80/29 199.101.145.224/29 199.101.146.216/29 199.101.147.16/29 199.101.147.160/29 199.192.75.16/30 199.192.76.22/32 199.192.76.134/32 199.192.76.182/32 199.192.76.186/32 199.192.76.202/32 199.192.76.238/32 199.192.77.38/32 199.192.77.62/32 199.192.77.66/32 199.192.77.86/32 199.192.77.122/32 199.192.77.146/32 199.192.77.202/32 199.192.77.230/32 199.192.77.234/32 199.192.78.98/32 199.192.78.170/32 199.192.79.142/32 199.229.227.160/27 199.231.64.0/28 199.231.64.32/27 199.231.64.64/27 199.231.64.101/32 199.231.64.103/32 199.231.64.104/29 199.231.64.113/32 199.231.64.114/31 199.231.64.116/31 199.231.64.118/32 199.231.64.120/31 199.231.64.122/32 199.231.64.125/32 199.231.64.128/26 199.231.65.0/25 199.231.65.128/26 199.231.65.192/27 199.231.66.0/28 199.231.66.16/29 199.231.66.32/27 199.231.66.64/26 199.231.66.128/26 199.231.67.32/27 199.231.67.64/26 199.231.67.128/27 199.231.68.0/26 199.231.68.128/27 199.231.68.160/29 199.231.68.176/28 199.231.68.192/26 199.231.69.0/25 199.231.69.128/26 199.231.69.192/27 199.231.70.0/25 199.231.70.128/26 199.231.70.192/27 199.231.70.224/28 199.231.70.240/29 199.231.71.0/29 199.231.71.16/28 199.231.71.32/27 199.231.71.64/26 199.231.71.128/25 199.231.228.248/29 199.231.231.32/29 199.241.192.80/29 199.241.192.240/28 199.245.56.0/23 202.0.100.0/23 202.0.122.0/23 202.2.144.64/26 202.3.128.0/23 202.4.128.0/19 202.4.252.0/22 202.6.6.0/23 202.6.66.0/23 202.6.72.0/23 202.6.87.0/24 202.6.88.0/23 202.6.92.0/23 202.6.103.0/24 202.6.108.0/24 202.6.110.0/23 202.6.114.0/24 202.6.176.0/20 202.8.0.0/24 202.8.2.0/23 202.8.4.0/23 202.8.12.0/24 202.8.24.0/24 202.8.77.0/24 202.8.128.0/19 202.8.192.0/20 202.9.32.0/24 202.9.34.0/23 202.9.48.0/23 202.9.51.0/24 202.9.52.0/23 202.9.54.0/24 202.9.57.0/24 202.9.58.0/23 202.10.64.0/20 202.12.1.0/24 202.12.2.0/24 202.12.17.0/24 202.12.18.0/24 202.12.72.0/24 202.12.84.0/23 202.12.96.0/24 202.12.98.0/23 202.12.106.0/24 202.12.111.0/24 202.12.116.0/24 202.14.64.0/23 202.14.69.0/24 202.14.73.0/24 202.14.74.0/23 202.14.76.0/24 202.14.78.0/23 202.14.88.0/24 202.14.97.0/24 202.14.104.0/23 202.14.108.0/23 202.14.111.0/24 202.14.114.0/23 202.14.118.0/23 202.14.124.0/23 202.14.127.0/24 202.14.129.0/24 202.14.135.0/24 202.14.136.0/24 202.14.149.0/24 202.14.151.0/24 202.14.157.0/24 202.14.158.0/23 202.14.169.0/24 202.14.170.0/23 202.14.176.0/24 202.14.184.0/23 202.14.208.0/23 202.14.213.0/24 202.14.219.0/24 202.14.220.0/24 202.14.222.0/23 202.14.225.0/24 202.14.226.0/23 202.14.231.0/24 202.14.235.0/24 202.14.236.0/22 202.14.246.0/24 202.14.251.0/24 202.20.66.0/24 202.20.79.0/24 202.20.87.0/24 202.20.88.0/23 202.20.90.0/24 202.20.94.0/23 202.20.114.0/24 202.20.117.0/24 202.20.120.0/24 202.20.125.0/24 202.20.127.0/24 202.21.131.0/24 202.21.132.0/24 202.21.141.0/24 202.21.142.0/24 202.21.147.0/24 202.21.148.0/24 202.21.150.0/23 202.21.152.0/23 202.21.154.0/24 202.21.156.0/24 202.22.248.0/21 202.27.136.0/23 202.38.0.0/22 202.38.8.0/21 202.38.48.0/20 202.38.64.0/18 202.38.128.0/21 202.38.136.0/23 202.38.138.0/24 202.38.140.0/22 202.38.146.0/23 202.38.149.0/24 202.38.150.0/23 202.38.152.0/22 202.38.156.0/24 202.38.158.0/23 202.38.160.0/23 202.38.164.0/22 202.38.168.0/22 202.38.176.0/23 202.38.184.0/21 202.38.192.0/18 202.40.4.0/23 202.40.7.0/24 202.40.15.0/24 202.40.135.0/24 202.40.136.0/24 202.40.140.0/24 202.40.143.0/24 202.40.144.0/23 202.40.150.0/24 202.40.155.0/24 202.40.156.0/24 202.40.158.0/23 202.40.162.0/24 202.41.8.0/23 202.41.11.0/24 202.41.12.0/23 202.41.128.0/24 202.41.130.0/23 202.41.142.0/24 202.41.152.0/21 202.41.192.0/24 202.41.240.0/20 202.43.76.0/22 202.43.144.0/20 202.44.16.0/20 202.44.67.0/24 202.44.74.0/24 202.44.129.0/24 202.44.132.0/23 202.44.146.0/23 202.45.0.0/23 202.45.2.0/24 202.45.15.0/24 202.45.16.0/20 202.46.16.0/23 202.46.18.0/24 202.46.20.0/23 202.46.32.0/19 202.46.128.0/24 202.46.224.0/20 202.47.82.0/23 202.47.126.0/24 202.47.128.0/24 202.47.130.0/23 202.57.240.0/20 202.58.0.0/24 202.59.0.0/24 202.59.212.0/22 202.59.232.0/23 202.59.236.0/24 202.60.48.0/21 202.60.96.0/21 202.60.112.0/20 202.60.132.0/22 202.60.136.0/21 202.60.144.0/20 202.62.112.0/22 202.62.248.0/22 202.62.252.0/24 202.62.255.0/24 202.63.81.0/24 202.63.82.0/23 202.63.84.0/22 202.63.88.0/21 202.63.160.0/19 202.63.248.0/22 202.65.0.0/21 202.65.8.0/23 202.67.0.0/22 202.69.4.0/22 202.69.16.0/20 202.70.0.0/19 202.70.96.0/20 202.70.161.0/24 202.70.162.0/23 202.70.164.0/23 202.70.166.0/24 202.70.169.0/24 202.70.170.0/23 202.70.172.0/22 202.70.192.0/20 202.72.40.0/21 202.72.80.0/20 202.73.128.0/22 202.74.8.0/21 202.74.80.0/20 202.74.254.0/23 202.75.208.0/20 202.75.252.0/22 202.76.252.0/22 202.77.80.0/21 202.77.92.0/22 202.77.170.48/28 202.78.8.0/21 202.79.224.0/21 202.79.248.0/22 202.80.192.0/20 202.81.0.0/22 202.83.252.0/22 202.84.4.0/22 202.84.8.0/21 202.84.24.0/21 202.85.208.0/20 202.86.249.0/24 202.86.252.0/22 202.87.80.0/20 202.89.8.0/21 202.89.232.0/21 202.90.0.0/22 202.90.112.0/20 202.90.196.0/24 202.90.224.0/20 202.91.0.0/22 202.91.96.0/20 202.91.128.0/22 202.91.176.0/20 202.91.224.0/19 202.92.0.0/22 202.92.8.0/21 202.92.48.0/20 202.92.252.0/22 202.93.0.0/22 202.93.252.0/22 202.94.92.0/22 202.95.0.0/19 202.95.240.0/21 202.95.252.0/22 202.96.0.0/16 202.97.0.0/18 202.97.64.0/19 202.97.128.0/17 202.98.0.0/15 202.100.0.0/14 202.104.0.0/13 202.112.0.0/13 202.120.0.0/20 202.120.16.0/21 202.120.24.0/25 202.120.24.128/26 202.120.24.192/27 202.120.25.0/24 202.120.26.0/23 202.120.28.0/22 202.120.32.0/19 202.120.64.0/18 202.120.128.0/17 202.121.0.0/16 202.122.0.0/21 202.122.32.0/21 202.122.64.0/19 202.122.112.0/20 202.122.128.0/24 202.122.132.0/24 202.123.96.0/20 202.124.16.0/21 202.124.24.0/22 202.125.112.0/20 202.125.176.0/20 202.127.0.0/21 202.127.12.0/22 202.127.16.0/20 202.127.40.0/21 202.127.48.0/20 202.127.112.0/20 202.127.128.0/19 202.127.160.0/21 202.127.192.0/20 202.127.208.0/23 202.127.212.0/22 202.127.216.0/21 202.127.224.0/19 202.130.0.0/19 202.130.224.0/19 202.131.16.0/21 202.131.48.0/20 202.131.208.0/20 202.133.32.0/20 202.134.58.0/24 202.134.128.0/20 202.136.48.0/20 202.136.208.0/20 202.136.224.0/20 202.137.231.0/24 202.141.160.0/19 202.142.16.0/20 202.143.4.0/22 202.143.16.0/20 202.143.32.0/20 202.143.56.0/21 202.146.160.0/20 202.146.188.0/22 202.146.196.0/22 202.146.200.0/21 202.147.144.0/20 202.148.32.0/20 202.148.64.0/18 202.149.32.0/19 202.149.160.0/19 202.149.224.0/19 202.150.16.0/20 202.150.32.0/20 202.150.56.0/22 202.150.192.0/20 202.150.224.0/19 202.151.0.0/22 202.151.128.0/19 202.152.176.0/20 202.153.0.0/22 202.153.48.0/20 202.153.194.192/27 202.157.192.0/19 202.158.160.0/19 202.160.176.0/20 202.162.67.0/24 202.162.75.0/24 202.164.0.0/20 202.164.96.0/19 202.165.176.0/20 202.165.208.0/20 202.165.239.0/24 202.165.240.0/23 202.165.243.0/24 202.165.245.0/24 202.165.251.0/24 202.165.252.0/22 202.166.224.0/19 202.168.160.0/19 202.170.128.0/19 202.170.216.0/21 202.170.224.0/19 202.171.216.0/21 202.171.235.0/24 202.172.0.0/22 202.173.0.0/22 202.173.8.0/21 202.173.224.0/19 202.174.64.0/20 202.176.224.0/19 202.179.240.0/20 202.180.128.0/19 202.180.208.0/21 202.181.112.0/20 202.182.32.0/20 202.182.192.0/19 202.189.0.0/18 202.189.80.0/20 202.189.184.0/21 202.191.0.0/24 202.191.68.0/22 202.191.72.0/21 202.191.80.0/20 202.192.0.0/17 202.192.128.0/18 202.192.192.0/19 202.192.224.0/20 202.192.240.0/23 202.192.243.0/24 202.192.244.0/22 202.192.248.0/21 202.193.0.0/16 202.194.0.0/15 202.196.0.0/14 202.200.0.0/13 203.0.4.0/22 203.0.18.0/24 203.0.24.0/24 203.0.45.0/24 203.0.81.0/24 203.0.82.0/24 203.0.90.0/23 203.0.104.0/21 203.0.114.0/23 203.0.122.0/24 203.0.128.0/24 203.0.132.0/22 203.0.137.0/24 203.0.142.0/24 203.0.144.0/24 203.0.146.0/24 203.0.148.0/24 203.0.150.0/23 203.0.152.0/24 203.0.177.0/24 203.0.224.0/24 203.1.18.0/24 203.1.26.0/23 203.1.65.0/24 203.1.66.0/23 203.1.90.0/24 203.1.97.0/24 203.1.108.0/24 203.1.253.0/24 203.1.254.0/24 203.2.73.0/24 203.2.112.0/21 203.2.140.0/24 203.2.150.0/24 203.2.152.0/22 203.2.156.0/23 203.2.160.0/21 203.2.196.0/23 203.2.209.0/24 203.2.214.0/23 203.2.229.0/24 203.3.68.0/24 203.3.75.0/24 203.3.80.0/21 203.3.96.0/22 203.3.105.0/24 203.3.112.0/21 203.3.120.0/24 203.3.123.0/24 203.3.135.0/24 203.3.139.0/24 203.3.143.0/24 203.4.132.0/23 203.4.134.0/24 203.4.151.0/24 203.4.152.0/22 203.4.180.0/24 203.4.186.0/24 203.4.205.0/24 203.4.208.0/22 203.4.227.0/24 203.5.4.0/23 203.5.7.0/24 203.5.11.0/24 203.5.21.0/24 203.5.22.0/24 203.5.44.0/24 203.5.46.0/23 203.5.52.0/22 203.5.56.0/23 203.5.118.0/24 203.5.120.0/24 203.5.172.0/24 203.5.180.0/23 203.5.182.0/24 203.5.185.0/24 203.5.186.0/24 203.5.190.0/24 203.5.195.0/24 203.6.131.0/24 203.6.136.0/24 203.6.142.0/24 203.6.150.0/23 203.6.157.0/24 203.6.159.0/24 203.6.224.0/20 203.7.129.0/24 203.7.147.0/24 203.7.150.0/23 203.7.158.0/24 203.7.192.0/23 203.7.200.0/24 203.8.0.0/24 203.8.8.0/24 203.8.23.0/24 203.8.24.0/21 203.8.70.0/24 203.8.82.0/24 203.8.86.0/23 203.8.91.0/24 203.8.110.0/23 203.8.115.0/24 203.8.166.0/23 203.8.169.0/24 203.8.173.0/24 203.8.184.0/24 203.8.186.0/23 203.8.190.0/23 203.8.192.0/24 203.8.197.0/24 203.8.198.0/23 203.8.203.0/24 203.8.209.0/24 203.8.210.0/23 203.8.212.0/22 203.8.217.0/24 203.8.220.0/24 203.9.32.0/24 203.9.36.0/23 203.9.57.0/24 203.9.63.0/24 203.9.65.0/24 203.9.70.0/23 203.9.72.0/24 203.9.75.0/24 203.9.96.0/22 203.9.108.0/24 203.9.158.0/24 203.10.34.0/24 203.10.56.0/24 203.10.74.0/23 203.10.84.0/22 203.10.88.0/24 203.10.95.0/24 203.10.125.0/24 203.11.70.0/24 203.11.76.0/22 203.11.82.0/24 203.11.84.0/22 203.11.100.0/22 203.11.109.0/24 203.11.117.0/24 203.11.122.0/24 203.11.126.0/24 203.11.136.0/22 203.11.141.0/24 203.11.142.0/23 203.11.180.0/22 203.11.208.0/22 203.12.16.0/24 203.12.19.0/24 203.12.24.0/24 203.12.57.0/24 203.12.65.0/24 203.12.66.0/24 203.12.87.0/24 203.12.88.0/21 203.12.100.0/23 203.12.103.0/24 203.12.114.0/24 203.12.118.0/24 203.12.130.0/24 203.12.137.0/24 203.12.196.0/22 203.12.200.0/21 203.12.211.0/24 203.12.219.0/24 203.12.226.0/24 203.13.18.0/24 203.13.24.0/24 203.13.44.0/23 203.13.80.0/21 203.13.92.0/22 203.13.145.0/24 203.13.173.0/24 203.13.227.0/24 203.13.233.0/24 203.14.24.0/22 203.14.33.0/24 203.14.56.0/24 203.14.61.0/24 203.14.62.0/24 203.14.104.0/24 203.14.114.0/23 203.14.118.0/24 203.14.162.0/24 203.14.184.0/21 203.14.192.0/24 203.14.214.0/24 203.14.231.0/24 203.14.246.0/24 203.15.0.0/20 203.15.22.0/24 203.15.87.0/24 203.15.88.0/23 203.15.105.0/24 203.15.112.0/21 203.15.130.0/23 203.15.149.0/24 203.15.151.0/24 203.15.156.0/22 203.15.174.0/24 203.15.227.0/24 203.15.232.0/21 203.15.240.0/23 203.15.246.0/24 203.16.10.0/24 203.16.12.0/23 203.16.16.0/21 203.16.27.0/24 203.16.38.0/24 203.16.49.0/24 203.16.50.0/23 203.16.58.0/24 203.16.133.0/24 203.16.161.0/24 203.16.162.0/24 203.16.228.0/24 203.16.238.0/24 203.16.240.0/24 203.16.245.0/24 203.17.2.0/24 203.17.18.0/24 203.17.28.0/24 203.17.39.0/24 203.17.56.0/24 203.17.74.0/23 203.17.136.0/24 203.17.164.0/24 203.17.187.0/24 203.17.231.0/24 203.17.233.0/24 203.17.248.0/24 203.17.255.0/24 203.18.4.0/24 203.18.7.0/24 203.18.31.0/24 203.18.37.0/24 203.18.48.0/23 203.18.50.0/24 203.18.52.0/24 203.18.72.0/22 203.18.87.0/24 203.18.105.0/24 203.18.107.0/24 203.18.110.0/24 203.18.129.0/24 203.18.131.0/24 203.18.144.0/24 203.18.153.0/24 203.18.199.0/24 203.18.208.0/24 203.18.211.0/24 203.18.215.0/24 203.19.18.0/24 203.19.24.0/24 203.19.30.0/24 203.19.32.0/21 203.19.41.0/24 203.19.44.0/23 203.19.46.0/24 203.19.58.0/24 203.19.64.0/24 203.19.68.0/24 203.19.72.0/24 203.19.101.0/24 203.19.111.0/24 203.19.131.0/24 203.19.133.0/24 203.19.144.0/24 203.19.149.0/24 203.19.156.0/24 203.19.176.0/24 203.19.178.0/23 203.19.208.0/24 203.19.228.0/22 203.19.233.0/24 203.19.242.0/24 203.19.248.0/23 203.19.255.0/24 203.20.17.0/24 203.20.40.0/23 203.20.48.0/24 203.20.61.0/24 203.20.65.0/24 203.20.89.0/24 203.20.106.0/23 203.20.115.0/24 203.20.117.0/24 203.20.122.0/24 203.20.135.0/24 203.20.136.0/21 203.20.150.0/24 203.20.230.0/24 203.20.232.0/24 203.20.236.0/24 203.21.0.0/23 203.21.2.0/24 203.21.8.0/24 203.21.10.0/24 203.21.18.0/24 203.21.33.0/24 203.21.34.0/24 203.21.41.0/24 203.21.44.0/24 203.21.68.0/24 203.21.82.0/24 203.21.96.0/22 203.21.124.0/24 203.21.136.0/23 203.21.145.0/24 203.21.206.0/24 203.22.24.0/24 203.22.28.0/23 203.22.31.0/24 203.22.68.0/24 203.22.76.0/24 203.22.78.0/24 203.22.84.0/24 203.22.87.0/24 203.22.99.0/24 203.22.106.0/24 203.22.131.0/24 203.22.163.0/24 203.22.166.0/24 203.22.170.0/24 203.22.176.0/21 203.22.194.0/24 203.22.242.0/23 203.22.245.0/24 203.22.246.0/24 203.22.252.0/23 203.23.0.0/24 203.23.47.0/24 203.23.61.0/24 203.23.73.0/24 203.23.85.0/24 203.23.92.0/22 203.23.98.0/24 203.23.107.0/24 203.23.112.0/24 203.23.130.0/24 203.23.140.0/23 203.23.172.0/24 203.23.182.0/24 203.23.186.0/23 203.23.192.0/24 203.23.197.0/24 203.23.198.0/24 203.23.224.0/24 203.23.226.0/23 203.23.228.0/22 203.23.249.0/24 203.23.251.0/24 203.24.13.0/24 203.24.18.0/24 203.24.27.0/24 203.24.43.0/24 203.24.56.0/24 203.24.58.0/24 203.24.67.0/24 203.24.74.0/24 203.24.79.0/24 203.24.86.0/24 203.24.90.0/24 203.24.111.0/24 203.24.112.0/24 203.24.116.0/24 203.24.122.0/23 203.24.145.0/24 203.24.157.0/24 203.24.161.0/24 203.24.167.0/24 203.24.186.0/23 203.24.199.0/24 203.24.202.0/24 203.24.217.0/24 203.24.219.0/24 203.24.244.0/24 203.25.19.0/24 203.25.46.0/24 203.25.48.0/21 203.25.64.0/23 203.25.91.0/24 203.25.99.0/24 203.25.100.0/24 203.25.106.0/24 203.25.131.0/24 203.25.135.0/24 203.25.138.0/24 203.25.147.0/24 203.25.153.0/24 203.25.154.0/23 203.25.164.0/24 203.25.166.0/24 203.25.174.0/23 203.25.180.0/24 203.25.182.0/24 203.25.191.0/24 203.25.199.0/24 203.25.200.0/24 203.25.208.0/20 203.25.229.0/24 203.25.235.0/24 203.25.236.0/24 203.25.242.0/24 203.26.12.0/24 203.26.34.0/24 203.26.49.0/24 203.26.50.0/24 203.26.55.0/24 203.26.56.0/23 203.26.60.0/24 203.26.65.0/24 203.26.68.0/24 203.26.76.0/24 203.26.80.0/24 203.26.84.0/24 203.26.97.0/24 203.26.102.0/23 203.26.115.0/24 203.26.116.0/24 203.26.129.0/24 203.26.143.0/24 203.26.144.0/24 203.26.148.0/23 203.26.154.0/24 203.26.158.0/23 203.26.170.0/24 203.26.173.0/24 203.26.176.0/24 203.26.185.0/24 203.26.210.0/24 203.26.214.0/24 203.26.222.0/24 203.26.224.0/24 203.26.228.0/24 203.26.232.0/24 203.27.0.0/24 203.27.10.0/24 203.27.15.0/24 203.27.16.0/24 203.27.20.0/24 203.27.22.0/23 203.27.40.0/24 203.27.45.0/24 203.27.53.0/24 203.27.65.0/24 203.27.66.0/24 203.27.81.0/24 203.27.88.0/24 203.27.102.0/24 203.27.109.0/24 203.27.117.0/24 203.27.121.0/24 203.27.125.0/24 203.27.200.0/24 203.27.202.0/24 203.27.233.0/24 203.27.241.0/24 203.27.250.0/24 203.28.10.0/24 203.28.12.0/24 203.28.33.0/24 203.28.43.0/24 203.28.44.0/24 203.28.54.0/24 203.28.56.0/24 203.28.73.0/24 203.28.74.0/24 203.28.76.0/24 203.28.86.0/24 203.28.88.0/24 203.28.112.0/24 203.28.131.0/24 203.28.136.0/24 203.28.140.0/24 203.28.145.0/24 203.28.165.0/24 203.28.169.0/24 203.28.170.0/24 203.28.178.0/23 203.28.185.0/24 203.28.187.0/24 203.28.196.0/24 203.28.226.0/23 203.28.239.0/24 203.29.2.0/24 203.29.13.0/24 203.29.14.0/24 203.29.28.0/24 203.29.46.0/24 203.29.57.0/24 203.29.61.0/24 203.29.63.0/24 203.29.69.0/24 203.29.73.0/24 203.29.81.0/24 203.29.90.0/24 203.29.95.0/24 203.29.100.0/24 203.29.103.0/24 203.29.112.0/24 203.29.120.0/22 203.29.182.0/23 203.29.187.0/24 203.29.189.0/24 203.29.190.0/24 203.29.205.0/24 203.29.210.0/24 203.29.217.0/24 203.29.227.0/24 203.29.231.0/24 203.29.233.0/24 203.29.234.0/24 203.29.248.0/24 203.30.25.0/24 203.30.27.0/24 203.30.29.0/24 203.30.66.0/24 203.30.81.0/24 203.30.87.0/24 203.30.111.0/24 203.30.121.0/24 203.30.123.0/24 203.30.152.0/24 203.30.156.0/24 203.30.162.0/24 203.30.173.0/24 203.30.175.0/24 203.30.187.0/24 203.30.194.0/24 203.30.217.0/24 203.30.220.0/24 203.30.222.0/24 203.30.232.0/23 203.30.235.0/24 203.30.246.0/24 203.30.250.0/23 203.31.45.0/24 203.31.46.0/24 203.31.49.0/24 203.31.51.0/24 203.31.54.0/23 203.31.69.0/24 203.31.72.0/24 203.31.80.0/24 203.31.85.0/24 203.31.97.0/24 203.31.105.0/24 203.31.106.0/24 203.31.108.0/23 203.31.124.0/24 203.31.162.0/24 203.31.174.0/24 203.31.177.0/24 203.31.181.0/24 203.31.187.0/24 203.31.189.0/24 203.31.204.0/24 203.31.220.0/24 203.31.222.0/23 203.31.225.0/24 203.31.229.0/24 203.31.253.0/24 203.32.20.0/24 203.32.56.0/24 203.32.60.0/24 203.32.62.0/24 203.32.68.0/23 203.32.76.0/24 203.32.81.0/24 203.32.95.0/24 203.32.102.0/24 203.32.105.0/24 203.32.130.0/24 203.32.133.0/24 203.32.140.0/24 203.32.152.0/24 203.32.186.0/23 203.32.192.0/24 203.32.196.0/24 203.32.203.0/24 203.32.212.0/24 203.33.4.0/24 203.33.7.0/24 203.33.8.0/21 203.33.21.0/24 203.33.26.0/24 203.33.32.0/24 203.33.63.0/24 203.33.64.0/24 203.33.67.0/24 203.33.68.0/24 203.33.73.0/24 203.33.79.0/24 203.33.100.0/24 203.33.122.0/24 203.33.129.0/24 203.33.131.0/24 203.33.145.0/24 203.33.156.0/24 203.33.174.0/24 203.33.185.0/24 203.33.200.0/24 203.33.204.0/24 203.33.226.0/24 203.33.233.0/24 203.33.243.0/24 203.33.250.0/24 203.34.4.0/24 203.34.21.0/24 203.34.27.0/24 203.34.39.0/24 203.34.54.0/24 203.34.67.0/24 203.34.69.0/24 203.34.76.0/24 203.34.92.0/24 203.34.106.0/24 203.34.113.0/24 203.34.147.0/24 203.34.150.0/24 203.34.152.0/23 203.34.161.0/24 203.34.162.0/24 203.34.187.0/24 203.34.192.0/21 203.34.204.0/22 203.34.232.0/24 203.34.240.0/24 203.34.242.0/24 203.34.245.0/24 203.34.251.0/24 203.55.4.0/24 203.55.10.0/24 203.55.13.0/24 203.55.22.0/24 203.55.30.0/24 203.55.93.0/24 203.55.101.0/24 203.55.109.0/24 203.55.110.0/24 203.55.116.0/23 203.55.119.0/24 203.55.128.0/23 203.55.192.0/24 203.55.196.0/24 203.55.218.0/24 203.55.221.0/24 203.55.224.0/24 203.56.1.0/24 203.56.4.0/24 203.56.12.0/24 203.56.24.0/24 203.56.38.0/24 203.56.40.0/24 203.56.46.0/24 203.56.48.0/21 203.56.82.0/23 203.56.95.0/24 203.56.110.0/24 203.56.121.0/24 203.56.161.0/24 203.56.169.0/24 203.56.175.0/24 203.56.183.0/24 203.56.185.0/24 203.56.187.0/24 203.56.192.0/24 203.56.198.0/24 203.56.201.0/24 203.56.208.0/23 203.56.210.0/24 203.56.214.0/24 203.56.216.0/24 203.56.227.0/24 203.56.228.0/24 203.56.232.0/24 203.56.240.0/24 203.56.252.0/24 203.56.254.0/24 203.57.5.0/24 203.57.6.0/24 203.57.12.0/23 203.57.28.0/24 203.57.39.0/24 203.57.46.0/24 203.57.58.0/24 203.57.61.0/24 203.57.66.0/24 203.57.69.0/24 203.57.70.0/23 203.57.73.0/24 203.57.90.0/24 203.57.101.0/24 203.57.109.0/24 203.57.123.0/24 203.57.157.0/24 203.57.200.0/24 203.57.202.0/24 203.57.206.0/24 203.57.222.0/24 203.57.224.0/20 203.57.249.0/24 203.57.253.0/24 203.57.254.0/23 203.62.2.0/24 203.62.131.0/24 203.62.139.0/24 203.62.161.0/24 203.62.197.0/24 203.62.228.0/22 203.62.234.0/24 203.62.246.0/24 203.76.160.0/22 203.76.168.0/22 203.77.180.0/22 203.78.48.0/20 203.79.0.0/20 203.80.4.0/23 203.80.32.0/20 203.80.57.0/24 203.80.132.0/22 203.80.136.0/21 203.80.144.0/20 203.81.0.0/21 203.81.16.0/20 203.82.0.0/23 203.82.16.0/21 203.83.0.0/22 203.83.56.0/21 203.83.224.0/20 203.86.0.0/17 203.86.254.0/23 203.88.32.0/19 203.88.192.0/19 203.89.0.0/22 203.89.8.0/21 203.89.136.0/22 203.90.0.0/22 203.90.8.0/22 203.90.128.0/18 203.90.192.0/19 203.91.32.0/19 203.91.96.0/20 203.91.120.0/21 203.92.0.0/22 203.92.160.0/19 203.93.0.0/16 203.94.0.0/19 203.95.0.0/21 203.95.96.0/19 203.95.128.0/18 203.95.224.0/19 203.98.193.168/29 203.98.193.240/29 203.98.194.16/28 203.98.195.48/28 203.98.195.192/26 203.98.201.0/24 203.98.202.64/26 203.98.202.160/28 203.98.202.176/29 203.98.205.32/28 203.98.205.168/29 203.98.207.0/29 203.98.207.80/28 203.98.207.96/28 203.98.209.128/26 203.98.210.0/28 203.98.210.96/29 203.98.213.24/29 203.98.213.176/29 203.99.8.0/21 203.99.16.0/20 203.99.80.0/20 203.100.32.0/20 203.100.48.0/21 203.100.63.0/24 203.100.80.0/20 203.100.96.0/19 203.100.158.253/32 203.100.158.254/32 203.100.192.0/20 203.104.32.0/20 203.105.96.0/19 203.105.128.0/19 203.107.0.0/17 203.110.160.0/19 203.110.208.0/20 203.110.232.0/23 203.110.234.0/24 203.114.244.0/22 203.118.192.0/19 203.118.241.0/24 203.118.248.0/22 203.119.24.0/21 203.119.32.0/22 203.119.80.0/22 203.119.85.0/24 203.119.113.0/24 203.119.114.0/23 203.119.116.0/22 203.119.120.0/21 203.119.128.0/17 203.128.32.0/19 203.128.96.0/19 203.128.224.0/21 203.129.8.0/21 203.130.32.0/19 203.132.32.0/19 203.134.240.0/21 203.135.96.0/19 203.135.160.0/20 203.142.12.0/23 203.142.224.0/19 203.144.96.0/19 203.145.0.0/19 203.148.0.0/18 203.148.64.0/20 203.148.80.0/22 203.148.86.0/23 203.149.92.0/22 203.152.64.0/19 203.152.128.0/19 203.153.0.0/22 203.156.192.0/18 203.158.16.0/21 203.160.104.0/21 203.160.129.0/24 203.160.192.0/19 203.161.0.0/22 203.161.180.0/24 203.161.192.0/19 203.166.160.0/19 203.168.0.0/19 203.170.58.0/23 203.171.0.0/22 203.171.224.0/20 203.174.4.0/24 203.174.7.0/24 203.174.96.0/19 203.175.128.0/19 203.175.192.0/18 203.176.0.0/18 203.176.64.0/19 203.176.168.0/21 203.184.80.0/20 203.187.160.0/19 203.189.0.0/23 203.189.6.0/23 203.189.112.0/22 203.189.192.0/19 203.190.96.0/20 203.190.249.0/24 203.191.0.0/23 203.191.16.0/21 203.191.28.0/22 203.191.64.0/18 203.191.144.0/20 203.192.0.0/19 203.192.149.113/32 203.192.149.114/32 203.193.224.0/19 203.194.120.0/21 203.195.64.0/19 203.195.112.0/21 203.195.128.0/17 203.196.0.0/20 203.202.236.0/22 203.205.64.0/19 203.205.128.0/17 203.207.64.0/18 203.207.128.0/17 203.208.0.0/20 203.208.16.0/22 203.208.32.0/19 203.208.199.0/28 203.208.206.0/24 203.209.224.0/19 203.212.0.0/20 203.212.80.0/20 203.215.146.0/24 203.215.232.0/21 203.222.44.160/28 203.222.166.96/29 203.222.166.192/26 203.222.167.56/29 203.222.167.128/28 203.222.174.200/29 203.222.176.96/29 203.222.176.108/30 203.222.176.244/30 203.222.176.248/29 203.222.177.0/26 203.222.177.64/29 203.222.177.104/29 203.222.177.192/26 203.222.182.96/28 203.222.182.144/28 203.222.187.176/29 203.223.16.0/21 205.134.243.80/29 205.134.243.240/29 205.145.139.128/28 206.73.60.96/27 206.73.68.128/27 206.73.91.32/27 206.73.91.64/27 206.73.195.64/27 206.73.210.96/27 206.73.227.64/26 206.73.228.24/29 206.73.246.32/28 206.125.174.16/29 206.126.99.120/29 206.168.117.160/28 206.182.199.208/29 206.182.200.128/26 206.182.221.112/28 206.182.221.192/26 206.196.232.0/24 206.221.217.232/30 206.221.218.32/29 206.221.220.8/29 206.221.221.64/29 206.221.221.176/29 206.221.222.208/29 206.222.26.32/29 206.226.64.48/29 206.226.64.160/29 206.226.64.176/29 206.226.64.208/28 206.226.64.240/29 206.226.65.0/29 206.226.65.16/29 206.226.65.48/29 206.226.65.80/29 206.226.65.112/28 206.226.65.128/29 206.226.65.152/29 206.226.65.160/29 206.226.65.200/29 206.226.65.216/29 206.226.66.0/29 206.226.66.40/29 206.226.66.48/29 206.226.66.136/29 206.226.66.144/29 206.226.67.88/29 206.226.67.96/29 206.226.67.152/29 206.226.67.168/29 206.226.67.176/29 206.226.67.224/29 206.226.68.24/29 206.226.68.32/29 206.226.68.104/29 206.226.68.112/29 206.226.68.176/29 206.226.69.80/29 206.226.69.104/29 206.226.69.128/29 206.226.69.240/29 206.226.70.0/29 206.226.70.208/28 206.226.70.248/29 206.226.71.64/29 206.226.71.88/29 206.226.71.120/29 206.226.71.128/29 206.226.71.200/29 206.226.71.216/29 206.226.72.48/29 206.226.72.104/29 206.226.72.152/29 206.226.72.160/29 206.226.72.232/29 206.226.73.0/24 206.226.74.96/29 206.226.74.120/29 206.226.74.144/28 206.226.74.200/29 206.226.75.32/28 206.226.75.56/29 206.226.75.64/29 206.226.75.200/29 206.226.75.208/29 206.226.76.0/28 206.226.76.16/29 206.226.76.48/29 206.226.76.96/29 206.226.76.136/29 206.226.76.168/29 206.226.76.248/29 206.226.77.24/29 206.226.77.32/28 206.226.77.64/28 206.226.77.104/29 206.226.77.240/29 206.226.78.0/29 206.226.78.72/29 206.226.78.120/29 206.226.78.160/29 206.226.78.192/29 206.226.78.208/29 206.226.78.224/29 206.226.79.8/29 206.226.79.48/29 206.226.79.160/29 206.226.79.224/29 206.226.80.32/28 206.226.80.200/29 206.226.80.208/29 206.226.81.32/29 206.226.81.56/29 206.226.81.64/29 206.226.81.136/29 206.226.81.160/29 206.226.82.0/29 206.226.82.48/29 206.226.82.80/29 206.226.82.120/29 206.226.82.248/29 206.226.83.8/29 206.226.83.24/29 206.226.83.32/29 206.226.83.56/29 206.226.83.96/29 206.226.83.168/29 206.226.83.200/29 206.226.84.16/29 206.226.84.48/28 206.226.84.64/29 206.226.84.80/29 206.226.85.72/29 206.226.85.80/29 206.226.85.104/29 206.226.85.176/29 206.226.85.224/28 206.226.86.24/29 206.226.86.32/29 206.226.87.40/29 206.226.87.80/29 206.226.88.16/29 206.226.88.56/29 206.226.88.64/29 206.226.88.128/29 206.226.88.160/28 206.226.88.200/29 206.226.88.232/29 206.226.89.8/29 206.226.89.16/29 206.226.89.32/29 206.226.89.72/29 206.226.89.96/29 206.226.89.120/29 206.226.89.152/29 206.226.89.176/28 206.226.89.208/29 206.226.89.240/29 206.226.90.56/29 206.226.90.104/29 206.226.90.136/29 206.226.90.216/29 206.226.91.0/24 206.226.92.96/29 206.226.92.168/29 206.226.92.224/29 206.226.93.8/29 206.226.93.56/29 206.226.93.128/29 206.226.93.176/29 206.226.93.200/29 206.226.93.224/28 206.226.93.240/29 206.226.94.104/29 206.226.94.128/29 206.226.94.192/29 206.226.94.248/29 206.226.95.0/29 206.226.95.104/29 206.226.95.112/28 206.226.95.160/29 206.226.95.232/29 206.226.95.248/29 206.226.96.48/29 206.226.96.120/29 206.226.96.144/28 206.226.96.160/28 206.226.96.192/29 206.226.96.240/28 206.226.97.48/28 206.226.97.64/28 206.226.97.96/29 206.226.97.152/29 206.226.98.8/29 206.226.98.40/29 206.226.98.72/29 206.226.98.168/29 206.226.99.0/29 206.226.99.200/29 206.226.99.248/29 206.226.100.88/29 206.226.100.248/29 206.226.101.8/29 206.226.101.16/29 206.226.101.32/29 206.226.101.48/29 206.226.101.208/29 206.226.102.8/29 206.226.102.16/29 206.226.102.48/29 206.226.102.104/29 206.226.102.240/29 206.226.103.8/29 206.226.103.24/29 206.226.103.64/28 206.226.103.136/29 206.226.103.152/29 206.226.103.160/29 206.226.103.176/28 206.226.103.192/29 206.226.103.224/29 206.226.104.8/29 206.226.104.24/29 206.226.104.56/29 206.226.104.88/29 206.226.104.128/29 206.226.104.168/29 206.226.104.176/28 206.226.104.200/29 206.226.104.216/29 206.226.104.232/29 206.226.105.16/29 206.226.105.48/29 206.226.105.152/29 206.226.105.168/29 206.226.106.200/29 206.226.107.0/29 206.226.107.16/29 206.226.107.40/29 206.226.107.104/29 206.226.107.112/29 206.226.107.168/29 206.226.107.224/29 206.226.108.16/29 206.226.108.40/29 206.226.108.96/29 206.226.108.136/29 206.226.108.176/29 206.226.108.200/29 206.226.109.72/29 206.226.109.128/28 206.226.110.40/29 206.226.110.64/29 206.226.110.104/29 206.226.110.112/29 206.226.110.176/29 206.226.110.192/29 206.226.111.24/29 206.226.111.32/28 206.226.111.72/29 206.226.111.80/29 206.226.111.144/28 206.226.111.168/29 206.226.111.248/29 206.226.112.0/28 206.226.112.48/29 206.226.112.80/29 206.226.112.96/29 206.226.112.152/29 206.226.112.160/29 206.226.112.184/29 206.226.112.192/29 206.226.112.208/29 206.226.112.224/29 206.226.113.40/29 206.226.113.48/29 206.226.113.104/29 206.226.113.112/29 206.226.113.136/29 206.226.113.168/29 206.226.113.232/29 206.226.114.32/29 206.226.114.104/29 206.226.114.112/29 206.226.114.176/28 206.226.114.200/29 206.226.114.232/29 206.226.115.32/29 206.226.115.56/29 206.226.115.80/29 206.226.115.96/29 206.226.115.128/29 206.226.115.176/29 206.226.115.224/29 206.226.116.8/29 206.226.116.56/29 206.226.116.72/29 206.226.116.80/29 206.226.116.152/29 206.226.116.176/29 206.226.116.216/29 206.226.117.8/29 206.226.117.104/29 206.226.117.112/29 206.226.117.168/29 206.226.117.176/29 206.226.122.128/26 207.117.165.0/24 207.150.160.30/31 207.150.160.32/29 207.150.169.200/29 207.150.169.208/31 207.209.206.128/25 207.209.226.32/28 207.254.178.0/25 207.254.179.0/25 207.254.180.0/24 207.254.183.0/24 207.254.184.0/27 207.254.184.32/28 207.254.184.48/32 207.254.185.0/24 207.254.186.0/23 207.254.188.0/24 208.51.62.168/29 208.74.175.1/32 208.74.175.2/31 208.74.175.4/30 208.74.175.8/29 208.74.175.16/28 208.79.90.96/29 208.115.104.72/29 208.115.104.80/29 208.115.106.8/29 208.115.106.64/28 208.115.107.64/29 208.115.108.32/29 208.115.109.64/28 208.115.109.88/29 208.115.109.96/28 208.115.109.144/28 208.115.109.160/27 208.115.112.80/28 208.115.113.96/29 208.115.113.120/29 208.115.114.40/29 208.115.124.0/29 208.115.124.16/28 208.115.124.64/26 208.115.124.128/28 208.115.125.32/28 208.115.125.136/29 208.115.125.160/27 208.115.125.192/27 208.115.140.0/24 209.93.110.0/25 209.93.112.128/25 209.93.114.128/26 209.93.178.0/25 209.93.178.128/27 209.93.178.160/32 209.93.184.192/26 209.93.189.0/24 209.93.190.0/23 209.93.194.64/26 209.93.236.104/29 209.93.238.128/25 209.93.240.128/25 209.239.113.131/32 209.239.119.244/32 210.2.0.0/19 210.5.0.0/19 210.5.56.0/21 210.5.128.0/19 210.12.0.0/15 210.14.64.0/19 210.14.112.0/20 210.14.128.0/17 210.15.0.0/17 210.15.128.0/18 210.16.128.0/18 210.21.0.0/16 210.22.0.0/16 210.23.32.0/19 210.25.0.0/16 210.26.0.0/15 210.28.0.0/15 210.30.0.0/16 210.31.0.0/17 210.31.128.0/18 210.31.192.0/20 210.31.208.0/23 210.31.210.0/30 210.31.210.8/29 210.31.210.16/28 210.31.210.32/27 210.31.210.64/26 210.31.210.128/25 210.31.211.0/24 210.31.212.0/22 210.31.216.0/21 210.31.224.0/19 210.32.0.0/12 210.48.136.0/21 210.51.0.0/16 210.52.0.0/15 210.56.192.0/19 210.72.0.0/14 210.76.0.0/15 210.78.0.0/16 210.79.64.0/18 210.79.224.0/19 210.82.0.0/15 210.87.128.0/18 210.185.192.0/18 210.192.96.0/19 211.64.0.0/13 211.80.0.0/12 211.96.0.0/13 211.136.0.0/13 211.144.0.0/12 211.160.0.0/13 212.63.181.28/30 212.63.181.40/30 212.63.181.76/30 212.63.181.140/30 212.63.181.180/30 212.63.181.200/30 212.63.181.224/30 212.63.183.4/32 212.63.183.9/32 212.63.183.15/32 212.63.183.23/32 212.63.183.28/32 212.63.183.62/32 212.63.183.64/32 212.63.183.68/32 212.63.183.212/30 212.63.183.216/30 212.63.183.228/30 216.15.146.80/29 216.15.172.0/24 216.15.214.72/29 216.15.238.160/27 216.110.34.96/29 216.119.138.32/29 216.131.121.64/28 216.136.136.128/28 216.136.162.208/28 216.136.183.184/29 216.136.217.176/28 216.139.183.80/29 216.158.143.65/32 216.158.143.66/31 216.158.143.68/30 216.158.143.72/29 216.158.143.80/28 216.158.143.96/27 216.158.143.128/32 216.158.147.1/32 216.158.147.2/31 216.158.147.4/30 216.158.147.8/29 216.158.147.16/28 216.158.147.32/27 216.158.147.64/26 216.158.147.128/25 216.176.180.32/29 216.176.190.0/29 216.176.190.32/27 216.176.190.96/27 216.176.190.192/27 216.194.160.192/29 216.218.168.16/32 216.218.168.18/31 216.218.168.20/30 216.218.168.24/29 216.218.214.224/32 216.218.214.226/31 216.218.214.228/30 218.0.0.0/12 218.16.0.0/13 218.24.0.0/14 218.28.0.0/15 218.30.0.0/19 218.30.32.0/20 218.30.48.0/21 218.30.64.0/18 218.30.128.0/17 218.31.0.0/16 218.56.0.0/13 218.64.0.0/11 218.96.0.0/14 218.100.88.0/21 218.100.96.0/19 218.100.128.0/17 218.104.0.0/14 218.108.0.0/15 218.185.192.0/19 218.185.240.0/21 218.192.0.0/12 218.240.0.0/13 218.249.0.0/16 219.72.0.0/16 219.82.0.0/16 219.83.128.0/17 219.128.0.0/11 219.216.0.0/13 219.224.0.0/12 219.242.0.0/15 219.244.0.0/14 220.101.192.0/18 220.112.0.0/14 220.152.128.0/17 220.154.0.0/15 220.160.0.0/11 220.192.0.0/12 220.231.0.0/18 220.231.128.0/17 220.232.64.0/18 220.234.0.0/16 220.242.0.0/15 220.247.136.0/21 220.248.0.0/14 220.252.0.0/16 221.0.0.0/13 221.8.0.0/14 221.12.0.0/17 221.12.128.0/18 221.13.0.0/16 221.14.0.0/15 221.122.0.0/15 221.128.128.0/17 221.129.0.0/16 221.130.0.0/15 221.133.224.0/19 221.136.0.0/15 221.172.0.0/14 221.176.0.0/13 221.192.0.0/14 221.196.0.0/15 221.198.0.0/16 221.199.0.0/17 221.199.128.0/18 221.199.192.0/20 221.199.224.0/19 221.200.0.0/13 221.208.0.0/12 221.224.0.0/12 222.16.0.0/12 222.32.0.0/11 222.64.0.0/11 222.125.0.0/16 222.126.128.0/17 222.128.0.0/12 222.160.0.0/14 222.168.0.0/13 222.176.0.0/12 222.192.0.0/11 222.240.0.0/13 222.248.0.0/15 223.0.0.0/12 223.20.0.0/15 223.27.162.0/24 223.27.184.0/22 223.64.0.0/11 223.96.0.0/12 223.112.0.0/14 223.116.0.0/15 223.120.0.0/13 223.128.0.0/15 223.144.0.0/12 223.160.0.0/14 223.166.0.0/15 223.192.0.0/15 223.198.0.0/15 223.201.0.0/16 223.202.0.0/15 223.208.0.0/13 223.220.0.0/15 223.223.176.0/20 223.223.192.0/20 223.240.0.0/13 223.248.0.0/14 223.252.128.0/19 223.252.192.0/18 223.254.0.0/16 223.255.0.0/17 223.255.236.0/22 223.255.252.0/23');
        $this->db->insert($this->t('ip'), $data);
        $data = array('id_label'=>6, 'content'=>'217.113.81.128/29 93.93.72.169/32 92.39.117.128/25 84.246.66.0/24 83.229.124.28/30 83.229.116.192/29 83.229.116.184/29 83.229.110.40/29 83.229.107.104/29 83.229.107.72/29 83.229.107.48/29 83.229.107.32/29 83.229.106.200/29 83.229.106.176/29 83.229.106.104/29 83.229.106.80/29 83.229.106.56/32 93.93.97.152/30 93.93.97.176/29 217.113.81.64/26 217.113.76.0/24 216.236.202.0/28 216.236.201.64/26 216.236.200.0/27 216.226.69.192/27 216.201.81.160/28 209.162.178.40/31 209.162.178.32/29 209.162.178.28/30 209.162.178.26/31 209.28.214.0/23 197.214.0.0/18 154.66.220.0/22 93.93.97.216/30 93.93.97.184/30 83.229.106.52/30 83.229.106.16/29 78.138.49.240/29 78.138.49.168/29 78.138.49.152/29 78.138.49.64/29 78.138.45.48/29 63.228.185.200/29 57.84.80.0/20 46.231.243.0/24 46.231.240.0/23 46.36.203.124/31 46.36.203.122/31 46.36.203.121/32 41.203.128.0/19 41.190.228.0/22 41.138.32.0/19 41.78.116.0/22 78.138.53.0/29 78.138.53.16/29 83.229.106.8/29 83.229.89.248/29 83.229.89.200/29 83.229.89.104/29 83.229.67.168/29 83.229.67.104/29 83.229.66.16/29 83.229.48.56/29 83.229.29.232/29 83.229.13.240/29 83.229.6.0/29 81.199.131.0/24 80.78.31.200/29 80.78.19.200/29 78.138.59.80/29 78.138.54.128/29 27.111.203.40/29');
        $this->db->insert($this->t('ip'), $data);
        $data = array('id_label'=>7, 'content'=>'2.56.0.0/14 5.1.0.0/19 5.22.156.0/22 5.34.176.0/21 5.45.206.0/23 5.53.112.0/21 5.56.24.0/21 5.57.64.0/21 5.58.0.0/16 5.83.16.0/21 5.104.32.0/19 5.105.0.0/16 5.149.120.0/21 5.149.208.0/20 5.153.128.0/18 5.180.0.0/14 5.199.232.0/21 5.207.0.0/16 5.248.0.0/16 5.255.160.0/19 23.234.84.216/29 23.234.114.96/27 23.234.118.64/27 31.3.24.0/21 31.3.192.0/20 31.6.96.0/19 31.12.128.0/17 31.24.208.0/21 31.28.160.0/19 31.28.224.0/19 31.31.96.0/19 31.40.16.0/21 31.40.104.0/21 31.40.128.0/17 31.41.8.0/21 31.41.48.0/21 31.41.64.0/20 31.41.88.0/21 31.41.104.0/21 31.41.112.0/21 31.41.128.0/21 31.41.216.0/21 31.42.48.0/20 31.42.64.0/20 31.42.112.0/20 31.42.160.0/19 31.43.0.0/17 31.43.128.0/18 31.43.224.0/19 31.44.188.0/24 31.128.64.0/18 31.128.160.0/19 31.128.224.0/19 31.129.64.0/18 31.129.160.0/19 31.129.224.0/19 31.131.16.0/20 31.131.32.0/21 31.131.64.0/20 31.131.96.0/19 31.131.128.0/20 31.133.32.0/19 31.133.64.0/19 31.133.112.0/21 31.134.64.0/19 31.134.104.0/21 31.134.112.0/20 31.134.208.0/21 31.135.96.0/19 31.135.128.0/19 31.135.176.0/21 31.144.0.0/16 31.148.168.0/23 31.148.180.0/23 31.148.188.0/22 31.148.202.0/23 31.148.217.0/24 31.148.218.0/24 31.148.252.0/22 31.170.128.0/19 31.172.136.0/21 31.184.235.0/24 31.193.80.0/20 31.201.69.0/24 31.201.137.0/24 31.201.205.0/24 31.202.0.0/16 31.223.224.0/21 37.9.32.0/21 37.9.51.0/24 37.17.240.0/21 37.19.128.0/17 37.25.0.0/19 37.25.32.0/21 37.25.96.0/19 37.46.216.0/21 37.46.224.0/19 37.52.0.0/14 37.57.0.0/16 37.72.40.0/21 37.73.0.0/16 37.75.216.0/21 37.115.0.0/16 37.139.96.0/20 37.139.160.0/19 37.143.88.0/21 37.203.0.0/19 37.221.128.0/19 37.229.0.0/16 38.28.1.88/32 38.28.1.131/32 38.28.1.133/32 38.28.1.157/32 38.28.1.230/32 38.28.1.255/32 38.103.78.78/31 46.8.34.0/24 46.8.80.0/20 46.17.240.0/21 46.18.0.0/21 46.28.64.0/23 46.28.66.16/28 46.28.66.32/27 46.28.66.64/26 46.28.66.128/25 46.28.67.0/24 46.28.68.0/22 46.28.192.0/21 46.29.128.0/21 46.30.160.0/21 46.33.32.0/19 46.33.224.0/19 46.35.224.0/19 46.36.197.71/32 46.36.197.72/29 46.36.197.80/32 46.37.192.0/19 46.39.64.0/19 46.46.64.0/18 46.63.0.0/17 46.96.0.0/16 46.98.0.0/16 46.118.0.0/15 46.133.0.0/16 46.136.80.0/24 46.148.16.0/20 46.148.112.0/20 46.149.48.0/20 46.149.80.0/20 46.149.96.0/20 46.149.160.0/19 46.150.0.0/19 46.150.64.0/18 46.151.40.0/21 46.151.48.0/21 46.151.64.0/21 46.151.80.0/21 46.151.144.0/21 46.151.176.0/21 46.151.192.0/21 46.151.248.0/21 46.160.64.0/18 46.162.0.0/18 46.164.128.0/18 46.172.64.0/19 46.172.128.0/19 46.172.192.0/18 46.173.64.0/18 46.173.128.0/19 46.173.160.0/20 46.174.40.0/21 46.174.64.0/21 46.174.120.0/21 46.174.160.0/21 46.174.216.0/21 46.174.240.0/21 46.175.16.0/21 46.175.64.0/20 46.175.80.0/21 46.175.136.0/21 46.175.144.0/21 46.175.160.0/21 46.175.184.0/21 46.175.240.0/20 46.182.80.0/21 46.185.0.0/17 46.200.0.0/14 46.211.0.0/16 46.219.0.0/16 46.227.136.0/21 46.229.48.0/20 46.231.224.0/21 46.232.232.0/21 46.243.184.0/21 46.247.128.0/17 46.250.0.0/19 46.250.96.0/19 46.252.208.0/20 46.255.32.0/21 57.87.192.0/20 61.14.145.29/32 61.14.145.30/31 62.16.0.0/19 62.32.63.0/25 62.32.63.192/27 62.32.63.224/28 62.32.63.248/29 62.64.64.0/18 62.72.160.0/19 62.80.160.0/19 62.84.248.0/21 62.115.32.137/32 62.115.32.138/32 62.115.35.77/32 62.115.35.78/32 62.122.0.0/21 62.122.56.0/21 62.122.64.0/21 62.122.104.0/21 62.122.152.0/21 62.122.200.0/21 62.128.96.0/22 62.128.102.152/29 62.128.102.200/29 62.128.102.208/28 62.128.103.0/24 62.128.104.48/28 62.128.104.64/26 62.128.104.128/25 62.128.105.0/24 62.128.106.56/29 62.128.106.64/26 62.128.106.128/25 62.128.107.0/24 62.128.108.0/22 62.128.113.192/26 62.128.114.0/23 62.128.120.0/24 62.128.124.96/27 62.128.124.128/25 62.128.125.0/24 62.128.126.32/27 62.128.126.64/26 62.128.126.128/25 62.128.127.0/25 62.128.127.128/27 62.149.0.0/19 62.182.64.0/21 62.182.80.0/21 62.182.120.0/21 62.182.152.0/21 62.182.160.0/21 62.187.208.0/24 62.205.128.0/19 62.216.32.0/19 62.221.32.0/19 62.244.0.0/18 64.38.233.64/28 64.69.66.96/27 64.106.158.40/29 64.106.158.48/31 64.106.201.140/30 64.106.201.144/28 64.106.202.200/29 64.106.202.208/31 64.225.123.16/28 64.225.123.32/27 64.225.123.64/27 65.110.37.130/31 65.110.37.132/30 65.110.37.136/30 65.110.39.180/30 65.110.39.184/30 65.110.39.188/31 65.110.51.160/29 65.110.51.168/31 65.110.57.120/29 65.110.57.128/31 65.110.63.130/31 65.110.63.132/30 65.110.63.136/30 66.111.39.230/31 66.111.39.232/29 66.111.41.50/31 66.111.41.52/30 66.111.41.56/30 66.111.42.70/31 66.111.42.72/29 70.252.247.32/27 74.117.181.64/26 74.117.181.128/26 74.127.26.248/29 76.9.27.0/24 77.47.128.0/17 77.52.0.0/16 77.75.144.0/21 77.87.32.0/20 77.87.144.0/20 77.87.192.0/21 77.88.192.0/18 77.90.192.0/18 77.91.128.0/18 77.93.32.0/19 77.109.0.0/18 77.120.0.0/14 77.222.128.0/19 77.239.160.0/19 77.242.160.0/20 77.244.32.0/20 77.247.16.0/20 77.247.216.0/21 78.24.72.0/21 78.24.204.96/28 78.25.0.0/18 78.26.128.0/17 78.27.128.0/17 78.30.192.0/18 78.31.176.0/20 78.31.192.0/21 78.31.232.0/21 78.31.248.0/21 78.109.16.0/24 78.109.17.0/28 78.109.17.24/29 78.109.17.32/27 78.109.17.64/27 78.109.17.104/29 78.109.17.112/28 78.109.17.128/25 78.109.18.0/26 78.109.18.64/27 78.109.18.104/29 78.109.18.120/29 78.109.18.128/25 78.109.19.0/26 78.109.19.72/29 78.109.19.80/29 78.109.19.96/28 78.109.19.128/29 78.109.19.144/28 78.109.19.160/27 78.109.19.192/26 78.109.20.0/28 78.109.20.16/29 78.109.20.32/27 78.109.20.64/26 78.109.20.128/26 78.109.20.192/27 78.109.20.224/28 78.109.20.240/29 78.109.21.0/26 78.109.21.64/27 78.109.21.96/28 78.109.21.120/29 78.109.21.128/27 78.109.21.160/29 78.109.21.176/28 78.109.21.192/27 78.109.21.232/29 78.109.21.240/28 78.109.22.0/27 78.109.22.48/29 78.109.22.64/26 78.109.22.136/29 78.109.22.144/28 78.109.22.160/27 78.109.22.192/26 78.109.23.0/24 78.109.24.0/22 78.109.28.0/23 78.109.30.0/25 78.109.30.128/27 78.109.30.160/28 78.109.30.176/29 78.109.30.192/26 78.109.31.0/24 78.111.16.0/20 78.111.176.0/20 78.111.208.0/20 78.137.0.0/18 78.152.160.0/19 78.154.160.0/19 78.159.32.0/19 79.110.16.0/20 79.110.48.0/20 79.110.96.0/20 79.110.128.0/20 79.110.208.0/20 79.110.224.0/20 79.124.128.0/17 79.135.192.0/19 79.140.0.0/20 79.142.192.0/20 79.143.32.0/20 79.171.120.0/21 80.64.80.0/20 80.70.64.0/19 80.73.0.0/20 80.77.32.0/20 80.78.32.0/19 80.82.198.0/23 80.84.176.0/20 80.90.224.0/20 80.91.160.0/19 80.91.247.59/32 80.91.247.179/32 80.91.248.65/32 80.91.248.66/32 80.91.249.222/31 80.91.255.24/32 80.91.255.29/32 80.92.224.0/20 80.93.112.0/20 80.239.196.58/32 80.242.96.0/20 80.243.144.0/20 80.245.112.0/20 80.249.224.0/20 80.252.240.0/20 80.254.0.0/20 80.255.64.0/20 81.17.128.0/20 81.21.0.0/20 81.22.128.0/20 81.23.16.0/20 81.24.208.0/20 81.25.224.0/20 81.30.160.0/20 81.85.14.0/24 81.90.224.0/20 81.95.176.0/20 81.162.56.0/21 81.162.64.0/20 81.162.96.0/19 81.162.216.0/21 81.162.224.0/19 81.163.88.0/21 81.163.112.0/20 81.163.128.0/19 81.163.208.0/20 82.117.224.0/19 82.118.16.0/23 82.118.18.0/24 82.118.19.0/25 82.118.19.128/26 82.118.19.192/27 82.118.19.224/28 82.118.20.0/24 82.118.21.0/25 82.118.21.128/28 82.118.21.160/27 82.118.21.192/26 82.118.22.0/23 82.119.245.198/32 82.144.192.0/19 82.193.96.0/19 82.207.0.0/17 83.137.88.0/21 83.142.104.0/21 83.142.208.0/21 83.142.232.0/21 83.143.200.0/21 83.143.232.0/21 83.170.192.0/18 83.218.224.0/19 83.242.96.0/20 84.201.128.0/24 84.201.144.0/24 84.201.145.0/27 84.201.145.32/29 84.201.145.128/27 84.201.145.164/30 84.201.145.168/29 84.201.145.176/28 84.201.145.192/26 84.233.193.125/32 84.234.96.0/20 84.242.48.0/20 85.90.192.0/19 85.114.192.0/19 85.159.0.0/21 85.198.128.0/18 85.202.160.0/20 85.223.128.0/17 85.238.96.0/19 85.255.176.0/21 86.106.83.0/24 86.111.64.0/19 86.111.224.0/21 87.76.64.0/18 87.76.128.0/17 87.238.152.0/21 87.244.128.0/18 87.245.216.0/25 87.245.216.128/32 87.245.216.240/28 87.245.217.0/29 87.245.217.16/29 87.245.218.0/23 87.245.220.64/26 87.245.220.128/25 87.245.221.0/24 87.245.222.0/23 87.245.224.101/32 87.245.224.102/32 87.245.224.111/32 87.245.224.115/32 87.245.224.123/32 87.245.224.125/32 87.245.224.130/32 87.245.225.101/32 87.245.225.102/32 87.245.225.111/32 87.245.225.115/32 87.245.225.123/32 87.245.225.125/32 87.245.225.130/32 87.245.232.37/32 87.245.232.85/32 87.245.232.89/32 87.245.232.105/32 87.245.232.109/32 87.245.232.110/32 87.245.232.122/32 87.245.232.129/32 87.245.232.162/32 87.245.232.165/32 87.245.232.166/32 87.245.232.190/32 87.245.232.193/32 87.245.232.194/32 87.245.232.209/32 87.245.232.210/32 87.245.232.214/32 87.245.232.217/32 87.245.232.249/32 87.245.233.26/32 87.245.233.57/32 87.245.233.81/32 87.245.233.82/32 87.245.233.85/32 87.245.233.106/32 87.245.233.122/32 87.245.233.129/32 87.245.233.137/32 87.245.233.142/32 87.245.233.193/32 87.245.233.194/32 87.245.233.197/32 87.245.233.210/32 87.245.233.213/32 87.245.233.214/32 87.245.235.1/32 87.245.235.2/31 87.245.235.9/32 87.245.235.10/32 87.245.237.0/29 87.245.237.9/32 87.245.237.16/29 87.245.237.25/32 87.245.237.32/29 87.245.237.41/32 87.245.237.48/28 87.245.237.64/26 87.245.237.129/32 87.245.237.136/29 87.245.237.144/28 87.245.237.160/27 87.245.237.192/28 87.245.237.209/32 87.245.237.216/29 87.245.237.224/27 87.245.239.0/28 87.245.239.16/29 87.245.239.25/32 87.245.239.32/28 87.245.239.49/32 87.245.239.56/29 87.245.239.64/29 87.245.239.73/32 87.245.239.80/29 87.245.239.89/32 87.245.239.96/27 87.245.239.128/29 87.245.239.136/32 87.245.239.138/31 87.245.239.140/32 87.245.239.142/31 87.245.239.160/32 87.245.239.162/31 87.245.239.164/30 87.245.239.168/29 87.245.239.176/28 87.245.239.192/26 87.245.243.128/26 87.245.243.192/27 87.245.243.224/29 87.245.243.233/32 87.245.243.249/32 87.245.247.0/27 87.245.247.32/28 87.245.247.48/29 87.245.247.57/32 87.245.247.58/32 87.245.247.64/26 87.245.247.129/32 87.245.247.136/29 87.245.247.145/32 87.245.247.152/29 87.245.247.169/32 87.245.247.176/28 87.245.247.192/29 87.245.247.201/32 87.245.247.208/28 87.245.247.224/30 87.245.247.228/32 87.245.247.230/31 87.245.247.232/32 87.245.247.234/31 87.245.247.236/32 87.245.247.238/31 87.245.247.248/32 87.245.247.250/31 87.245.247.252/32 87.245.247.254/31 88.81.224.0/19 88.135.16.0/20 88.135.64.0/19 88.135.112.0/20 88.135.192.0/18 88.154.0.0/15 88.202.107.112/28 88.202.111.112/28 88.214.64.0/18 89.19.96.0/19 89.21.64.0/19 89.22.40.0/21 89.22.200.0/21 89.22.240.0/20 89.28.200.0/21 89.105.224.0/19 89.162.128.0/17 89.184.64.0/19 89.185.0.0/19 89.187.0.0/19 89.200.232.0/21 89.200.248.0/21 89.209.0.0/16 89.251.16.0/20 89.252.0.0/18 91.90.8.0/21 91.90.16.0/21 91.90.192.0/19 91.102.176.0/21 91.103.120.0/21 91.105.204.0/22 91.123.144.0/20 91.124.0.0/16 91.142.160.0/20 91.145.192.0/18 91.189.128.0/21 91.189.152.0/21 91.192.44.0/22 91.192.104.0/23 91.192.128.0/21 91.192.136.0/22 91.192.152.0/21 91.192.160.0/22 91.192.180.0/22 91.192.184.0/22 91.192.200.0/23 91.192.216.0/22 91.193.8.0/22 91.193.32.0/22 91.193.68.0/22 91.193.76.0/22 91.193.80.0/22 91.193.124.0/22 91.193.128.0/22 91.193.164.0/22 91.193.172.0/22 91.193.204.0/22 91.193.232.0/22 91.193.252.0/22 91.194.34.0/23 91.194.40.0/23 91.194.50.0/23 91.194.56.0/23 91.194.72.0/23 91.194.78.0/23 91.194.80.0/23 91.194.88.0/23 91.194.134.0/23 91.194.162.0/23 91.194.168.0/23 91.194.238.0/23 91.194.250.0/23 91.195.2.0/23 91.195.10.0/23 91.195.12.0/23 91.195.20.0/23 91.195.52.0/23 91.195.68.0/23 91.195.74.0/23 91.195.86.0/23 91.195.90.0/23 91.195.96.0/23 91.195.120.0/23 91.195.156.0/23 91.195.172.0/23 91.195.184.0/23 91.195.214.0/23 91.195.230.0/23 91.195.244.0/23 91.195.248.0/23 91.196.0.0/22 91.196.52.0/22 91.196.60.0/22 91.196.80.0/22 91.196.88.0/21 91.196.96.0/21 91.196.120.0/22 91.196.132.0/22 91.196.148.0/22 91.196.156.0/22 91.196.160.0/21 91.196.176.0/22 91.196.192.0/21 91.196.228.0/22 91.197.4.0/22 91.197.16.0/22 91.197.24.0/22 91.197.48.0/22 91.197.56.0/22 91.197.80.0/22 91.197.128.0/21 91.197.144.0/22 91.197.156.0/22 91.197.168.0/22 91.197.184.0/22 91.197.216.0/21 91.197.236.0/22 91.197.252.0/22 91.198.1.0/24 91.198.4.0/24 91.198.10.0/24 91.198.31.0/24 91.198.34.0/24 91.198.36.0/24 91.198.40.0/24 91.198.50.0/24 91.198.83.0/24 91.198.101.0/24 91.198.116.0/24 91.198.133.0/24 91.198.140.0/24 91.198.143.0/24 91.198.153.0/24 91.198.175.0/24 91.198.188.0/24 91.198.215.0/24 91.198.233.0/24 91.198.235.0/24 91.198.247.0/24 91.198.249.0/24 91.199.13.0/24 91.199.28.0/24 91.199.33.0/24 91.199.35.0/24 91.199.37.0/24 91.199.54.0/24 91.199.91.0/24 91.199.92.0/23 91.199.106.0/24 91.199.112.0/24 91.199.115.0/24 91.199.138.0/23 91.199.143.0/24 91.199.144.0/24 91.199.182.0/24 91.199.188.0/24 91.199.194.0/24 91.199.206.0/24 91.199.222.0/24 91.199.245.0/24 91.200.0.0/20 91.200.40.0/21 91.200.52.0/22 91.200.56.0/22 91.200.60.0/24 91.200.72.0/22 91.200.80.0/22 91.200.112.0/22 91.200.124.0/22 91.200.136.0/22 91.200.156.0/22 91.200.160.0/22 91.200.180.0/22 91.200.200.0/22 91.200.212.0/22 91.200.220.0/22 91.200.232.0/22 91.200.244.0/22 91.200.248.0/22 91.201.24.0/22 91.201.36.0/22 91.201.40.0/22 91.201.68.0/22 91.201.84.0/22 91.201.108.0/22 91.201.144.0/22 91.201.156.0/22 91.201.168.0/22 91.201.176.0/21 91.201.188.0/22 91.201.202.0/23 91.201.224.0/22 91.201.232.0/21 91.201.240.0/21 91.202.0.0/22 91.202.8.0/22 91.202.56.0/22 91.202.72.0/22 91.202.104.0/21 91.202.128.0/21 91.202.144.0/22 91.202.160.0/22 91.202.208.0/21 91.202.240.0/22 91.203.4.0/22 91.203.12.0/22 91.203.24.0/22 91.203.48.0/22 91.203.60.0/22 91.203.88.0/22 91.203.107.0/24 91.203.112.0/22 91.203.136.0/21 91.203.144.0/22 91.203.164.0/22 91.204.12.0/22 91.204.36.0/22 91.204.60.0/22 91.204.76.0/22 91.204.84.0/22 91.204.92.0/22 91.204.112.0/22 91.204.120.0/22 91.204.132.0/22 91.204.180.0/22 91.204.196.0/22 91.204.212.0/22 91.204.248.0/22 91.205.4.0/22 91.205.16.0/22 91.205.64.0/22 91.205.156.0/22 91.205.164.0/22 91.205.204.0/22 91.205.216.0/22 91.206.4.0/23 91.206.30.0/23 91.206.32.0/23 91.206.48.0/23 91.206.110.0/23 91.206.124.0/23 91.206.186.0/23 91.206.200.0/23 91.206.212.0/23 91.206.218.0/23 91.206.226.0/23 91.206.252.0/23 91.207.4.0/22 91.207.8.0/23 91.207.26.0/23 91.207.44.0/22 91.207.54.0/23 91.207.66.0/23 91.207.88.0/23 91.207.98.0/23 91.207.104.0/22 91.207.116.0/23 91.207.122.0/23 91.207.146.0/23 91.207.182.0/23 91.207.194.0/23 91.207.204.0/23 91.207.210.0/23 91.207.218.0/23 91.207.224.0/23 91.207.244.0/23 91.208.25.0/24 91.208.52.0/24 91.208.59.0/24 91.208.65.0/24 91.208.94.0/24 91.208.97.0/24 91.208.115.0/24 91.208.116.0/24 91.208.127.0/24 91.208.138.0/24 91.208.145.0/24 91.208.153.0/24 91.208.154.0/24 91.208.194.0/24 91.208.198.0/24 91.208.231.0/24 91.208.251.0/24 91.209.1.0/24 91.209.11.0/24 91.209.24.0/24 91.209.36.0/24 91.209.51.0/24 91.209.54.0/23 91.209.64.0/24 91.209.90.0/24 91.209.96.0/24 91.209.118.0/24 91.209.126.0/24 91.209.138.0/23 91.209.157.0/24 91.209.176.0/24 91.209.193.0/24 91.209.201.0/24 91.209.202.0/23 91.209.206.0/24 91.209.210.0/24 91.209.214.0/24 91.209.226.0/24 91.209.235.0/24 91.209.239.0/24 91.209.249.0/24 91.210.8.0/22 91.210.20.0/22 91.210.28.0/22 91.210.32.0/21 91.210.76.0/22 91.210.100.0/22 91.210.108.0/22 91.210.116.0/22 91.210.120.0/22 91.210.144.0/21 91.210.156.0/22 91.210.164.0/22 91.210.176.0/22 91.210.188.0/22 91.210.248.0/22 91.211.12.0/22 91.211.16.0/22 91.211.68.0/22 91.211.116.0/22 91.211.120.0/22 91.211.132.0/22 91.211.172.0/22 91.211.176.0/22 91.211.204.0/22 91.211.212.0/22 91.212.22.0/24 91.212.34.0/24 91.212.56.0/24 91.212.80.0/24 91.212.124.0/23 91.212.153.0/24 91.212.154.0/24 91.212.173.0/24 91.212.174.0/24 91.212.177.0/24 91.212.194.0/24 91.212.203.0/24 91.212.214.0/24 91.212.248.0/24 91.212.253.0/24 91.213.8.0/24 91.213.33.0/24 91.213.59.0/24 91.213.62.0/24 91.213.74.0/24 91.213.92.0/24 91.213.94.0/24 91.213.119.0/24 91.213.121.0/24 91.213.125.0/24 91.213.149.0/24 91.213.154.0/24 91.213.175.0/24 91.213.182.0/24 91.213.185.0/24 91.213.187.0/24 91.213.192.0/24 91.213.209.0/24 91.213.231.0/24 91.214.16.0/22 91.214.28.0/22 91.214.48.0/22 91.214.80.0/21 91.214.112.0/22 91.214.124.0/22 91.214.128.0/21 91.214.136.0/22 91.214.160.0/22 91.214.176.0/21 91.214.208.0/22 91.214.232.0/22 91.214.244.0/22 91.214.248.0/22 91.215.8.0/22 91.215.24.0/22 91.215.52.0/22 91.215.56.0/21 91.215.68.0/22 91.215.124.0/22 91.215.136.0/22 91.215.144.0/22 91.215.248.0/22 91.216.15.0/24 91.216.22.0/24 91.216.28.0/24 91.216.61.0/24 91.216.94.0/24 91.216.106.0/24 91.216.133.0/24 91.216.135.0/24 91.216.161.0/24 91.216.165.0/24 91.216.173.0/24 91.216.196.0/24 91.216.231.0/24 91.216.232.0/24 91.216.237.0/24 91.216.240.0/24 91.216.244.0/24 91.216.251.0/24 91.217.4.0/23 91.217.8.0/22 91.217.12.0/23 91.217.58.0/23 91.217.60.0/23 91.217.66.0/23 91.217.68.0/23 91.217.78.0/23 91.217.100.0/23 91.217.106.0/23 91.217.112.0/23 91.217.144.0/24 91.217.162.0/24 91.217.166.0/24 91.217.179.0/24 91.217.183.0/24 91.217.197.0/24 91.217.240.0/24 91.217.244.0/24 91.217.253.0/24 91.217.254.0/24 91.218.12.0/22 91.218.16.0/22 91.218.28.0/22 91.218.32.0/21 91.218.44.0/22 91.218.48.0/22 91.218.64.0/22 91.218.72.0/22 91.218.88.0/22 91.218.96.0/22 91.218.104.0/22 91.218.192.0/22 91.218.212.0/22 91.219.28.0/22 91.219.32.0/22 91.219.52.0/22 91.219.60.0/22 91.219.80.0/22 91.219.140.0/22 91.219.144.0/22 91.219.168.0/22 91.219.196.0/22 91.219.220.0/22 91.219.232.0/22 91.219.252.0/22 91.220.3.0/24 91.220.6.0/24 91.220.60.0/23 91.220.80.0/24 91.220.91.0/24 91.220.104.0/24 91.220.106.0/24 91.220.113.0/24 91.220.140.0/24 91.220.151.0/24 91.220.161.0/24 91.220.163.0/24 91.220.174.0/24 91.220.190.0/24 91.220.198.0/24 91.220.204.0/24 91.220.230.0/24 91.220.235.0/24 91.220.252.0/23 91.220.255.0/24 91.221.6.0/23 91.221.14.0/23 91.221.52.0/23 91.221.64.0/23 91.221.74.0/23 91.221.84.0/23 91.221.124.0/23 91.221.132.0/23 91.221.136.0/23 91.221.154.0/23 91.221.166.0/23 91.221.178.0/23 91.221.190.0/23 91.221.210.0/23 91.221.218.0/23 91.221.234.0/23 91.221.252.0/23 91.222.0.0/22 91.222.12.0/22 91.222.16.0/22 91.222.32.0/21 91.222.40.0/22 91.222.48.0/22 91.222.60.0/22 91.222.64.0/22 91.222.76.0/22 91.222.80.0/22 91.222.112.0/22 91.222.136.0/22 91.222.152.0/22 91.222.168.0/21 91.222.180.0/22 91.222.220.0/22 91.222.244.0/22 91.222.248.0/21 91.223.19.0/24 91.223.27.0/24 91.223.30.0/24 91.223.48.0/24 91.223.53.0/24 91.223.59.0/24 91.223.77.0/24 91.223.86.0/24 91.223.88.0/24 91.223.97.0/24 91.223.102.0/23 91.223.110.0/24 91.223.112.0/24 91.223.122.0/24 91.223.127.0/24 91.223.133.0/24 91.223.136.0/24 91.223.149.0/24 91.223.152.0/24 91.223.177.0/24 91.223.180.0/24 91.223.205.0/24 91.223.214.0/24 91.223.216.0/24 91.223.223.0/24 91.223.227.0/24 91.223.234.0/24 91.224.2.0/23 91.224.8.0/22 91.224.16.0/23 91.224.24.0/23 91.224.30.0/23 91.224.44.0/23 91.224.68.0/23 91.224.74.0/23 91.224.78.0/23 91.224.84.0/22 91.224.96.0/23 91.224.112.0/23 91.224.156.0/23 91.224.168.0/23 91.224.178.0/23 91.224.200.0/22 91.224.222.0/23 91.224.252.0/22 91.225.4.0/22 91.225.36.0/22 91.225.40.0/21 91.225.48.0/22 91.225.56.0/22 91.225.72.0/22 91.225.80.0/22 91.225.88.0/21 91.225.108.0/22 91.225.116.0/22 91.225.120.0/22 91.225.136.0/21 91.225.144.0/22 91.225.152.0/22 91.225.160.0/20 91.225.196.0/22 91.225.200.0/22 91.225.220.0/22 91.225.224.0/21 91.225.232.0/22 91.226.4.0/23 91.226.14.0/23 91.226.34.0/23 91.226.42.0/23 91.226.44.0/23 91.226.56.0/22 91.226.72.0/24 91.226.132.0/22 91.226.138.0/23 91.226.187.0/24 91.226.190.0/23 91.226.192.0/24 91.226.201.0/24 91.226.207.0/24 91.226.252.0/22 91.227.42.0/24 91.227.69.0/24 91.227.150.0/23 91.227.180.0/22 91.227.206.0/23 91.227.254.0/24 91.228.12.0/22 91.228.58.0/23 91.228.68.0/22 91.228.122.0/23 91.228.146.0/23 91.228.228.0/23 91.228.236.0/24 91.229.52.0/22 91.229.66.0/23 91.229.76.0/22 91.229.101.0/24 91.229.115.0/24 91.229.120.0/22 91.229.128.0/23 91.229.167.0/24 91.229.204.0/22 91.229.216.0/22 91.229.226.0/24 91.229.240.0/22 91.230.25.0/24 91.230.28.0/22 91.230.92.0/23 91.230.121.0/24 91.230.198.0/23 91.230.220.0/24 91.231.1.0/24 91.231.40.0/22 91.231.54.0/23 91.231.69.0/24 91.231.84.0/22 91.231.98.0/23 91.231.127.0/24 91.231.142.0/23 91.231.148.0/22 91.231.160.0/24 91.231.170.0/24 91.231.185.0/24 91.231.206.0/24 91.231.229.0/24 91.231.240.0/24 91.232.0.0/22 91.232.21.0/24 91.232.28.0/22 91.232.99.0/24 91.232.103.0/24 91.232.116.0/22 91.232.127.0/24 91.232.128.0/24 91.232.157.0/24 91.232.158.0/23 91.232.177.0/24 91.232.178.0/23 91.232.199.0/24 91.232.208.0/24 91.232.238.0/24 91.232.240.0/23 91.233.17.0/24 91.233.48.0/23 91.233.90.0/23 91.233.96.0/21 91.233.108.0/22 91.233.123.0/24 91.233.127.0/24 91.233.140.0/24 91.233.151.0/24 91.233.172.0/23 91.234.0.0/21 91.234.20.0/24 91.234.24.0/22 91.234.28.0/23 91.234.31.0/24 91.234.32.0/22 91.234.37.0/24 91.234.40.0/22 91.234.47.0/24 91.234.72.0/21 91.234.84.0/22 91.234.88.0/22 91.234.98.0/23 91.234.136.0/22 91.234.140.0/23 91.234.196.0/22 91.234.208.0/22 91.234.226.0/23 91.235.12.0/22 91.235.46.0/24 91.235.67.0/24 91.235.68.0/22 91.235.84.0/22 91.235.102.0/24 91.235.128.0/23 91.235.142.0/23 91.235.182.0/24 91.235.196.0/22 91.235.224.0/22 91.236.36.0/24 91.236.96.0/22 91.236.114.0/24 91.236.118.0/23 91.236.126.0/23 91.236.134.0/23 91.236.138.0/23 91.236.156.0/22 91.236.171.0/24 91.236.213.0/24 91.236.221.0/24 91.236.224.0/22 91.236.248.0/22 91.237.0.0/20 91.237.24.0/22 91.237.40.0/23 91.237.64.0/24 91.237.107.0/24 91.237.108.0/22 91.237.114.0/23 91.237.116.0/24 91.237.119.0/24 91.237.120.0/21 91.237.128.0/23 91.237.142.0/24 91.237.152.0/24 91.237.164.0/22 91.237.182.0/23 91.237.204.0/22 91.237.208.0/24 91.237.232.0/22 91.237.240.0/21 91.237.250.0/24 91.238.4.0/23 91.238.9.0/24 91.238.16.0/23 91.238.23.0/24 91.238.74.0/23 91.238.100.0/22 91.238.104.0/23 91.238.132.0/24 91.238.146.0/23 91.238.192.0/22 91.239.48.0/22 91.239.74.0/24 91.239.76.0/22 91.239.80.0/22 91.239.95.0/24 91.239.104.0/22 91.239.136.0/22 91.239.150.0/24 91.239.160.0/22 91.239.187.0/24 91.239.190.0/24 91.239.232.0/22 91.239.241.0/24 91.240.18.0/23 91.240.20.0/22 91.240.48.0/22 91.240.90.0/23 91.240.96.0/22 91.240.140.0/22 91.240.145.0/24 91.240.175.0/24 91.240.190.0/23 91.240.244.0/24 91.241.71.0/24 91.241.80.0/22 91.241.93.0/24 91.241.96.0/19 91.241.128.0/17 91.242.52.0/22 91.242.192.0/21 91.242.208.0/22 91.242.222.0/24 91.242.224.0/19 91.243.0.0/19 91.243.192.0/19 91.244.0.0/18 91.244.69.0/24 91.244.118.0/23 91.244.124.0/24 91.244.127.0/24 91.244.128.0/19 91.244.232.0/22 91.245.72.0/21 91.245.96.0/19 91.245.213.0/24 91.245.216.0/23 91.245.221.0/24 91.245.223.0/24 91.245.224.0/19 91.246.4.0/22 91.246.32.0/19 91.246.128.0/19 91.246.220.0/22 91.246.224.0/19 91.247.32.0/19 91.247.76.0/22 91.247.80.0/20 91.247.160.0/19 91.247.221.0/24 91.247.224.0/22 91.250.0.0/18 92.49.192.0/18 92.52.128.0/18 92.60.176.0/20 92.112.0.0/15 92.240.96.0/19 92.242.96.0/19 92.244.96.0/19 92.249.64.0/18 93.72.0.0/13 93.88.48.0/20 93.89.208.0/20 93.92.43.0/24 93.95.184.0/21 93.126.64.0/18 93.127.0.0/17 93.157.8.0/21 93.157.24.0/21 93.157.104.0/21 93.170.0.0/23 93.170.4.0/24 93.170.25.0/24 93.170.28.0/22 93.170.32.0/21 93.170.48.0/22 93.170.54.0/23 93.170.64.0/21 93.170.72.0/22 93.170.80.0/23 93.170.83.0/24 93.170.112.0/22 93.170.132.0/23 93.170.136.0/23 93.170.144.0/20 93.170.188.0/23 93.171.64.0/20 93.171.128.0/22 93.171.132.0/23 93.171.137.0/24 93.171.138.0/24 93.171.170.0/23 93.171.185.0/24 93.171.205.0/24 93.171.209.0/24 93.171.218.0/24 93.171.225.0/24 93.171.240.0/20 93.175.192.0/18 93.178.192.0/18 93.179.66.0/24 93.180.192.0/18 93.183.192.0/18 93.185.208.0/20 93.188.32.0/21 93.190.40.0/21 93.190.176.0/21 94.27.0.0/17 94.45.32.0/19 94.45.64.0/18 94.45.128.0/19 94.74.64.0/18 94.76.64.0/18 94.100.208.0/20 94.124.160.0/22 94.124.164.0/24 94.124.166.0/23 94.125.120.0/21 94.130.0.0/16 94.153.0.0/16 94.154.32.0/20 94.154.112.0/20 94.154.192.0/18 94.158.32.0/20 94.158.64.0/19 94.158.144.0/20 94.158.240.0/20 94.178.0.0/15 94.199.160.0/21 94.230.192.0/20 94.231.32.0/20 94.231.64.0/20 94.231.176.0/20 94.232.72.0/21 94.232.176.0/21 94.232.208.0/21 94.240.128.0/18 94.244.0.0/18 94.244.128.0/18 94.247.224.0/21 94.248.0.0/17 95.46.144.0/21 95.47.104.0/21 95.47.112.0/22 95.47.116.0/24 95.47.124.0/22 95.47.136.0/23 95.47.142.0/23 95.47.145.0/24 95.47.148.0/22 95.47.156.0/24 95.47.162.0/23 95.47.167.0/24 95.47.190.0/24 95.47.204.0/22 95.67.0.0/17 95.69.128.0/17 95.81.0.0/18 95.109.128.0/17 95.111.128.0/17 95.132.0.0/14 95.141.27.0/24 95.158.0.0/18 95.164.0.0/16 95.169.0.0/19 95.182.79.0/24 95.215.4.0/22 95.215.36.0/22 95.215.40.0/22 95.215.48.0/22 95.215.92.0/22 95.215.104.0/22 95.215.112.0/21 95.215.124.0/22 95.215.144.0/22 95.215.156.0/22 95.215.164.0/22 95.215.216.0/22 95.215.223.0/24 95.216.0.0/15 109.68.40.0/21 109.70.104.0/21 109.72.112.0/20 109.72.152.0/22 109.75.192.0/20 109.86.0.0/15 109.94.160.0/20 109.94.208.0/20 109.95.32.0/20 109.95.48.0/21 109.106.0.0/19 109.108.64.0/19 109.108.224.0/19 109.110.64.0/19 109.122.0.0/18 109.162.0.0/17 109.167.0.0/17 109.196.160.0/19 109.197.166.0/23 109.197.216.0/21 109.200.128.0/19 109.200.224.0/19 109.201.224.0/19 109.206.32.0/19 109.207.64.0/20 109.207.112.0/20 109.207.192.0/20 109.227.64.0/18 109.229.0.0/19 109.235.8.0/21 109.237.80.0/20 109.251.0.0/16 109.254.0.0/16 128.0.104.0/23 128.0.168.0/21 128.124.0.0/16 130.0.32.0/19 130.0.232.0/21 130.117.3.70/32 130.117.24.8/29 130.117.24.16/28 130.117.24.32/28 130.117.24.64/28 130.117.24.88/29 130.117.24.96/28 130.117.24.129/32 130.117.24.168/29 130.117.24.177/32 130.117.24.184/29 130.117.49.42/32 130.117.49.134/32 130.117.50.45/32 130.117.51.38/32 130.117.51.42/32 130.117.51.170/32 130.117.188.176/32 130.117.188.178/31 130.117.188.180/30 130.117.254.233/32 130.117.254.234/32 130.180.208.0/20 130.185.0.0/18 130.255.128.0/20 134.249.0.0/16 141.8.144.0/24 141.8.147.160/29 141.8.174.0/28 141.8.174.32/29 141.8.174.48/29 141.8.174.64/26 141.8.174.128/27 141.8.174.160/28 141.8.174.176/29 141.8.174.184/30 141.8.175.0/25 141.8.175.128/26 141.8.175.192/28 141.8.175.208/31 141.8.175.224/27 141.101.0.0/19 141.101.226.0/23 141.101.228.0/22 141.101.232.0/23 141.105.128.0/20 141.138.96.0/19 141.170.224.0/19 146.0.80.0/21 146.120.24.0/21 146.120.89.0/24 146.185.232.0/24 146.185.240.0/23 146.185.243.0/31 146.185.243.10/31 146.185.243.12/30 146.185.243.16/28 146.185.243.32/27 146.185.243.64/26 146.185.243.128/25 149.6.76.1/32 149.6.76.9/32 149.6.76.17/32 149.6.76.25/32 149.6.190.0/24 149.6.191.48/29 149.6.191.57/32 149.6.191.61/32 149.6.191.97/32 149.6.191.104/29 149.6.191.112/29 149.6.191.121/32 149.6.191.125/32 149.11.56.1/32 149.11.56.5/32 149.11.56.9/32 149.11.56.13/32 149.11.56.16/29 149.11.56.24/30 149.11.56.33/32 149.11.56.41/32 149.11.80.4/32 149.11.80.6/31 149.11.94.1/32 149.11.94.4/30 149.11.94.9/32 149.14.58.1/32 149.14.58.5/32 149.14.58.9/32 149.14.58.17/32 149.14.58.21/32 149.154.160.0/19 151.0.0.0/18 151.249.96.0/21 154.54.17.5/32 154.54.17.6/32 154.54.17.245/32 154.54.17.246/32 154.54.19.93/32 154.54.19.94/32 154.54.21.88/32 154.54.21.131/32 154.54.21.133/32 154.54.21.157/32 154.54.21.230/32 154.54.35.113/32 154.54.38.105/32 154.54.38.106/32 154.54.38.109/32 154.54.38.110/32 154.54.38.113/32 154.54.38.129/32 154.54.38.130/32 154.54.38.133/32 154.54.38.134/32 154.54.38.137/32 154.54.38.138/32 154.54.38.141/32 154.54.38.142/32 154.54.39.37/32 154.54.39.41/32 154.54.39.121/32 154.54.56.30/32 154.54.56.34/32 154.54.57.5/32 154.54.57.6/32 154.54.57.9/32 154.54.57.10/32 154.54.57.13/32 154.54.57.14/32 154.54.57.17/32 154.54.57.18/32 154.54.60.5/32 154.54.60.6/32 154.54.63.206/32 154.54.73.161/32 154.54.73.162/32 154.54.73.165/32 154.54.73.166/32 154.54.75.197/32 154.54.75.198/32 154.54.75.201/32 154.54.75.202/32 154.54.76.69/32 154.54.76.70/32 154.54.76.225/32 154.54.76.229/32 154.54.78.29/32 154.54.78.33/32 154.54.78.221/32 154.54.78.225/32 154.54.78.229/32 154.54.78.233/32 154.56.0.0/24 159.146.128.0/17 159.160.0.0/16 159.224.0.0/16 162.211.121.192/29 162.211.127.20/30 171.25.168.0/21 171.25.204.0/22 171.25.236.0/22 171.25.240.0/23 173.224.124.70/31 173.224.124.72/32 173.224.124.105/32 173.224.126.0/31 173.224.126.135/32 173.224.126.153/32 173.224.126.167/32 173.224.126.168/32 173.224.127.19/32 173.224.127.20/32 173.224.127.143/32 173.224.127.146/32 173.249.200.0/23 173.253.120.16/29 176.8.0.0/16 176.32.0.0/20 176.32.214.160/29 176.36.0.0/14 176.47.0.0/16 176.57.48.0/20 176.67.0.0/19 176.67.84.152/29 176.96.72.0/21 176.96.192.0/19 176.97.0.0/21 176.97.56.0/21 176.97.112.0/20 176.98.0.0/19 176.98.48.0/21 176.98.64.0/19 176.99.96.0/19 176.100.0.0/19 176.100.56.0/21 176.100.160.0/19 176.101.0.0/20 176.101.96.0/19 176.101.184.0/21 176.101.192.0/19 176.102.0.0/18 176.102.192.0/19 176.103.0.0/19 176.103.64.0/21 176.103.112.0/21 176.103.200.0/21 176.103.208.0/21 176.103.224.0/20 176.104.0.0/18 176.104.64.0/19 176.104.96.0/21 176.104.168.0/21 176.104.184.0/21 176.104.224.0/19 176.105.0.0/17 176.105.160.0/19 176.105.192.0/19 176.106.0.0/21 176.106.192.0/19 176.107.88.0/21 176.107.128.0/19 176.107.176.0/20 176.107.192.0/21 176.108.0.0/19 176.108.96.0/19 176.108.184.0/21 176.108.232.0/21 176.109.8.0/21 176.109.168.0/21 176.109.176.0/20 176.109.192.0/18 176.110.0.0/18 176.110.102.0/23 176.111.32.0/19 176.111.80.0/20 176.111.176.0/20 176.111.240.0/21 176.112.0.0/19 176.112.120.0/21 176.113.144.0/20 176.113.160.0/20 176.113.208.0/20 176.113.224.0/19 176.114.0.0/20 176.114.32.0/20 176.115.96.0/21 176.116.64.0/19 176.116.192.0/21 176.117.64.0/19 176.117.160.0/19 176.118.40.0/21 176.118.48.0/21 176.118.144.0/21 176.119.0.0/20 176.119.24.0/21 176.119.64.0/18 176.119.128.0/19 176.119.232.0/21 176.120.32.0/19 176.120.96.0/20 176.120.112.0/21 176.121.0.0/20 176.121.72.0/21 176.121.192.0/20 176.121.224.0/19 176.122.88.0/21 176.122.96.0/19 176.122.128.0/18 176.123.220.0/23 176.124.0.0/18 176.124.128.0/21 176.124.138.0/23 176.124.228.0/22 176.124.232.0/21 176.125.32.0/19 176.125.224.0/19 176.126.60.0/22 176.126.96.0/19 176.126.128.0/19 176.126.161.0/24 176.126.162.0/23 176.221.48.0/21 176.241.104.0/21 176.241.128.0/19 178.16.80.0/20 178.20.152.0/21 178.23.136.0/21 178.54.0.0/16 178.74.192.0/18 178.86.0.0/15 178.92.0.0/14 178.133.0.0/16 178.136.0.0/15 178.150.0.0/15 178.154.154.0/23 178.158.0.0/16 178.159.32.0/20 178.159.96.0/19 178.159.176.0/20 178.159.208.0/20 178.159.224.0/20 178.165.0.0/17 178.170.166.0/24 178.209.64.0/19 178.210.128.0/19 178.210.192.0/19 178.211.96.0/19 178.211.128.0/19 178.212.64.0/21 178.212.96.0/20 178.212.192.0/21 178.212.232.0/21 178.212.240.0/21 178.213.0.0/21 178.213.104.0/21 178.213.168.0/21 178.213.184.0/21 178.214.160.0/19 178.214.192.0/19 178.215.160.0/19 178.215.224.0/19 178.216.0.0/20 178.216.16.0/21 178.216.32.0/21 178.216.96.0/21 178.216.120.0/21 178.216.168.0/21 178.216.184.0/21 178.216.224.0/20 178.217.160.0/21 178.217.200.0/21 178.217.208.0/21 178.218.64.0/20 178.219.80.0/20 178.219.160.0/20 178.219.192.0/20 178.219.240.0/20 178.251.104.0/21 178.255.176.0/21 185.1.16.0/24 185.2.108.0/22 185.2.212.0/22 185.3.148.0/22 185.4.40.0/22 185.4.236.0/22 185.5.104.0/22 185.5.252.0/22 185.6.184.0/22 185.8.40.0/22 185.9.40.0/22 185.9.160.0/22 185.10.208.0/22 185.11.28.0/22 185.11.80.0/22 185.12.8.0/22 185.12.112.0/22 185.12.140.0/22 185.13.4.0/22 185.13.248.0/22 185.14.16.0/22 185.14.28.0/22 185.14.192.0/22 185.15.4.0/22 185.16.28.0/22 185.16.124.0/22 185.16.228.0/22 185.17.124.0/22 185.19.4.0/22 185.19.244.0/22 185.20.216.0/22 185.21.176.0/22 185.22.16.0/22 185.23.104.0/22 185.25.116.0/22 185.27.12.0/22 185.28.108.0/22 185.28.236.0/22 185.29.236.0/22 185.29.252.0/22 185.30.84.0/22 185.30.200.0/22 185.31.84.0/22 185.32.60.0/22 185.33.140.0/22 185.34.224.0/22 185.35.8.0/22 185.35.100.0/22 185.36.56.0/22 185.36.188.0/22 185.36.196.0/22 185.37.192.0/22 185.38.72.0/22 185.38.208.0/22 185.38.216.0/22 185.39.28.0/22 185.39.72.0/22 185.39.196.0/22 185.39.224.0/22 185.40.48.0/22 185.40.181.0/24 185.40.182.0/23 185.41.20.0/22 185.41.192.0/22 185.42.40.0/22 185.42.128.0/22 188.0.64.0/18 188.0.128.0/19 188.95.152.0/22 188.95.156.0/26 188.95.156.64/28 188.95.156.96/27 188.95.156.128/25 188.95.157.0/24 188.95.158.0/23 188.115.128.0/18 188.130.176.0/22 188.130.192.0/19 188.130.246.0/24 188.130.251.0/24 188.163.0.0/16 188.190.0.0/17 188.190.160.0/19 188.190.192.0/18 188.191.16.0/20 188.191.32.0/20 188.191.64.0/20 188.191.96.0/20 188.191.144.0/21 188.191.232.0/21 188.230.0.0/17 188.231.128.0/17 188.239.0.0/16 188.247.96.0/19 192.102.6.0/23 192.121.68.128/25 192.162.19.0/24 192.162.32.0/22 192.162.76.0/22 192.162.80.0/22 192.162.108.0/22 192.162.112.0/21 192.162.132.0/22 192.162.140.0/22 192.162.152.0/22 192.162.160.0/22 192.162.200.0/22 192.162.208.0/22 192.162.224.0/22 192.162.232.0/21 192.166.100.0/22 192.166.112.0/23 192.166.128.0/21 192.166.144.0/23 192.196.221.32/27 192.196.226.24/29 192.196.229.0/27 192.245.152.0/24 193.0.61.0/24 193.0.142.0/23 193.0.204.0/22 193.0.216.0/22 193.0.220.0/23 193.0.223.0/24 193.0.227.0/24 193.0.228.0/24 193.0.240.0/24 193.0.247.0/24 193.8.78.0/23 193.8.201.0/24 193.9.28.0/24 193.9.158.0/24 193.9.250.0/23 193.16.45.0/24 193.16.47.0/24 193.16.101.0/24 193.16.105.0/24 193.16.158.0/24 193.16.233.0/24 193.16.247.0/24 193.17.46.0/24 193.17.69.0/24 193.17.75.0/24 193.17.208.0/24 193.17.213.0/24 193.17.216.0/23 193.19.74.0/23 193.19.84.0/22 193.19.96.0/23 193.19.100.0/23 193.19.108.0/22 193.19.118.0/23 193.19.132.0/22 193.19.152.0/22 193.19.184.0/22 193.19.204.0/22 193.19.228.0/22 193.19.240.0/21 193.19.252.0/22 193.22.84.0/24 193.22.138.0/24 193.22.140.0/24 193.23.0.0/24 193.23.53.0/24 193.23.58.0/24 193.23.60.0/24 193.23.122.0/24 193.23.140.0/24 193.23.157.0/24 193.23.175.0/24 193.23.181.0/24 193.23.183.0/24 193.23.225.0/24 193.24.25.0/24 193.24.30.0/24 193.24.220.0/22 193.25.106.0/23 193.25.120.0/23 193.25.176.0/23 193.25.180.0/23 193.25.255.0/24 193.26.3.0/24 193.26.13.0/24 193.26.20.0/24 193.26.27.0/24 193.26.134.0/24 193.26.209.0/24 193.27.80.0/23 193.27.196.0/23 193.27.208.0/23 193.27.234.0/23 193.27.242.0/23 193.28.85.0/24 193.28.92.0/24 193.28.156.0/24 193.28.177.0/24 193.28.184.0/24 193.28.186.0/24 193.28.200.0/24 193.29.16.0/24 193.29.128.0/24 193.29.200.0/24 193.29.203.0/24 193.29.204.0/24 193.29.220.0/24 193.30.40.0/24 193.30.240.0/21 193.30.253.0/24 193.32.12.0/22 193.32.21.0/24 193.32.57.0/24 193.33.44.0/23 193.33.48.0/23 193.33.54.0/23 193.33.64.0/23 193.33.86.0/23 193.33.104.0/23 193.33.196.0/23 193.33.202.0/23 193.33.206.0/23 193.33.212.0/23 193.33.236.0/23 193.34.20.0/22 193.34.60.0/22 193.34.72.0/21 193.34.92.0/22 193.34.96.0/22 193.34.128.0/23 193.34.140.0/23 193.34.154.0/23 193.34.168.0/23 193.34.172.0/23 193.34.200.128/25 193.34.203.0/24 193.34.216.0/22 193.35.25.0/24 193.37.133.0/24 193.37.141.0/24 193.37.156.0/24 193.39.69.0/24 193.39.72.0/24 193.39.75.0/24 193.39.76.0/23 193.39.114.0/24 193.39.118.0/24 193.41.4.0/23 193.41.38.0/24 193.41.48.0/22 193.41.60.0/22 193.41.80.0/24 193.41.88.0/24 193.41.128.0/22 193.41.184.0/22 193.41.218.0/23 193.41.239.0/24 193.42.148.0/24 193.42.157.0/24 193.43.95.0/24 193.43.210.0/23 193.43.222.0/23 193.43.224.0/22 193.43.248.0/21 193.46.46.0/24 193.46.66.0/24 193.46.81.0/24 193.46.86.0/24 193.46.89.0/24 193.46.201.0/24 193.46.202.0/24 193.46.210.0/24 193.47.70.0/24 193.47.85.0/24 193.47.137.0/24 193.47.166.0/24 193.47.189.0/24 193.53.83.0/24 193.53.89.0/24 193.58.246.0/24 193.58.248.0/24 193.58.254.0/24 193.84.17.0/24 193.84.19.0/24 193.84.22.0/23 193.84.50.0/24 193.84.72.0/24 193.84.76.0/23 193.84.90.0/24 193.93.12.0/22 193.93.16.0/22 193.93.48.0/21 193.93.76.0/22 193.93.100.0/22 193.93.108.0/22 193.93.116.0/22 193.93.160.0/22 193.93.168.0/22 193.93.184.0/21 193.93.216.0/22 193.93.228.0/22 193.104.7.0/24 193.104.16.0/24 193.104.58.0/24 193.104.67.0/24 193.104.85.0/24 193.104.96.0/24 193.104.133.0/24 193.104.145.0/24 193.104.178.0/24 193.104.182.0/23 193.104.189.0/24 193.104.203.0/24 193.104.208.0/23 193.104.210.0/24 193.104.213.0/24 193.104.242.0/24 193.104.249.0/24 193.105.7.0/24 193.105.30.0/24 193.105.41.0/24 193.105.56.0/24 193.105.62.0/24 193.105.70.0/24 193.105.92.0/24 193.105.121.0/24 193.105.124.0/24 193.105.135.0/24 193.105.152.0/24 193.105.164.0/24 193.105.169.0/24 193.105.189.0/24 193.105.201.0/24 193.105.210.0/24 193.105.216.0/24 193.105.219.0/24 193.105.249.0/24 193.105.254.0/24 193.106.24.0/22 193.106.56.0/21 193.106.64.0/22 193.106.80.0/22 193.106.136.0/22 193.106.144.0/22 193.106.160.0/22 193.106.184.0/22 193.106.200.0/22 193.106.211.0/24 193.106.220.0/22 193.106.236.0/23 193.107.24.0/22 193.107.40.0/22 193.107.72.0/22 193.107.80.0/22 193.107.104.0/21 193.107.112.0/22 193.107.128.0/21 193.107.136.0/22 193.107.168.0/21 193.107.176.0/22 193.107.184.0/22 193.107.200.0/22 193.107.224.0/22 193.107.244.0/22 193.108.38.0/23 193.108.44.0/23 193.108.48.0/22 193.108.56.0/22 193.108.102.0/23 193.108.104.0/23 193.108.112.0/21 193.108.120.0/22 193.108.128.0/23 193.108.162.0/23 193.108.170.0/23 193.108.209.0/24 193.108.226.0/23 193.108.236.0/23 193.108.240.0/22 193.108.248.0/22 193.109.8.0/22 193.109.80.0/24 193.109.84.0/24 193.109.108.0/22 193.109.118.0/24 193.109.128.0/23 193.109.144.0/22 193.109.160.0/21 193.109.168.0/22 193.109.216.0/24 193.109.240.0/23 193.109.248.0/23 193.110.16.0/21 193.110.72.0/21 193.110.89.0/24 193.110.100.0/23 193.110.106.0/23 193.110.112.0/22 193.110.124.0/22 193.110.144.0/24 193.110.156.0/24 193.110.160.0/22 193.110.168.0/23 193.110.172.0/22 193.110.176.0/23 193.110.184.0/23 193.110.188.0/23 193.111.0.0/23 193.111.6.0/23 193.111.8.0/23 193.111.48.0/22 193.111.60.0/22 193.111.83.0/24 193.111.85.0/24 193.111.114.0/23 193.111.126.0/23 193.111.156.0/22 193.111.173.0/24 193.111.188.0/22 193.111.204.0/23 193.111.239.0/24 193.111.240.0/22 193.111.248.0/22 193.124.48.0/24 193.124.54.0/24 193.124.73.0/24 193.138.84.0/24 193.138.87.0/24 193.138.93.0/24 193.138.122.0/24 193.138.132.0/22 193.138.144.0/22 193.138.184.0/22 193.138.236.0/22 193.138.244.0/22 193.142.114.0/24 193.142.124.0/24 193.142.159.0/24 193.142.213.0/24 193.142.218.0/23 193.142.221.0/24 193.150.6.0/24 193.150.28.0/22 193.150.49.0/24 193.150.56.0/22 193.150.88.0/23 193.150.96.0/23 193.150.122.0/24 193.151.12.0/22 193.151.56.0/22 193.151.88.0/22 193.151.104.0/22 193.151.160.0/19 193.151.240.0/21 193.151.252.0/22 193.160.97.0/24 193.160.224.0/22 193.161.12.0/22 193.164.92.0/22 193.164.130.0/24 193.164.149.0/24 193.169.80.0/23 193.169.124.0/22 193.169.132.0/22 193.169.146.0/23 193.169.188.0/23 193.169.214.0/23 193.169.240.0/23 193.169.252.0/23 193.176.147.0/24 193.176.248.0/22 193.178.34.0/24 193.178.124.0/22 193.178.144.0/22 193.178.162.0/24 193.178.172.0/24 193.178.187.0/24 193.178.190.0/23 193.178.228.0/23 193.178.236.0/23 193.178.248.0/22 193.186.9.0/24 193.186.15.0/24 193.187.76.0/22 193.188.254.0/24 193.189.96.0/23 193.189.126.0/23 193.192.36.0/23 193.193.192.0/19 193.194.106.0/23 193.194.108.0/22 193.194.112.0/22 193.194.124.0/22 193.200.22.0/24 193.200.32.0/23 193.200.36.0/22 193.200.64.0/23 193.200.68.0/23 193.200.76.0/23 193.200.84.0/23 193.200.151.0/24 193.200.161.0/24 193.200.167.0/24 193.200.169.0/24 193.200.173.0/24 193.200.175.0/24 193.200.179.0/24 193.200.183.0/24 193.200.190.0/24 193.200.205.0/24 193.200.209.0/24 193.200.212.0/24 193.200.219.0/24 193.200.229.0/24 193.200.246.0/24 193.200.248.0/24 193.200.255.0/24 193.201.60.0/22 193.201.80.0/22 193.201.98.0/23 193.201.100.0/24 193.201.105.0/24 193.201.116.0/23 193.201.140.0/22 193.201.175.0/24 193.201.198.0/23 193.201.206.0/23 193.201.216.0/22 193.201.224.0/22 193.202.21.0/24 193.202.118.0/24 193.203.110.0/23 193.203.218.0/23 193.203.236.0/23 193.218.118.0/24 193.218.156.0/22 193.218.184.0/22 193.219.99.0/24 193.219.124.0/24 193.222.52.0/22 193.222.111.0/24 193.222.140.0/24 193.223.98.0/24 193.227.96.0/23 193.227.115.0/24 193.227.119.0/24 193.227.120.0/24 193.227.206.0/23 193.227.208.0/22 193.227.230.0/23 193.227.250.0/23 193.228.2.0/24 193.228.148.0/24 193.232.65.0/24 193.238.20.0/22 193.238.32.0/21 193.238.44.0/22 193.238.72.0/22 193.238.96.0/22 193.238.108.0/22 193.238.116.0/22 193.238.152.0/22 193.238.192.0/22 193.238.224.0/22 193.239.24.0/22 193.239.68.0/22 193.239.72.0/22 193.239.128.0/23 193.239.142.0/23 193.239.152.0/23 193.239.170.0/23 193.239.180.0/23 193.239.216.0/23 193.239.228.0/23 193.239.234.0/23 193.239.238.0/23 193.239.250.0/23 193.239.254.0/23 193.242.114.0/24 193.242.156.0/22 193.242.160.0/22 193.242.166.0/24 193.243.152.0/23 193.243.156.0/22 193.243.167.0/24 193.254.196.0/23 193.254.216.0/23 193.254.220.0/23 193.254.232.0/23 194.0.52.0/22 194.0.56.0/23 194.0.88.0/22 194.0.116.0/22 194.0.131.0/24 194.0.138.0/24 194.0.148.0/24 194.0.150.0/24 194.0.157.0/24 194.0.187.0/24 194.0.200.0/24 194.0.206.0/24 194.0.218.0/24 194.0.237.0/24 194.1.159.0/24 194.1.177.0/24 194.1.193.0/24 194.1.195.0/24 194.1.220.0/23 194.6.231.0/24 194.6.232.0/23 194.8.51.0/24 194.8.56.0/24 194.8.59.0/24 194.8.64.0/23 194.8.144.0/22 194.8.156.0/22 194.8.240.0/23 194.8.248.0/23 194.9.14.0/23 194.9.26.0/23 194.9.36.0/23 194.9.50.0/23 194.9.70.0/23 194.11.28.0/24 194.11.225.0/24 194.12.64.0/18 194.15.112.0/22 194.15.147.0/24 194.24.162.0/23 194.24.182.0/23 194.24.184.0/22 194.24.190.0/23 194.24.236.0/23 194.24.246.0/23 194.28.4.0/22 194.28.16.0/22 194.28.36.0/22 194.28.52.0/22 194.28.60.0/22 194.28.64.0/21 194.28.84.0/22 194.28.88.0/22 194.28.100.0/22 194.28.132.0/22 194.28.140.0/22 194.28.144.0/22 194.28.172.0/22 194.28.176.0/21 194.28.196.0/22 194.29.60.0/22 194.29.205.0/24 194.30.163.0/24 194.30.170.0/24 194.30.172.0/24 194.31.44.0/22 194.31.248.0/23 194.33.15.0/24 194.33.109.0/24 194.33.180.0/23 194.33.188.0/23 194.37.248.0/24 194.42.106.0/23 194.42.112.0/23 194.42.192.0/20 194.44.0.0/16 194.48.209.0/24 194.48.212.0/24 194.50.0.0/24 194.50.9.0/24 194.50.51.0/24 194.50.79.0/24 194.50.81.0/24 194.50.85.0/24 194.50.98.0/24 194.50.114.0/24 194.50.119.0/24 194.50.125.0/24 194.50.144.0/22 194.50.161.0/24 194.50.167.0/24 194.50.169.0/24 194.50.241.0/24 194.50.254.0/24 194.54.76.0/22 194.54.80.0/22 194.54.88.0/22 194.54.152.0/21 194.54.160.0/22 194.54.184.0/22 194.60.69.0/24 194.60.77.0/24 194.60.252.0/22 194.69.1.0/24 194.79.8.0/22 194.79.20.0/22 194.79.60.0/22 194.88.1.0/24 194.88.150.0/23 194.88.152.0/23 194.88.206.0/23 194.88.218.0/23 194.88.220.0/23 194.93.160.0/19 194.105.136.0/23 194.105.144.0/23 194.106.208.0/23 194.106.216.0/22 194.107.115.0/24 194.107.117.0/24 194.110.6.0/24 194.110.79.0/24 194.110.126.0/24 194.110.129.0/24 194.110.219.0/24 194.110.248.0/24 194.110.252.0/24 194.114.132.0/22 194.114.136.0/22 194.116.162.0/23 194.116.170.0/23 194.116.194.0/23 194.116.232.0/23 194.116.238.0/23 194.116.244.0/23 194.117.55.0/25 194.117.119.69/32 194.117.252.0/23 194.125.224.0/22 194.125.244.0/23 194.126.180.0/22 194.126.204.0/24 194.126.224.0/24 194.140.228.0/24 194.140.237.0/24 194.140.239.0/24 194.143.136.0/23 194.143.144.0/22 194.145.117.0/24 194.145.198.0/23 194.145.202.0/23 194.145.214.0/23 194.145.216.0/23 194.145.220.0/23 194.145.227.0/24 194.146.110.0/24 194.146.112.0/24 194.146.132.0/22 194.146.136.0/21 194.146.156.0/22 194.146.180.0/22 194.146.188.0/22 194.146.196.0/22 194.146.228.0/22 194.149.92.0/24 194.150.72.0/21 194.150.92.0/22 194.150.174.0/23 194.150.192.0/23 194.150.204.0/23 194.150.220.0/23 194.150.232.0/23 194.153.128.0/23 194.153.148.0/23 194.165.6.0/23 194.165.46.0/24 194.165.62.0/23 194.169.193.0/24 194.169.204.0/22 194.169.210.0/24 194.169.238.0/24 194.176.97.0/24 194.176.117.0/24 194.177.0.0/22 194.177.24.0/22 194.183.160.0/19 194.187.24.0/21 194.187.48.0/22 194.187.104.0/21 194.187.128.0/22 194.187.148.0/22 194.187.152.0/22 194.187.208.0/22 194.187.216.0/22 194.187.228.0/22 194.213.23.0/24 194.213.104.0/22 194.213.110.0/23 194.213.120.0/23 194.221.103.1/32 194.221.103.2/32 194.221.103.5/32 194.221.103.6/32 194.221.103.9/32 194.221.103.10/32 194.221.103.13/32 194.221.103.14/32 194.221.103.17/32 194.221.103.18/32 194.242.53.0/24 194.242.59.0/24 194.242.60.0/24 194.242.96.0/21 194.242.116.0/22 194.246.74.0/23 194.246.77.0/24 194.246.99.0/24 194.246.104.0/23 194.246.116.0/23 194.246.120.0/23 194.247.12.0/23 194.247.16.0/22 194.247.33.0/24 194.247.35.0/24 194.247.37.0/24 194.247.42.0/24 194.247.60.0/24 194.247.172.0/22 194.247.176.0/22 195.2.1.52/32 195.2.28.45/32 195.2.28.53/32 195.2.30.10/32 195.2.30.146/32 195.2.204.0/23 195.2.236.0/23 195.2.242.0/23 195.3.128.0/21 195.3.148.0/22 195.3.156.0/22 195.3.196.0/22 195.3.204.0/22 195.3.224.0/22 195.3.236.0/22 195.3.244.0/22 195.5.0.0/18 195.5.106.0/23 195.5.108.0/23 195.5.124.0/23 195.5.126.0/24 195.5.184.0/24 195.8.40.0/22 195.8.56.0/23 195.8.200.0/23 195.10.210.0/24 195.10.218.0/24 195.12.36.0/22 195.12.56.0/22 195.13.47.0/24 195.14.17.0/24 195.14.122.0/23 195.14.124.0/23 195.16.76.0/22 195.16.88.0/22 195.18.12.0/22 195.18.16.0/22 195.18.28.0/22 195.20.4.0/22 195.20.28.0/22 195.20.96.0/23 195.20.102.0/23 195.20.118.0/23 195.20.124.0/23 195.20.130.0/24 195.20.148.0/24 195.20.154.0/24 195.22.108.0/22 195.22.112.0/22 195.22.130.0/23 195.22.132.0/23 195.22.140.0/23 195.24.128.0/19 195.24.234.0/23 195.24.242.0/23 195.24.252.0/23 195.26.16.0/22 195.26.64.0/22 195.26.80.0/21 195.26.92.0/22 195.28.30.0/23 195.34.74.0/23 195.34.90.0/23 195.34.94.0/23 195.34.196.0/22 195.34.204.0/22 195.35.65.0/24 195.38.16.0/23 195.38.18.0/24 195.39.196.0/23 195.39.210.0/23 195.39.232.0/23 195.39.240.0/22 195.39.248.0/23 195.39.252.0/23 195.42.126.0/23 195.42.130.0/23 195.42.136.0/23 195.43.40.0/22 195.43.70.0/23 195.43.146.0/24 195.43.148.0/24 195.46.32.0/22 195.46.56.0/22 195.47.202.0/24 195.47.212.0/24 195.47.219.0/24 195.47.248.0/24 195.47.253.0/24 195.49.128.0/22 195.49.148.0/22 195.49.164.0/22 195.49.200.0/21 195.54.42.0/23 195.54.52.0/23 195.54.162.0/23 195.54.166.0/23 195.58.224.0/19 195.60.66.0/23 195.60.70.0/23 195.60.92.128/25 195.60.168.0/23 195.60.174.0/23 195.60.184.0/23 195.60.200.0/21 195.60.224.0/21 195.62.14.0/23 195.62.24.0/23 195.62.36.0/23 195.64.136.0/23 195.64.142.0/23 195.64.148.0/23 195.64.154.0/23 195.64.162.0/23 195.64.166.0/23 195.64.182.0/23 195.64.184.0/23 195.64.190.0/23 195.64.224.0/19 195.66.65.0/24 195.66.66.0/24 195.66.79.0/24 195.66.93.0/24 195.66.105.0/24 195.66.110.0/24 195.66.114.0/24 195.66.120.0/24 195.66.136.0/22 195.66.140.0/23 195.66.146.0/23 195.66.152.0/23 195.66.156.0/23 195.66.192.0/19 195.68.196.0/23 195.68.202.0/23 195.68.210.0/23 195.68.216.0/22 195.68.222.0/23 195.68.248.0/23 195.69.76.0/22 195.69.84.0/22 195.69.132.0/22 195.69.136.0/22 195.69.176.0/22 195.69.184.0/22 195.69.196.0/22 195.69.200.0/22 195.69.220.0/22 195.69.244.0/22 195.69.248.0/22 195.72.144.0/22 195.72.156.0/22 195.74.67.0/24 195.74.72.0/24 195.78.38.0/23 195.78.42.0/23 195.78.56.0/22 195.78.68.0/23 195.78.92.0/23 195.78.96.0/23 195.78.100.0/23 195.78.104.0/23 195.78.112.0/23 195.78.232.0/22 195.78.244.0/22 195.78.252.0/22 195.80.140.0/23 195.80.231.0/24 195.80.232.0/24 195.82.150.0/23 195.85.198.0/24 195.85.208.0/24 195.85.214.0/24 195.85.216.0/24 195.85.219.0/24 195.85.223.0/24 195.85.233.0/24 195.85.250.0/24 195.88.14.0/23 195.88.26.0/23 195.88.42.0/23 195.88.52.0/23 195.88.72.0/23 195.88.76.0/22 195.88.112.0/23 195.88.124.0/23 195.88.138.0/23 195.88.158.0/23 195.88.242.0/23 195.90.122.0/23 195.93.138.0/23 195.93.154.0/23 195.93.172.0/23 195.93.184.0/23 195.93.190.0/23 195.93.204.0/23 195.93.212.0/22 195.95.139.0/24 195.95.147.0/24 195.95.151.0/24 195.95.157.0/24 195.95.159.0/24 195.95.165.0/24 195.95.171.0/24 195.95.185.0/24 195.95.189.0/24 195.95.202.0/23 195.95.206.0/23 195.95.210.0/23 195.95.222.0/23 195.95.232.0/23 195.110.6.0/23 195.110.56.0/23 195.112.165.252/30 195.112.175.184/30 195.112.180.183/32 195.114.6.0/23 195.114.30.0/23 195.114.96.0/23 195.114.120.0/23 195.114.128.0/19 195.123.0.0/16 195.128.16.0/22 195.128.139.0/24 195.128.182.0/23 195.128.226.0/23 195.128.230.0/23 195.128.248.0/23 195.128.252.0/23 195.130.219.0/24 195.135.196.0/22 195.135.244.0/22 195.137.167.0/24 195.137.185.0/24 195.137.192.0/23 195.137.196.0/23 195.137.202.0/23 195.137.206.0/23 195.137.218.0/23 195.137.226.0/23 195.137.232.0/23 195.137.240.0/23 195.137.244.0/23 195.137.250.0/23 195.137.254.0/23 195.138.64.0/19 195.138.136.81/32 195.138.136.145/32 195.138.160.0/19 195.138.193.0/24 195.138.198.0/24 195.138.217.0/24 195.138.218.0/23 195.140.160.0/22 195.140.168.0/22 195.140.176.0/22 195.140.224.0/21 195.140.244.0/22 195.143.235.50/32 195.144.14.0/24 195.144.21.0/24 195.144.25.0/24 195.144.28.0/24 195.149.70.0/24 195.149.88.0/24 195.149.96.0/24 195.149.106.0/24 195.149.108.0/23 195.149.112.0/24 195.149.114.0/24 195.149.117.0/24 195.149.125.0/24 195.160.192.0/22 195.160.204.0/22 195.160.220.0/22 195.160.232.0/22 195.162.0.0/23 195.162.64.0/23 195.162.70.0/23 195.162.74.0/23 195.162.80.0/22 195.162.88.0/22 195.170.179.0/24 195.177.68.0/22 195.177.72.0/22 195.177.92.0/22 195.177.116.0/22 195.177.124.0/22 195.177.208.0/23 195.177.222.0/23 195.177.236.0/22 195.177.240.0/23 195.178.10.0/23 195.178.18.0/23 195.178.30.0/23 195.178.104.0/23 195.178.128.0/19 195.182.0.0/24 195.182.7.0/24 195.182.21.0/24 195.182.22.0/24 195.182.63.0/24 195.182.192.0/22 195.182.202.0/23 195.184.70.0/23 195.184.80.0/23 195.184.86.0/23 195.184.192.0/19 195.189.8.0/22 195.189.16.0/22 195.189.36.0/22 195.189.44.0/22 195.189.48.0/22 195.189.60.0/22 195.189.96.0/22 195.189.104.0/22 195.189.132.0/23 195.189.196.0/23 195.189.200.0/23 195.189.214.0/23 195.189.226.0/23 195.189.228.0/23 195.189.234.0/23 195.189.240.0/23 195.189.246.0/23 195.189.248.0/23 195.190.13.0/24 195.190.152.0/24 195.190.157.0/24 195.191.12.0/23 195.191.24.0/23 195.191.38.0/23 195.191.56.0/22 195.191.72.0/23 195.191.80.0/23 195.191.90.0/23 195.191.104.0/23 195.191.126.0/23 195.191.138.0/23 195.191.168.0/23 195.191.178.0/23 195.191.194.0/23 195.191.200.0/23 195.191.214.0/23 195.191.226.0/23 195.191.234.0/23 195.191.246.0/23 195.200.64.0/23 195.200.90.0/23 195.200.196.0/24 195.200.221.0/24 195.200.244.0/23 195.200.250.0/23 195.206.232.0/23 195.206.238.0/23 195.211.60.0/22 195.211.84.0/22 195.211.96.0/22 195.211.108.0/22 195.211.116.0/22 195.211.136.0/21 195.211.144.0/21 195.211.152.0/22 195.211.160.0/22 195.211.172.0/22 195.211.184.0/21 195.211.208.0/21 195.211.228.0/22 195.211.240.0/22 195.211.252.0/22 195.214.192.0/21 195.214.208.0/21 195.214.220.0/22 195.214.236.0/22 195.216.204.0/22 195.216.210.0/23 195.216.212.0/23 195.216.226.0/24 195.216.248.0/24 195.219.39.0/24 195.222.124.0/22 195.225.48.0/23 195.225.52.0/23 195.225.112.0/22 195.225.144.0/22 195.225.156.0/22 195.225.172.0/22 195.225.228.0/22 195.226.192.0/24 195.226.197.0/24 195.226.213.0/24 195.230.96.0/24 195.230.103.0/24 195.230.112.0/24 195.230.115.0/24 195.230.119.0/24 195.230.125.0/24 195.230.127.0/24 195.230.128.0/19 195.234.4.0/22 195.234.68.0/22 195.234.72.0/21 195.234.112.0/22 195.234.124.0/22 195.234.132.0/24 195.234.148.0/24 195.234.174.0/24 195.234.200.0/22 195.234.212.0/22 195.234.220.0/22 195.238.92.0/23 195.238.108.0/22 195.238.112.0/21 195.238.176.0/21 195.238.188.0/22 195.242.80.0/23 195.242.94.0/23 195.242.114.0/23 195.242.150.0/23 195.242.161.0/24 195.242.179.0/24 195.242.200.0/22 195.242.218.0/23 195.244.4.0/23 195.244.8.0/23 195.245.112.0/23 195.245.118.0/23 195.245.120.0/23 195.245.200.0/24 195.245.215.0/24 195.245.221.0/24 195.245.253.0/24 195.246.124.0/23 195.246.217.0/24 195.248.93.0/24 195.248.160.0/19 195.248.224.0/23 195.248.234.0/23 195.250.36.0/24 195.250.43.0/24 195.250.47.0/24 195.250.62.0/24 195.254.142.0/23 195.254.150.0/23 198.44.134.120/29 198.210.51.2/31 198.210.51.4/30 198.210.51.8/29 198.210.51.16/28 198.210.51.32/27 198.210.51.64/27 198.210.51.96/30 198.210.51.100/32 199.66.219.16/28 199.115.77.112/28 203.160.0.0/24 205.153.132.0/22 205.214.72.0/28 205.214.72.16/29 205.214.81.152/29 206.226.114.208/29 206.226.117.0/29 207.150.163.10/31 207.150.163.12/30 207.150.163.16/30 208.70.77.160/28 208.70.79.144/28 208.244.51.32/27 209.18.106.100/30 209.18.106.104/30 209.18.106.108/31 209.18.107.160/29 209.18.107.168/31 209.239.113.44/32 209.239.113.54/32 209.239.113.178/31 209.239.113.180/32 209.239.113.208/32 209.239.119.158/32 209.239.119.181/32 209.239.119.186/32 209.239.119.229/32 209.239.119.254/32 209.239.120.182/31 209.239.121.189/32 209.239.121.224/32 209.239.124.25/32 209.239.124.144/32 212.1.64.0/18 212.2.128.0/19 212.3.96.0/19 212.8.32.0/19 212.9.224.0/19 212.15.128.0/19 212.22.192.0/19 212.26.128.0/19 212.38.96.160/28 212.38.109.112/28 212.40.32.0/19 212.42.64.0/19 212.47.192.0/24 212.47.193.0/26 212.47.193.64/27 212.47.204.8/29 212.47.204.64/26 212.47.204.128/26 212.47.205.128/25 212.47.206.0/24 212.55.64.0/19 212.58.160.0/19 212.66.32.0/19 212.68.160.0/19 212.74.234.0/24 212.79.112.0/20 212.80.32.0/19 212.82.192.0/23 212.82.194.0/24 212.82.195.0/27 212.82.195.32/28 212.82.195.56/29 212.82.195.64/26 212.82.195.128/25 212.82.196.0/22 212.82.200.0/21 212.82.208.0/20 212.86.96.0/19 212.86.224.0/19 212.87.160.0/19 212.90.32.0/19 212.90.160.0/19 212.92.224.0/19 212.109.32.0/19 212.110.128.0/19 212.111.192.0/19 212.113.32.0/22 212.113.36.0/23 212.113.39.0/24 212.113.40.0/21 212.113.48.0/20 212.115.224.0/19 212.178.0.0/19 213.5.192.0/21 213.108.40.0/21 213.108.72.0/21 213.108.176.0/21 213.109.80.0/20 213.109.128.0/20 213.109.208.0/20 213.109.224.0/20 213.110.96.0/19 213.110.128.0/18 213.111.64.0/18 213.111.128.0/17 213.130.0.0/19 213.133.160.0/19 213.151.0.0/19 213.154.192.0/19 213.155.0.0/24 213.155.1.0/25 213.155.1.144/29 213.155.1.160/27 213.155.1.192/26 213.155.2.0/23 213.155.4.0/22 213.155.8.0/22 213.155.12.0/24 213.155.13.0/29 213.155.13.16/28 213.155.13.32/27 213.155.13.128/26 213.155.13.208/28 213.155.13.224/27 213.155.14.0/23 213.155.16.0/22 213.155.20.0/23 213.155.22.0/28 213.155.22.16/29 213.155.22.32/27 213.155.22.64/26 213.155.22.128/27 213.155.22.160/29 213.155.22.176/28 213.155.22.192/26 213.155.23.0/24 213.155.24.0/21 213.156.64.0/19 213.159.224.0/19 213.160.128.0/19 213.169.64.0/19 213.174.0.0/19 213.179.224.0/19 213.186.96.0/19 213.186.192.0/19 213.197.73.96/28 213.197.73.112/29 213.200.32.0/19 213.227.192.0/18 213.231.0.0/18 213.238.0.0/19 213.248.96.169/32 213.248.96.170/32 213.248.104.218/32 213.255.231.128/26 213.255.231.192/27 216.15.194.248/29 216.119.136.200/29 216.119.139.96/29 216.119.139.112/29 217.9.0.0/20 217.12.192.0/21 217.12.202.0/23 217.12.204.0/22 217.12.210.0/23 217.12.212.0/22 217.12.216.0/21 217.20.160.0/19 217.24.160.0/20 217.25.192.0/20 217.27.144.0/20 217.28.240.0/24 217.28.241.0/26 217.28.244.246/31 217.28.244.248/29 217.28.245.0/24 217.28.246.68/30 217.28.246.160/27 217.28.246.192/26 217.28.247.10/31 217.28.247.12/30 217.28.247.52/30 217.28.247.64/28 217.28.247.82/31 217.28.247.96/27 217.28.247.128/25 217.28.250.110/31 217.28.254.0/24 217.30.192.0/20 217.65.240.0/20 217.66.96.0/20 217.67.64.0/20 217.68.6.64/29 217.73.80.0/20 217.76.192.0/20 217.77.208.0/20 217.112.208.0/20 217.115.96.0/20 217.117.64.0/20 217.146.2.0/24 217.146.240.0/20 217.147.160.0/20 217.175.0.0/20 217.175.80.0/20 217.194.133.176/29 217.194.135.128/28 217.194.135.224/28 217.194.155.0/27 217.194.155.32/29 217.194.155.80/28 217.194.155.96/29 217.194.155.112/28 217.196.160.0/20 217.199.224.0/20 219.3.97.0/24');
        $this->db->insert($this->t('ip'), $data);

        // LEECHES
        $data = array('id_label'=>3, 'content'=>'^http://.*awfulforum\.com');
        $this->db->insert($this->t('leech'), $data);

        // USERAGENTS
        $data = array('id_label'=>4);
        $data['content'] = '^$';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\<';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\>';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\'';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\%';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\_iRc';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\_Works';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\@\$x';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\<\?';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\$x0e';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\+select\+';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '\+union\+';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '1\,1\,1\,';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '2icommerce';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '3GSE';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '4all';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '59\.64\.153\.';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '88\.0\.106\.';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '98';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '85\.17\.';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'A\_Browser';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ABAC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Abont';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'abot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Accept';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Access';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Accoo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'AceFTP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Acme';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ActiveTouristBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Address';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Adopt';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'adress';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'adressendeutschland';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ADSARobot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'agent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ah\-ha';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ahead';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'AESOP\_com\_SpiderMan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'aipbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Alarm';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Albert';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Alek';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Alexibot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Alligator';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'AllSubmitter';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'alma';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'almaden';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ALot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Alpha';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'aktuelles';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Akregat';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Amfi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'amzn\_assoc';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Anal';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Anarchie';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'andit';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Anon';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'AnotherBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ansearch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'AnswerBus';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'antivirx';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Apexoo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'appie';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Aqua_Products';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Arachmo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'archive';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'arian';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ASPSe';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ASSORT';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'aster';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Atari';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ATHENS';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'AtHome';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Atlocal';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Atomic_Email_Hunter';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Atomz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Atrop';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^attach';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'attrib';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'autoemailspider';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'autohttp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'axod';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'batch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'b2w';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Back';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BackDoorBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BackStreet';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BackWeb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Badass';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Baid';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bali';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bandit';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Baidu';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Barry';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BasicHTTP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BatchFTP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'bdfetch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'beat';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Become';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Beij';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BenchMark';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'berts';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'bew';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'big.brother';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bigfoot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bilgi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bison';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bitacle';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Biz360';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Black';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Black.Hole';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BlackWidow';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'bladder.fusion';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Blaiz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Blog.Checker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Blogl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BlogPeople';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Blogshares.Spiders';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bloodhound';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Blow';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'bmclient';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Board';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BOI';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'boitho';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bond';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bookmark.search.tool';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'boris';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bost';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Boston.Project';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BotRightHere';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bot.mailto:craftbot@yahoo.com';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BotALot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'botpaidtoclick';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'botw';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'brandwatch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BravoBrian';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Brok';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bropwers';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Broth';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'browseabit';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'BrowseX';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Browsezilla';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bruin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'bsalsa';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Buddy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Build';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Built';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bulls';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'bumblebee';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Bunny';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Busca';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Busi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Buy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'bwh3';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'c\-spider';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'CafeK';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Cafi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'camel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Cand';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'captu';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Catch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'cd34';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ceg';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'CFNetwork';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'cgichk';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Cha0s';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Chang';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'chaos';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Char';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'char\(32\,35\)';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'charlotte';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'CheeseBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Chek';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'CherryPicker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'chill';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ChinaClaw';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'CICC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Cisco';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Cita';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Clam';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Claw';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Click.Bot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'clipping';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'clshttp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Clush';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'COAST';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ColdFusion';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Coll';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Comb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'commentreader';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Compan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'contact';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Control';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'contype';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Conc';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Conv';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Copernic';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Copi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Copy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Coral';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Corn';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'core-project';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'cosmos';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'costa';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'cr4nk';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'crank';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'craft';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Crap';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Crawler0';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Crazy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Cres';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'cs\-CZ';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'cuill';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Curl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Custo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Cute';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'CSHttp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Cyber';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'cyberalert';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^DA$';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'daoBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DARK';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Data';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Daten';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Daum';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'dcbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'dcs';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Deep';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DepS';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Detect';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Deweb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Diam';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Digger';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Digimarc';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'digout4uagent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DIIbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Dillo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ding';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DISC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'discobot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Disp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ditto';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DLC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DnloadMage';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DotBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Doubanbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Download';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Download.Demon';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Download.Devil';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Download.Wonder';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Downloader';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'drag';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DreamPassport';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Drec';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Drip';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'dsdl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'dsok';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DSurf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DTAAgent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DTS';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Dual';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'dumb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'DynaWeb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'e\-collector';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'eag';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'earn';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EARTHCOM';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EasyDL';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ebin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EBM-APPLE';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EBrowse';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'eCatch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'echo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ecollector';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Edco';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'edgeio';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'efp\@gmx\.net';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EirGrabber';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'email';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Email.Extractor';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EmailCollector';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EmailSearch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EmailSiphon';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EmailWolf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Emer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'empas';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Enfi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Enhan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Enterprise\_Search';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'envolk';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'erck';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EroCr';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ESurf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Eval';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Evil';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Evere';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EWH';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Exabot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Exact';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EXPLOITER';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Expre';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Extra';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ExtractorPro';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'EyeN';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FairAd';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Fake';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FANG';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FAST';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'fastlwspider';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FavOrg';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Favorites.Sweeper';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Faxo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FDM\_1';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FDSE';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'fetch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FEZhead';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Filan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FileHound';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'find';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Firebat';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Firefox.2\.0';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Firs';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Flam';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Flash';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FlickBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Flip';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'fluffy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'flunky';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'focus';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Foob';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Fooky';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Forex';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Forum';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ForV';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Fost';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Foto';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Foun';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Franklin.Locator';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'freefind';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FreshDownload';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FrontPage';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'FSurf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Fuck';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Fuer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'futile';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Fyber';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Gais';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'GalaxyBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Galbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Gamespy\_Arcade';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'GbPl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Gener';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'geni';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Geona';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Get';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'gigabaz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Gira';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ginxbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'gluc';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'glx.?v';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'gnome';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Go.Zilla';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Goldfire';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Google.Wireless.Transcoder';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Googlebot\-Image';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Got\-It';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'GOFORIT';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'gonzo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'GornKer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'GoSearch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^gotit$';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'gozilla';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'grab';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Grabber';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'GrabNet';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Grub';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Grup';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Graf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Green.Research';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'grub';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'grub\-client';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'gsa\-cra';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'GSearch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'GT\:\:WWW';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'GuideBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'guruji';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'gvfs';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Gyps';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'hack';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'haha';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'hailo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Harv';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Hatena';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Hax';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Head';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Helm';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'herit';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'hgre';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'hhjhj\@yahoo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Hippo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'hloader';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HMView';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'holm';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'holy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HomePageSearch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HooWWWer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HouxouCrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HMSE';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HPPrint';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'htdig';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HTTPConnect';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'httpdown';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'http.generic';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HTTPGet';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'httplib';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HTTPRetriever';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'HTTrack';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'human';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Huron';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'hverify';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Hybrid';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Hyper';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ia\_archiver';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'iaskspi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'IBM\_Planetwide';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'iCCra';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ichiro';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ID\-Search';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'IDA';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'IDBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'IEAuto';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'IEMPT';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'iexplore\.exe';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'iGetter';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ilse';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Iltrov';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Image';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Image.Stripper';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Image.Sucker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'imagefetch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'iimds\_monitor';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Incutio';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'IncyWincy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Indexer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Industry.Program';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Indy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'InetURL';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'informant';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'InfoNav';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'InfoTekies';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ingelin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Innerpr';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Inspect';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'InstallShield.DigitalWizard';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Insuran\.';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Intellig';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Intelliseek';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'InterGET';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Internet.Ninja';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Internet.x';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Internet\_Explorer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'InternetLinkagent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'InternetSeer.com';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Intraf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'IP2';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ipsel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Iria';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'IRLbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Iron33';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Irvine';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ISC\_Sys';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'iSilo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ISRCCrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ISSpi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'IUPUI.Research.Bot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Jady';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Jaka';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Jam';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^Java';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'java\/';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Java\(tm\)';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'JBH.agent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Jenny';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'JetB';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'JetC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'jeteye';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'jiro';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'JoBo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'JOC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'jupit';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Just';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Jyx';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Kapere';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'kash';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Kazo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'KBee';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Kenjin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Kernel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Keywo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'KFSW';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'KKma';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Know';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'kosmix';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'KRAE';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'KRetrieve';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Krug';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ksibot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ksoap';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Kum';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'KWebGet';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Lachesis';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'lanshan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Lapo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'larbin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'leacher';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'leech';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LeechFTP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LeechGet';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'leipzig\.de';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Lets';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Lexi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'lftp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Libby';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'libcrawl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'libcurl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'libfetch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'libghttp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'libWeb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'libwhisker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'libwww';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'libwww\-FM';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'libwww\-perl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LightningDownload';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'likse';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Linc';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Link';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Link.Sleuth';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LinkextractorPro';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Linkie';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LINKS.ARoMATIZED';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LinkScan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'linktiger';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LinkWalker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Lint';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'List';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'lmcrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LMQ';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LNSpiderguy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'loader';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LocalcomBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Locu';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'London';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'lone';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'looksmart';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'loop';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Lork';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LTH\_';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'lwp\-request';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'LWP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'lwp-request';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'lwp-trivial';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mac.Finder';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Macintosh\;.I\;.PPC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mac\_F';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'magi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mag\-Net';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Magnet';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Magp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mail.Sweeper';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'main';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'majest';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mam';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mana';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MarcoPolo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'mark.blonin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MarkWatch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MaSagool';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mass';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mass.Downloader';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mata';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'mavi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'McBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mecha';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MCspider';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'mediapartners';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^Memo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MEGAUPLOAD';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MetaProducts.Download.Express';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Metaspin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mete';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Microsoft.Data.Access';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Microsoft.URL';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Microsoft\_Internet\_Explorer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MIDo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MIIx';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'miner';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mira';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MIRE';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mirror';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Miss';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Missauga';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Missigua.Locator';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Missouri.College.Browse';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mist';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mizz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MJ12';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'mkdb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'mlbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MLM';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MMMoCrawl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MnoG';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'moge';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Moje';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Monster';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Monza.Browser';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mooz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Moreoverbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MOT\-MPx220';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'mothra\/netscan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'mouse';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MovableType';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozdex';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozi\!';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^Mozilla$';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/1\.22';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/22';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^Mozilla\/3\.0.\(compatible';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/3\.Mozilla\/2\.01';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/4\.0\(compatible';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/4\.08';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/4\.61.\(Macintosh';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/5\.0';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/7\.0';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/8';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/9';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\:';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mozilla\/Firefox';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^Mozilla.*Indy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^Mozilla.*NEWT';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^Mozilla*MSIECrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Mp3Bot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MPF';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MRA';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MS.FrontPage';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MS.?Search';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MSFrontPage';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MSIE\_6\.0';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MSIE6';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MSIECrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'msnbot\-media';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'msnbot\-Products';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MSNPTC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MSProxy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MSRBOT';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'multithreaddb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'musc';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MVAC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MWM';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'My\_age';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MyApp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MyDog';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MyEng';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MyFamilyBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MyGetRight';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'MyIE2';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'mysearch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'myurl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NAG';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NAMEPROTECT';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NASA.Search';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'nationaldirectory';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Naver';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Navr';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Near';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NetAnts';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'netattache';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Netcach';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NetCarta';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Netcraft';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NetCrawl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NetMech';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'netprospector';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NetResearchServer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NetSp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Net.Vampire';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'netX';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NetZ';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Neut';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'newLISP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NewsGatorInbox';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NEWT';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NEWT.ActiveX';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Next';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^NG';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NICE';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'nikto';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Nimb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ninja';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ninte';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NIPGCrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Noga';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'nogo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Noko';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Nomad';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Norb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'noxtrumbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NPbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NuSe';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Nutch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Nutex';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'NWSp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Obje';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ocel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Octo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ODI3';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'oegp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Offline';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Offline.Explorer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Offline.Navigator';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'OK.Mozilla';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'omg';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Omni';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Onfo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'onyx';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'OpaL';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'OpenBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Openf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'OpenTextSiteCrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'OpenU';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Orac';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'OrangeBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Orbit';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Oreg';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'osis';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Outf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Owl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'P3P';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PackRat';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PageGrabber';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PagmIEDownload';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'pansci';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Papa';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pars';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Patw';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'pavu';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pb2Pb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'pcBrow';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PEAR';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PEER';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PECL';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'pepe';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Perl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PerMan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PersonaPilot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Persuader';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'petit';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PHP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PHP.vers';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PHPot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Phras';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PicaLo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Piff';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pige';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'pigs';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^Ping';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pingd';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PingALink';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pipe';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Plag';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Plant';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'playstarmusic';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pluck';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pockey';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'POE\-Com';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Poirot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pomp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Port.Huron';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Post';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'powerset';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Preload';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'press';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Privoxy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Probe';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Program.Shareware';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Progressive.Download';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ProPowerBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'prospector';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Provider.Protocol.Discover';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ProWebWalker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Prowl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Proxy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Prozilla';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'psbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PSurf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'psycheclone';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^puf$';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pulse';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pump';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PushSite';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PussyCat';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PuxaRapido';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PycURL';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Pyth';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'PyQ';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'QuepasaCreep';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Query';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Quest';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'QRVA';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Qweer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'radian';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Radiation';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Rambler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'RAMP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'RealDownload';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Reap';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Recorder';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'RedCarpet';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'RedKernel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ReGet';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'relevantnoise';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'replacer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Repo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'requ';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Rese';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Retrieve';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Rip';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Rix';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'RMA';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Roboz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Rogue';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Rover';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'RPT\-HTTP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Rsync';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'RTG30';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '.ru\)';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ruby';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Rufus';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Salt';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sample';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SAPO';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sauger';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'savvy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SBIder';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SBP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SCAgent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'scan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SCEJ\_';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sched';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Schizo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Schlong';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Schmo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Scout';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Scooter';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Scorp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ScoutOut';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SCrawl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'screen';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'script';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SearchExpress';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'searchhippo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Searchme';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'searchpreview';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'searchterms';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Second.Street.Research';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Security.Kol';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Seekbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Seeker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sega';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sensis';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sept';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Serious';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sezn';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Shai';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Share';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sharp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Shaz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'shell';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'shelo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sherl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Shim';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Shiretoko';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ShopWiki';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SickleBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Simple';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Siph';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'sitecheck';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SiteCrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SiteSnagger';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Site.Sniper';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SiteSucker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'sitevigil';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SiteX';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sleip';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Slide';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Slurpy.Verifier';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sly';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Smag';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SmartDownload';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Smurf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'sna\-';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'snag';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Snake';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Snapbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Snip';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Snoop';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'So\-net';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SocSci';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'sogou';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sohu';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'solr';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'sootle';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Soso';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SpaceBison';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Spad';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Span';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'spanner';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Speed';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Spegla';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sphere';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sphider';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'spider';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SpiderBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SpiderEngine';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SpiderView';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Spin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'sproose';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Spurl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Spyder';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Squi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SQ.Webscanner';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'sqwid';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sqworm';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SSM\_Ag';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Stack';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Stamina';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'stamp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Stanford';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Statbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'State';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Steel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Strateg';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Stress';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Strip';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'studybot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Style';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'subot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Suck';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sume';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'sun4m';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sunrise';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SuperBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SuperBro';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Supervi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Surf4Me';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SuperHTTP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Surfbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'SurfWalker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Susi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'suza';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'suzu';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Sweep';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'sygol';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'syncrisis';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Systems';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Szukacz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tagger';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tagyu';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'tAke';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Talkro';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TALWinHttpClient';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'tamu';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tandem';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tarantula';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'tarspider';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'tBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TCF';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tcs\/1';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TeamSoft';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tecomi';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Teleport';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Telesoft';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Templeton';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tencent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Terrawiz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Test';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TexNut';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'trivial';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Turnitin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'The.Intraformant';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TheNomad';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Thomas';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TightTwatBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Timely';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Titan';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TMCrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TMhtload';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'toCrawl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Todobr';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tongco';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'topic';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Torrent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Track';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'translate';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Traveler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TREEVIEW';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'True';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tunnel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'turing';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Turnitin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TutorGig';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'TV33\_Mercator';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Twat';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tweak';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Twice';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Twisted.PageGetter';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Tygo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ubee';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'UCmore';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'UdmSearch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'UIowaCrawler';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Ultraseek';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'UMBC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'unf';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'UniversalFeedParser';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'unknown';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'UPG1';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'UtilMind';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'URLBase';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'URL.Control';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'URL\_Spider\_Pro';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'urldispatcher';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'URLGetFile';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'urllib';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'URLSpiderPro';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'URLy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'User\-Agent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'UserAgent';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'USyd';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Vacuum';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'vagabo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Valet';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Valid';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Vamp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'vayala';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'VB\_';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'VCI';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'VERI\~LI';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'verif';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'versus';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'via';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Viewer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'virtual';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'visibilitygap';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Visual';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'vobsub';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Void';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'VoilaBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'voyager';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'vspider';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'VSyn';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'w\:PACBHO60';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'w0000t';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'W3C';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'w3m';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'w3search';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'walhello';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Walker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Wand';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WAOL';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WAPT';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Watch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Wavefire';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'wbdbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Weather';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'web.by.mail';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Web.Data.Extractor';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Web.Downloader';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Web.Ima';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Web.Mole';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Web.Sucker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Web2Mal';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Web2WAP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebaltBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebAuto';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebBandit';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Webbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebCapture';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebCat';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webcraft\@bea';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Webclip';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webcollage';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebCollector';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebCopier';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebCopy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebCor';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webcrawl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebDat';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebDav';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webdevil';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webdownloader';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Webdup';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebEMail';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebEMailExtrac';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebEnhancer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebFetch';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebGo';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebHook';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Webinator';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebInd';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webitpr';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebFilter';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebFountain';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebLea';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Webmaster';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebmasterWorldForumBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebMin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebMirror';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webmole';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webpic';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebPin';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebPix';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebReaper';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebRipper';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebRobot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebSauger';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebSite';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Website.eXtractor';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Website.Quester';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebSnake';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webspider';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Webster';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebStripper';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'websucker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebTre';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebVac';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'webwalk';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebWasher';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebWeasel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebWhacker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WebZIP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Wells';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WEP\_S';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WEP.Search.00';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WeRelateBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'wget';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Whack';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Whacker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'whiz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WhosTalking';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Widow';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Win67';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'window.location';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Windows.95\;';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Windows.95\)';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Windows.98\;';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Windows.98\)';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Winodws';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Wildsoft.Surfer';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WinHT';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'winhttp';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WinHttpRequest';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WinHTTrack';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Winnie.Poh';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'wire';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WISEbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'wisenutbot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'wish';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Wizz';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WordP';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Works';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'world';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WUMPUS';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Wweb';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WWWC';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WWWOFFLE';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WWW\-Collector';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'WWW.Mechanize';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'www.ranks.nl';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'wwwster';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = '^x$';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'X12R1';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'x\-Tractor';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Xaldon';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Xenu';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'XGET';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'xirq';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Y\!OASIS';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Y\!Tunnel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'yacy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'YaDirectBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Yahoo\-MMAudVid';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'YahooSeeker';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'YahooYSMcm';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Yamm';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Yand';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'yang';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Yeti';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Yoono';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'yori';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Yotta';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'YTunnel';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Zade';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'zagre';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ZBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Zeal';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ZeBot';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'zerx';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Zeus';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'ZIPCode';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Zixy';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'zmao';
        $this->db->insert($this->t('useragent'), $data);
        $data['content'] = 'Zyborg';
        $this->db->insert($this->t('useragent'), $data);
        $this->db->insert($this->t('useragent'), $data);

    }
    
}
